// This class handles all device access
#ifndef __DEVICE
#define __DEVICE
#include "bwhelper.h"
struct SGlobalInfo
{
	int StructSize;
	DWORD RevisionID;
	long GlobalOpenCount;
	int CodecOpenCount;
	_COMPRESSINFO CodecState;
};

struct EEPROM {
	SHORT HardwareRev;
	SHORT FlashRev;
	DWORD SerialNumber;
	DWORD SnKey;
	BYTE  Res1[20];     // used to align key on 32 byte boundry
	DWORD Key[2];
	DWORD caps;
	DWORD WriteCount;
	SHORT HardwareType; //0 broadway 1 beginner 
	BYTE  Reserved[14];  // total size must be less then 2048 bytes AND MULTIPLE OF 32;
};


#define EEPROM_OFFSET(X) ( (DWORD)(&((EEPROM*)0)->X) )

inline void Write32(volatile DWORD *addr,DWORD data)
{	*addr=data;}

inline DWORD Read32(volatile DWORD *addr) 
{ return *addr;}

class CDevice :public DEVSTRUCT

{
public:
	struct _CAPTUREINFO CaptureInfo;
	struct _COMPRESSINFO CodecInfo;
	struct _RECODEINFO RecodeInfo;
	PDEVICE_INFO pDeviceInfo;
	DWORD   FB_PhysAddr;
	DWORD   *FB_LinAddr;
	DWORD   FB_Size; // in dwords
	short	ShortBurstsOnly;
	short 	Busy;			  // 1 if operation is in progress
	short   State;			  // Running stopped paused ...	
//	short 	PassthroughRunning;
	short 	CurrentMode;
	short 	CurrentVideoMode; // ntsc /pal
	short 	CurrentSource;
	int		DriverNumber;  // number of the driver opened
	HANDLE  hDriver;
private:
	volatile BOARD_STATE *pGlobalState; // Use Mutex to access
public:
	volatile SGlobalInfo *pGlobalInfo;
	class CMicrocode* MicroCode;
	TCHAR  ErrorMessage[128];

public:
#ifdef BUILD_DLL
	UCHAR GetRevisionID()  {return (char)pGlobalInfo->RevisionID; }
	void  SetRevisionID(UCHAR newid)  { pGlobalInfo->RevisionID =newid; }
#else
	UCHAR m_RevisionID;
	UCHAR GetRevisionID()  {return  pGlobalInfo->RevisionID==3 ? 3 : m_RevisionID; }
	void  SetRevisionID(UCHAR newid)  { m_RevisionID=newid; }
#endif
	DWORD GetBoardCaps() 
	{
		
		switch (GetRevisionID())
		{
		 case 1:return BROADWAY_DEFAULT_CAPS; 
		 case 2:return BROADWAY_DEFAULT_CAPS | BROADWAY_CANCAPTURE_MPEG;
		
		 case 3:return BROADWAY_DEFAULT_CAPS | BROADWAY_CANCAPTURE_MPEG | BROADWAY_CANOUTPUT_IMPEG;
		default:
				DWORD Caps;
				ReadFlash(&Caps,EEPROM_OFFSET(caps),4);
				return Caps;
		}
	}
	void SetGlobalState(BOARD_STATE newstate) {*pGlobalState=(BOARD_STATE)(newstate | 0x10000);}
	BOARD_STATE GetGlobalState() {return (BOARD_STATE)(*pGlobalState & 0xffff);}
	void FatalError(ECODE ecode,TCHAR *String);
	void ShowMessage(DWORD strid);

	static ECODE CheckDevice(CDevice& Dev,BOOL CheckMicroCode=TRUE);
	ECODE SetVipFilterMode(BOOL LoadValues=TRUE);
	CDevice(HANDLE hDev=NULL,PDEVICE_INFO pDI=NULL);
	~CDevice();
//	operator volatile VT_REGS*() {return Regs;}
	Read(VT_MEMTYPE Type, DWORD *DestPtr, DWORD Source, DWORD Words);
	Write(VT_MEMTYPE Type, DWORD Dest, const DWORD *SrcPtr, DWORD Words);
//private:
	void Reset();  // this is a lightweight reset function VT_Reset should still be called
	ECODE ReadFlash(DWORD *DestPtr, DWORD Source, DWORD Words);
	ECODE ReadVIP(DWORD *DestPtr, DWORD Source, DWORD Words);
	ECODE WriteFlash(DWORD Dest, const DWORD *SrcPtr, DWORD Words);
	ECODE WriteVIP(DWORD Dest, const DWORD *SrcPtr, DWORD Words);
	ECODE LoadFileBinSwap(VT_MEMTYPE Type,char *filename,DWORD Addr,DWORD LastAddr,DWORD FileOffset);
	static ECODE AcquireMutex();
	static ECODE ReleaseMutex();
	static HANDLE Mutex;
	static BOOL	CreateMutex() { return (Mutex=::CreateMutex(NULL,FALSE,"BroadwayAccess"))!=NULL;};
	void VrpPokeNoMutex(DWORD addr, DWORD data);
	BOOL CheckValidBoard();
	DWORD VrpPeekNoMutex(DWORD addr);
	void VrpChange(DWORD addr,DWORD mask, DWORD data);
	void  CrPoke(DWORD addr, DWORD data);
	DWORD CrPeek(DWORD addr);
	void CrChange(DWORD addr,DWORD mask, DWORD data);
	ECODE VidOutCtlPoke(DWORD data);

#ifdef _DEBUG
    void CheckMasterErr()
	{
		if (Regs->Csr & 0x10)
		{
			OutputDebugString("Master Abort\n");
			__asm int 3
		}
	}
#else
	 void CheckMasterErr() {};
#endif
};

class CAutoMutex
{
	int status;
public:
	CAutoMutex() {status=CDevice::AcquireMutex(); /*WaitForSingleObject(Mutex,INFINITE) == WAIT_OBJECT_0 ?VT_NO_ERROR:VT_ERR_OTHER */}
	~CAutoMutex() {CDevice::ReleaseMutex();}
};

#endif