#include <windows.h>
#include <vfw.h>
#include "xlate.h"
#include "mpeg2dec\ifrmdec.h"
#include "fconv.h"

int FrameConverter::ClipInited = 0;
unsigned char FrameConverter::ClipTable[1024];	//table to clip to 8 bits
unsigned char *FrameConverter::clip;				//0 point of table;
short FrameConverter::CrToR[256];		//look up Cr to red value (YUV to RGB conversion)
short FrameConverter::CrToG[256];		//look up Cr to green value (YUV to RGB conversion)
short FrameConverter::CbToB[256];		//look up Cb to red blue (YUV to RGB conversion)
short FrameConverter::CbToG[256];		//look up Cb to red green (YUV to RGB conversion)

int g_nBlankLines = 0;

extern "C" void __stdcall SetConverterBlankLines (int nLines)
{
	g_nBlankLines = nLines;
}

int FrameConverter::Init(int shres,int svres,int dhres,int dvres)
{

	if (dvres<0)
	{
		invert=TRUE;
		vdisplay=-dvres;
	}
	else
	{
		vdisplay=dvres;
	}
	xres=shres;
	yres=svres;
	hdisplay=dhres;

	if (shres!=dhres ||svres!=dvres)
		mMustStretch=TRUE;
	dyDst=vdisplay;
	dxDst=dhres;

	
	return 0;
}

int FrameConverter::InitEx(ICDECOMPRESSEX* pIc)
{
	vdisplay=pIc->lpbiDst->biHeight;
	hdisplay=pIc->lpbiDst->biWidth;
	if (vdisplay<0)
	{
		invert=FALSE;
		vdisplay=-vdisplay;
	}
	else
	{
		invert=TRUE;
	}
	vstart=pIc->yDst;
	hstart=pIc->xDst;
	yres=pIc->dySrc;
	xres=pIc->dxSrc;
	// input clipping still not supported Output windowing is supported

	if (pIc->dyDst!=pIc->dySrc || pIc->dxDst!=pIc->dxSrc)
		mMustStretch=TRUE;
	dyDst=pIc->dyDst;
	dxDst=pIc->dxDst;

	if (mCanBlank) 
	{
		InitBlankLine();
	}
	return 0;
}

DWORD FrameConverter::GetOutputFrameSize()
{
	return (OutputBytesPerPixel*xres*yres);
}
DWORD FrameConverter::GetInputFrameSize()
{
	return (InputBytesPerPixel*xres*yres);
}

DWORD FrameConverter::DecodeFrame(	DECODE_FRAME *SwDecoder,UCHAR *Output,UCHAR *Input,DWORD InputSizeBytes)
{
	DWORD ec;
	if (SoftwareDecodePostprocNeeded)
	{
		if (TempBuffer==NULL)
			TempBuffer=new UCHAR[352*288*3];  // this buffer will hold 24 bit rgb data
		ec=::DecodeFrame(SwDecoder,Input,InputSizeBytes,
					(UCHAR*)TempBuffer, SwDecodeOutputType);
		if (!ec) 
			return ICERR_DONTDRAW;
		ProcessFrame(Output,TempBuffer);
	}
	else
	{
		ec=::DecodeFrame(SwDecoder,Input,InputSizeBytes,
					(UCHAR*)TempBuffer, SwDecodeOutputType);
		if (!ec) 
			return ICERR_DONTDRAW;
	}
	return 0;
}

void FrameConverter::ProcessLineFirstStretch(UCHAR *pOutput,const UCHAR *pInput,UCHAR *buffer)
{
	ProcessLine(buffer,pInput);
	StretchLineUYVY(pOutput,buffer);
}

void FrameConverter::StretchFirstProcessLine(UCHAR *pOutput,const UCHAR *pInput,UCHAR *buffer)
{
	StretchLineUYVY(buffer,pInput);
	ProcessLine(pOutput,buffer);
}

void FrameConverter::ProcessFrame(UCHAR *Output,const UCHAR *Input)
{
	int row;
	int isize=InputBytesPerPixel*xres;
	if (isize& 3)
		isize=(isize& ~3) +4;
	int osize=OutputBytesPerPixel*hdisplay;
	if (!invert)
	{
		Output+=osize*(vdisplay-vstart-dyDst)+OutputBytesPerPixel*hstart;
	}
	else
	{
		Output+=osize*(vdisplay-1)-osize*vstart+OutputBytesPerPixel*hstart;
		osize=-osize;
	}
	if (mMustStretch)
	{
		//int docols=hdisplay/PixelsPerProcess;
		//int docols=hdisplay/PixelsPerProcess;
		UCHAR *linebuffer=new UCHAR[max(xres,dxDst)*2];
		long dx,dy,e,d,dx2;
		dx=dyDst-1;
		dy=yres-1;
		e=(dy<<1)-dx;
		dx2=dx<<1;
		dy<<=1;
		for(d=0;d<=dx;d++) 
		{
			StretchProcessLine(Output,Input,linebuffer);
			Output+=osize;
			while (e<0 && d<dx)  // off by one count so no =
			{
				e+=dy;
				memcpy(Output,Output-osize,abs(osize));
				Output+=osize;
				d++;
			}
			while(e>=0)	
			{
				Input+=isize;
				e-=dx2;
			}
			e+=dy;
		}
		delete [] linebuffer;
	}
	else
	{
		//int docols=hdisplay/PixelsPerProcess;
		for (row=0;row<yres;row++)
		{		
			if (mCanBlank && g_nBlankLines >= yres-row)
				ProcessLine(Output,mBlankLine);
			else
				ProcessLine(Output,Input);
				
				Output+=osize;
				Input+=isize;
		}
	}

}

FrameConverter::~FrameConverter()
{
	if (TempBuffer) delete [] TempBuffer;
	if (mBlankLine) delete [] mBlankLine;
}

FrameConverter::FrameConverter()
{
	Compression=0;
	OutputBytesPerPixel=2;
	InputBytesPerPixel=2;
	PixelsPerProcess=2;
	vstart=0;
	hstart=0;
	invert=TRUE;
	TempBuffer=NULL;
	SoftwareDecodePostprocNeeded=TRUE;
	SwDecodeOutputType=UYVY;
	mMustStretch=FALSE;
	mCanBlank=FALSE;
	mBlankLine = NULL;

	//init static clip table if needed
	if (!ClipInited) {
		clip = ClipTable + 384;
		for (int i=-384; i<640; i++) 
			clip[i] = (i<0) ? 0 : ((i>255) ? 255 : i);
		for (int val=0; val<256; val++) {
			CrToR[val] = (short)( 1.371*(val-128));
			CrToG[val] = (short)(-0.698*(val-128));
			CbToB[val] = (short)( 1.732*(val-128));
			CbToG[val] = (short)(-0.336*(val-128));
		}
		ClipInited = 1;
	}
}

FromRGBToYuv::FromRGBToYuv()
{
	OutputBytesPerPixel=2;
	InputBytesPerPixel=3;
	InputBitsPerPixel=24;
	OutputBitsPerPixel=16;
	PixelsPerProcess=2;
	invert=TRUE;
}

void FrameConverter::ProcessPixel(UCHAR *Output,const UCHAR *Input)
{
	
}

// rgb2playyuv422   Entry - pointer to pic data (3 bytes/pix = 6 bytes)
//                  Exit - 2 words of Aura playback YUV422


FromRGB32ToYuv::FromRGB32ToYuv()
{
	OutputBytesPerPixel=2;
	InputBytesPerPixel=3;
	PixelsPerProcess=2;
	InputBitsPerPixel=24;
	OutputBitsPerPixel=16;
	invert=TRUE;
}


FromRGB16ToYuv::FromRGB16ToYuv()
{
	OutputBytesPerPixel=2;
	InputBytesPerPixel=2;
	PixelsPerProcess=2;
	InputBitsPerPixel=16;
	OutputBitsPerPixel=16;
	invert=TRUE;
}

void FromRGB16ToYuv::ProcessPixel(UCHAR *Output,const UCHAR *Input)
{
    struct {UCHAR b,g,r;} p[2];
	unsigned short *in=(unsigned short *)Input; 
	for (int i=0;i<PixelsPerProcess;i++)
	{
		p[i].r=(*in &0x7c00) >> 7;
		p[i].g=(*in & 0x3e0) >> 2;
		p[i].b=(*in++ & 0x1f) << 3;
	}
	rgb2yuv422((const unsigned char*)&p, Output);
}




/*
void FromRGB32ToYuv::ProcessPixel(UCHAR *Output,const UCHAR *Input)
{
	rgb2yuv422((const unsigned char*)Input, (unsigned short*)Output);
}
*/
#if 0
void __inline FromRGBToYuv::rgb2yuv422(const unsigned char *lppic ,  unsigned short *w )
{
    unsigned int r, g, b;
    short y0, y1;
	double yt;
    double u;
    double v;

    b = *lppic++;
    g = *lppic++;
    r = *lppic++;
/*
    y0 = ( r * 30 + g * 59 + b * 11 + 50 ) / 100;

    u += ( (((int)b - y0) * 127) / 226 );
    v += ( (((int)r - y0) * 127) / 179 );
*/
#ifdef FULL_RANGE_VUV
	yt= (r * (77/256.0) + g* (150 /256.0) + b*(29/256.0)) ;
	u= -.1719*r  -0.3398*g +0.1517*b  ; // + 128? 
	v= 0.5117*r  -0.4297*g -0.0820*b ;
#else
	yt= 0.257*r + 0.504*g + 0.098*b +16;
	u= -0.148*r  -0.291*g+ +0.439*b ;
	v= 0.439*r  -0.368*g -0.071*b ;
	// u=(int)((b-yt)*0.5774+128.5);
	// v=(r-yt)*0.7294+128.5;
#endif
	y0=(short) (yt+0.5);	
	b = *lppic++;
    g = *lppic++;
    r = *lppic++;
#ifdef FULL_RANGE_VUV
	yt= (r * (77/256.0) + g* (150 /256.0) + b*(29/256.0)) ;

	u+= -.1719*r  -0.3398*g +0.1517*b+256.5  ; // + 128? 
	v+= 0.5117*r  -0.4297*g -0.0820*b+256.5 ;
	// u=(int)((b-yt)*0.5774+128.5);
	// v=(r-yt)*0.7294+128.5;
#else
	yt= 0.257*r + 0.504*g + 0.098*b +16;
	u+= -0.148*r  -0.291*g+ +0.439*b+256.5 ;
	// u=(int)((r-yt)*0.5774+128.5);
	v+= 0.439*r  -0.368*g -0.071*b +256.5;
#endif
	y1=(short) (yt+0.5);	

     *w++ = (y0<<8) + (((short)u>>1)&0xff);
     *w   = (y1<<8) + (((short)v>>1)&0xff);
}
#else
void __inline FromRGBToYuv::rgb2yuv422(const unsigned char *lppic ,  unsigned char *w )
/*
this function converts 2 pixels to yuv 422 format: u,y0,v,y1 -->  Cb,y0,Cr,y1
*/
{
    int r, g, b,r1,g1,b1;
    int y0, y1;
	int yt;
    int u;
    int v;

	//read rgb values for both pixels
#if	1 //CODEC
    b = *lppic++;
    g = *lppic++;
    r = *lppic++;
	b1 = *lppic++;
    g1 = *lppic++;
    r1 = *lppic++;
#else  // test
    r = *lppic++;
    g = *lppic++;
    b = *lppic++;
	r1 = *lppic++;
    g1 = *lppic++;
    b1 = *lppic++;
#endif
	//compute lum values for each output pixel
	y0= (77*r  + 150*g  + 29*b ) >> 8;
	y1= (77*r1 + 150*g1 + 29*b1) >> 8;

	//average both pixels to compute the chrom value
	yt=y0+y1;
	u = clip[((591*(b+b1-yt)) >>11)  + 128];
//	v = clip[((747*(r+r1-yt)) >>11) + 128];
	v = clip[((750*(r+r1-yt)) >>11)  + 128];  // compensation from actual invert

	//output the 422 pixels
#if 1  // codec
	*w++ = (unsigned char)u;
	*w++ = (unsigned char)y0;
	*w++ = (unsigned char)v;
	*w   = (unsigned char)y1;
#else // test
	*w++ = (unsigned char)u; //128; //(unsigned char)v;
	*w++ = (unsigned char)y0;
	*w++ = (unsigned char)v;
	*w   = (unsigned char)y1;
#endif
}
/*
void __inline FromRGBToYuv::rgb2yuv422(const unsigned char *lppic ,  unsigned short *w )
{
    unsigned int r, g, b,r1,b1;
    int y0, y1;
	int yt;
    int u;
    int v;

    b = *lppic++;
    g = *lppic++;
    r = *lppic++;

	y0= (int)(0.257*r + 0.504*g + 0.098*b +16.5);

	b1 = *lppic++;
    g = *lppic++;
    r1 = *lppic++;

	y1= (int)(0.257*r1 + 0.504*g + 0.098*b1 +16.5);

	yt=y0+y1;

	u=(int)(((b+b1)-yt)*0.5774+256.5);
	v=(int)(((r+r1)-yt)*0.7294+256.5);


	 *w++ = (y0<<8) + (((short)u>>1)&0xff);
     *w   = (y1<<8) + (((short)v>>1)&0xff);
}
*/
#endif


void FromRGBToYuv::ProcessPixel(UCHAR *Output,const UCHAR *Input)
{

	rgb2yuv422((const unsigned char*)Input, Output);
}


FromYuvToRGB::FromYuvToRGB()
{
	OutputBytesPerPixel=3;
	InputBytesPerPixel=2;
	PixelsPerProcess=2;
	InputBitsPerPixel=16;
	OutputBitsPerPixel=24;
	invert=TRUE;
}

FromYuvToRGB32::FromYuvToRGB32()
{
	OutputBytesPerPixel=4;
	InputBytesPerPixel=2;
	PixelsPerProcess=2;
	InputBitsPerPixel=16;
	OutputBitsPerPixel=32;
	invert=TRUE;
}

FromYuvToYUY2::FromYuvToYUY2()
{
	OutputBytesPerPixel=2;
	InputBytesPerPixel=2;
	PixelsPerProcess=2;
	InputBitsPerPixel=16;
	OutputBitsPerPixel=16;
	invert=TRUE;
}

void FromYuvToYUY2::ProcessLine(UCHAR *pOutput,const UCHAR *pInput)
{
	DWORD *pOut=(DWORD*)pOutput,*pIn=(DWORD*)pInput;
	int docols=xres>>1; // two pixels per process
	for (int col=0;col<docols;col++)
	{
		*pOut++=((*pIn & 0x00ff00ff)<<8) | ((*pIn & 0xff00ff00)>>8);
		pIn++;
	}
}

FromYUY2ToYUY2::FromYUY2ToYUY2()
{
	OutputBytesPerPixel=2;
	InputBytesPerPixel=2;
	PixelsPerProcess=2;
	InputBitsPerPixel=16;
	OutputBitsPerPixel=16;
	invert=TRUE;
	lastxres = 0;
	mCanBlank=TRUE;
	mBlankLine = NULL;
}

void FromYUY2ToYUY2::InitBlankLine()
{
	if (mBlankLine != NULL)
	{
		if (lastxres == xres)
			return;
		else
			delete [] mBlankLine;
	}

	mBlankLine = new UCHAR[xres*sizeof(WORD)];
	WORD *p = (WORD *)mBlankLine;
	for (int i=0;i<xres;i++)
		*(p++) = 0x8000;
}

void FromYUY2ToYUY2::ProcessLine(UCHAR *pOutput,const UCHAR *pInput)
{
	CopyMemory(pOutput,pInput,xres<<1); //two bytes per pixel
}

FromYUY2ToUYVY::FromYUY2ToUYVY()
{
	OutputBytesPerPixel=2;
	InputBytesPerPixel=2;
	PixelsPerProcess=2;
	InputBitsPerPixel=16;
	OutputBitsPerPixel=16;
	invert=TRUE;
	lastxres = 0;
	mCanBlank = TRUE;
	mBlankLine = NULL;
}

void FromYUY2ToUYVY::InitBlankLine()
{
	if (mBlankLine != NULL)
	{
		if (lastxres == xres)
			return;
		else
			delete [] mBlankLine;
	}

	mBlankLine = new UCHAR[xres*sizeof(WORD)];
	WORD *p = (WORD *)mBlankLine;
	for (int i=0;i<xres;i++)
		*(p++) = 0x8000;
}

void FromYUY2ToUYVY::ProcessLine(UCHAR *pOutput,const UCHAR *pInput)
{
	DWORD *pOut=(DWORD*)pOutput,*pIn=(DWORD*)pInput;
	int docols=xres>>1; // two pixels per process
	for (int col=0;col<docols;col++)
	{
		*pOut++=((*pIn & 0x00ff00ff)<<8) | ((*pIn & 0xff00ff00)>>8);
		pIn++;
	}
}



FromYuvToUYVY::FromYuvToUYVY()
{
	OutputBytesPerPixel=2;
	InputBytesPerPixel=2;
	PixelsPerProcess=2;
	InputBitsPerPixel=16;
	OutputBitsPerPixel=16;
	invert=TRUE;
}

void FromYuvToUYVY::ProcessLine(UCHAR *pOutput,const UCHAR *pInput)
{
	CopyMemory(pOutput,pInput,xres<<1); //two bytes per pixel
}


inline UCHAR Clip(int value)
{
	return value> 255 ?  255 : (value < 0 ? 0 :value);
}




void FromYuvToRGB::ProcessLine(UCHAR *Output,const UCHAR *Input)
{
	int docols=xres>>1; // /PixelsPerProcess;
	int Cr,Cb,y1,y2;
	int tr,tg,tb;
//	int isize=PixelsPerProcess*InputBytesPerPixel;
//	int osize=PixelsPerProcess*OutputBytesPerPixel;
	for (int col=0;col<docols;col++)
	{

#if 1	 // use pre built tables
		Cb=*Input++;
		y1=*Input++;
		Cr=*Input++;
		y2=*Input++;
		tr=CrToR[Cr];
		tg=CrToG[Cr] + CbToG[Cb];
		tb=CbToB[Cb];
#else		// math is 20% slower on pentium
		Cb=*Input++ -128;
		y1=*Input++;
		Cr=*Input++ -128;
		y2=*Input++;
		tr=1398*Cr>>10;
		tg=(-717*Cr - 342*Cb) >> 10;
		tb=1773*Cb>>10;
#endif
	#if 0 // function seems ~5% faster
		*Output++=clip[y1+tb];// blue 1
		*Output++=clip[y1+tg];// green 1
		*Output++=clip[y1+tr];// red 1
		*Output++=clip[y2+tb];// blue 2
		*Output++=clip[y2+tg];// green 2
		*Output++=clip[y2+tr];// red 2
	#else
		*Output++=Clip(y1+tb);// blue 1
		*Output++=Clip(y1+tg);// green 1
		*Output++=Clip(y1+tr);// red 1
		*Output++=Clip(y2+tb);// blue 2
		*Output++=Clip(y2+tg);// green 2
		*Output++=Clip(y2+tr);// red 2
	#endif
	}
}

FromYUY2ToRGB24::FromYUY2ToRGB24()
{
	lastxres = 0;
	mCanBlank = TRUE;
	mBlankLine = NULL;
}

void FromYUY2ToRGB24::InitBlankLine()
{
	if (mBlankLine != NULL)
	{
		if (lastxres == xres)
			return;
		else
			delete [] mBlankLine;
	}

	mBlankLine = new UCHAR[xres*sizeof(WORD)];
	WORD *p = (WORD *)mBlankLine;
	for (int i=0;i<xres;i++)
		*(p++) = 0x8000;
}

void FromYUY2ToRGB24::ProcessLine(UCHAR *Output,const UCHAR *Input)
{
	int docols=xres>>1; // /PixelsPerProcess;
	int Cr,Cb,y1,y2;
	int tr,tg,tb;
	for (int col=0;col<docols;col++)
	{

	 // use pre built tables
		y1=*Input++;
		Cb=*Input++;
		y2=*Input++;
		Cr=*Input++;
		tr=CrToR[Cr];
		tg=CrToG[Cr] + CbToG[Cb];
		tb=CbToB[Cb];
		*Output++=Clip(y1+tb);// blue 1
		*Output++=Clip(y1+tg);// green 1
		*Output++=Clip(y1+tr);// red 1
		*Output++=Clip(y2+tb);// blue 2
		*Output++=Clip(y2+tg);// green 2
		*Output++=Clip(y2+tr);// red 2
	}
}

	
FromYUY2ToRGB32::FromYUY2ToRGB32()
{
	lastxres = 0;
	mCanBlank = TRUE;
	mBlankLine = NULL;
}

void FromYUY2ToRGB32::InitBlankLine()
{
	if (mBlankLine != NULL)
	{
		if (lastxres == xres)
			return;
		else
			delete [] mBlankLine;
	}

	mBlankLine = new UCHAR[xres*sizeof(WORD)];
	WORD *p = (WORD *)mBlankLine;
	for (int i=0;i<xres;i++)
		*(p++) = 0x8000;
}

void FromYUY2ToRGB32::ProcessLine(UCHAR *Output,const UCHAR *Input)
{
	DWORD* Op=(DWORD*)Output;
	int docols=xres>>1; // /PixelsPerProcess;
	int Cr,Cb,y1,y2;
	int tr,tg,tb;
	for (int col=0;col<docols;col++)
	{

	 // use pre built tables
		y1=*Input++;
		Cb=*Input++;
		y2=*Input++;
		Cr=*Input++;
		tr=CrToR[Cr];
		tg=CrToG[Cr] + CbToG[Cb];
		tb=CbToB[Cb];
		*Op++=Clip(y1+tb) | (Clip(y1+tg) <<8) | (Clip(y1+tr)<<16);// red 1
		*Op++=Clip(y2+tb) | (Clip(y2+tg) <<8) | (Clip(y2+tr)<<16);// red 1
	}
}


  

void FromYuvToRGB32::ProcessLine(UCHAR *Output,const UCHAR *Input)
{
	int docols=xres>>1; // /PixelsPerProcess;
	int Cr,Cb,y1,y2;
	int tr,tg,tb;
	DWORD* Op=(DWORD*)Output;
//	int isize=PixelsPerProcess*InputBytesPerPixel;
//	int osize=PixelsPerProcess*OutputBytesPerPixel;
	for (int col=0;col<docols;col++)
	{

#if 1	 // use pre built tables
		Cb=*Input++;
		y1=*Input++;
		Cr=*Input++;
		y2=*Input++;
		tr=CrToR[Cr];
		tg=CrToG[Cr] + CbToG[Cb];
		tb=CbToB[Cb];
#else		// math is 20% slower on pentium
		Cb=*Input++ -128;
		y1=*Input++;
		Cr=*Input++ -128;
		y2=*Input++;
		tr=1398*Cr>>10;
		tg=(-717*Cr - 342*Cb) >> 10;
		tb=1773*Cb>>10;
#endif
	#if 0 // function seems ~5% faster
		*Output++=clip[y1+tb];// blue 1
		*Output++=clip[y1+tg];// green 1
		*Output++=clip[y1+tr];// red 1
		*Output++=clip[y2+tb];// blue 2
		*Output++=clip[y2+tg];// green 2
		*Output++=clip[y2+tr];// red 2
	#else
		*Op++=Clip(y1+tb) | (Clip(y1+tg) <<8) | (Clip(y1+tr)<<16);// red 1
		*Op++=Clip(y2+tb) | (Clip(y2+tg) <<8) | (Clip(y2+tr)<<16);// red 1
	#endif
	}
}


/*
 * Convert a YUV colour into a 15-bit RGB colour.
 *
 * The input Y is in the range 16..235; the input U and V components
 * are in the range -128..+127. The conversion equations for this are
 * (according to CCIR 601):
 *
 *	R = Y + 1.371 V
 *	G = Y - 0.698 V - 0.336 U
 *	B = Y + 1.732 U
 *
 * To avoid floating point, we scale all values by 1024.
 *
 * The resulting RGB values are in the range 16..235: we truncate these to
 * 5 bits each. and return a WORD containing 5-bits each for R, G and B
 * with bit 15 set to 0.
 */
#define RANGE(x, lo, hi)	max(lo, min(hi, x))

#if 0
WORD
//YUVToRGB15(int y, int u, int v)
YUVToRGB15(int y, int v, int u)
{
    int ScaledY = RANGE(y, 16, 235) * 1024;
    int red, green, blue;
	// added rounding (+4) because of 5 bit output
#if 1 // old equations
    red = RANGE((ScaledY + (1404 * v)) / 1024+4, 0, 255);
    green = RANGE( (ScaledY - (715 * v) - (344 * u)) / 1024, 0, 255);
    blue = RANGE( (ScaledY + (1774 * u)) / 1024+4, 0, 255);
#else
    red = RANGE((ScaledY + (1398 * v)) / 1024, 0, 255);
    green = RANGE( (ScaledY - (717 * v) - (342 * u)) / 1024, 0, 255);
    blue = RANGE( (ScaledY + (1773 * u)) / 1024, 0, 255);
#endif
//    return (WORD) (((red & 0xf8) << 7) | ((green & 0xf8) << 2) | ((blue & 0xf8) >>3) );
    return (WORD) (((blue & 0xf8) << 7) | ((green & 0xf8) << 2) | ((red & 0xf8) >>3) );
}
#else
WORD YUVToRGB15(int y, int u, int v)
{
	int Cr,Cb;
	int tr,tg,tb;
    int red, green, blue;
	// added rounding (+4) because of 5 bit output
		// math is 20% slower on pentium
		Cb=u-128; //u 
		Cr=v-128; //v
		tr=1398*Cr>>10;
		tg=(-717*Cr - 342*Cb) >> 10;
		tb=1773*Cb>>10;
		blue=Clip(y+tb);// blue 1
		green=Clip(y+tg);// green 1
		red=Clip(y+tr);// red 1
    return (WORD) (((red & 0xf8) << 7) | ((green & 0xf8) << 2) | ((blue & 0xf8) >>3) );
//    return (WORD) (((blue & 0xf8) << 7) | ((green & 0xf8) << 2) | ((red & 0xf8) >>3) );
}
#endif 

WORD //YUVToRGBBF16(int y, int u, int v)
YUVToRGBBF16(int y, int u, int v)
{
	int Cr,Cb;
	int tr,tg,tb;
    int red, green, blue;
	// added rounding (+4) because of 5 bit output
		// math is 20% slower on pentium
		Cb=u-128; //u 
		Cr=v-128; //v
		tr=1398*Cr>>10;
		tg=(-717*Cr - 342*Cb) >> 10;
		tb=1773*Cb>>10;
		blue=Clip(y+tb);// blue 1
		green=Clip(y+tg);// green 1
		red=Clip(y+tr);// red 1
    return (WORD) (((red & 0xf8) << 8) | ((green & 0xfc) << 3) | ((blue & 0xf8) >>3) );
}
#define ME 25.0

#define FIX(_X)    delta=abs(_X-sum); _X=delta< ME ? int(sum*(1-delta/ME) +_X*(delta/ME)) : _X
#define RDelta  0
#define GDelta  -RDelta
#define DDelta  0
#define ColorScale16	1.0
// Floating point version used in the constructor (with rounding)
WORD YUVToRGB15F(int y, int u, int v)
{
   
    int red, green, blue;
	u-=128;
	v-=128;
	y+=0;
	
	red =   RANGE((int)((double)y + 1.371 *ColorScale16 * (v+RDelta) + 2.371 / 2),0,255);
	green = RANGE((int)((double)y - 0.698 *ColorScale16* (v+GDelta) - 0.336*ColorScale16 * (u+GDelta) - 0.034 / 2 ),0,255);
    blue =  RANGE((int)((double)y + 1.732 *ColorScale16* (u+RDelta) + 2.732 / 2),0,255);

    return (WORD) (((red  & 0xf8) << 7) | ((green & 0xf8) << 2) | ((blue & 0xf8) >>3) );
}


WORD YUVToRGB16F(int y, int u, int v)
{
   
    int red, green, blue;
	u-=128-DDelta;
	v-=128+DDelta;
	y+=0;
	
	red =   RANGE((int)((double)y + 1.371 *ColorScale16 * (v+RDelta) + 2.371 / 2),0,255);
	green = RANGE((int)((double)y - 0.698 *ColorScale16* (v+GDelta) - 0.336*ColorScale16 * (u+GDelta) - 0.034 / 2 ),0,255);
    blue =  RANGE((int)((double)y + 1.732 *ColorScale16* (u+RDelta) + 2.732 / 2),0,255);
#if 0
	int sum=(red+blue+green)/3;
	double delta;
	FIX(red);
	FIX(green);
	FIX(blue);
#endif
    return (WORD) (((red  & 0xf8) << 8) | ((green & 0xfc) << 3) | ((blue & 0xf8) >>3) );
}
#ifdef TWO16BITTABLES

#define RDelta  0

WORD YUVToRGB16FA(int y, int u, int v)
{
    int red, green, blue;
	u-=128+DDelta;
	v-=128-DDelta;
	y+=0;
	
	red =   RANGE((int)((double)y + 1.371 *ColorScale16 * (v+RDelta) + 2.371 / 2),0,255);
	green = RANGE((int)((double)y - 0.698 *ColorScale16* (v+GDelta) - 0.336*ColorScale16 * (u+GDelta) - 0.034 / 2 ),0,255);
    blue =  RANGE((int)((double)y + 1.732 *ColorScale16* (u+RDelta) + 2.732 / 2),0,255);
#if 0
	int sum=(red+blue+green)/3;
	double delta;
	FIX(red);
	FIX(green);
	FIX(blue);
#endif
    return (WORD) (((red  & 0xf8) << 8) | ((green & 0xfc) << 3) | ((blue & 0xf8) >>3) );
}
#endif

FromYuvToRGB16::FromYuvToRGB16(BOOL BuildTable)
{
	OutputBytesPerPixel=2;
	InputBytesPerPixel=2;
	PixelsPerProcess=2;
	InputBitsPerPixel=16;
	OutputBitsPerPixel=16;
	invert=TRUE;
	BuildDitherTable();
    if (BuildTable) BuildConversionTable();
}

void FromYuvToRGB16::BuildDitherTable()
{
	for (int i=0;i<256;i++)
	{
#ifdef NODITHER
		YDitherTable[i]=i & 0xfc ;// min(i+4,255) & 0xfc;
#else
		YDitherTable[i]=min(i+4,255) & 0xfc;
#endif
		// old RoundClipShift3[i]=i<128 ? (i >> 3) +1: i>>3; 
		RoundClipShift3[i]=i-128 < 0 ? ((i+6) >> 3) : min(((i+2) >> 3),31); 
		//RoundClipShift3[i]=i>>3;
	}
}

void FromYuvToRGB16::BuildConversionTable()
{
    PWORD pRgb;
	UINT w;

    Table=new WORD[64 * 1024];
	pRgb = (PWORD)Table;

    /*
     * build a 16-bit yuv lookup table by stepping through each entry,
     * converting the yuv index to rgb and storing at that index. The index
     * to this table is a 16-bit value with the (6-bit) y in bits 15..10,
     * u in bits 9..5 and v in bits 4..0. All three components are unsigned.
     */
    for (w = 0; w < 64*1024; w++) {

		/*
		 * the YUVtoRGB55 conversion function takes values 0..255 for y,
		 * and -128..+127 for u and v. Pick out the relevant bits of the
		 * index for this cell, and shift to get values in this range.
		 * Subtract 128 from u and v to shift from 0..255 to -128..+127
		 */
 		
		*pRgb++ = YUVToRGB15F(
			   	    (w &  0xfc00) >> 8,
//				    ((w & 0x3e0) >> 2) -128 ,
//					((w & 0x1f) << 3) - 128);
				    ((w & 0x3e0) >> 2) ,
					((w & 0x1f) << 3) );
	 }
}

FromYuvToRGB16::~FromYuvToRGB16()
{
	delete [] Table;
}
/*
void FromYuvToRGB16::ProcessFrame(UCHAR *Output,const UCHAR *Input)
{
	CopyYUVToRGB555((PWORD)Table,xres,yres,Output,(UCHAR *)Input,TRUE);
}
*/

void FromYuvToRGB16::ProcessLine(UCHAR *pOutput,const UCHAR *pInput)
{
	int docols=xres>>1; // /PixelsPerProcess;
	unsigned short uv55 ,*pOut=(WORD*)pOutput;
	if (DitherCount++ & 1)
	{
		for (int col=0;col<docols;col++)
		{
			uv55=(RoundClipShift3[pInput[0]] <<5) | RoundClipShift3[pInput[2]];
			*pOut++ = Table[ ((pInput[1]  & 0xfc) << 8) | uv55 ] ;
			*pOut++ = Table[ (YDitherTable[pInput[3]] << 8) | uv55 ] ;
			pInput += sizeof(DWORD);
		}
	}
	else
	{
		for (int col=0;col<docols;col++)
		{
			uv55=(RoundClipShift3[pInput[0]] <<5) | RoundClipShift3[pInput[2]];
//			uv55=(pInput[0] & 0xf8) <<2 | (pInput[2] & 0xf8) >> 3;
			*pOut++ = Table[ (YDitherTable[pInput[1]] << 8) | uv55 ] ;
			*pOut++ = Table[ (((pInput[3] ) & 0xfc) << 8) | uv55 ] ;
			pInput += sizeof(DWORD);
		}
	}
}

FromYUY2ToRGB16::FromYUY2ToRGB16()
{
	lastxres = 0;
	mCanBlank = TRUE;
	mBlankLine = NULL;
}

void FromYUY2ToRGB16::InitBlankLine()
{
	if (mBlankLine != NULL)
	{
		if (lastxres == xres)
			return;
		else
			delete [] mBlankLine;
	}

	mBlankLine = new UCHAR[xres*sizeof(WORD)];
	WORD *p = (WORD *)mBlankLine;
	for (int i=0;i<xres;i++)
		*(p++) = 0x8000;
}

void FromYUY2ToRGB16::ProcessLine(UCHAR *pOutput,const UCHAR *pInput)
{
	int docols=xres>>1; // /PixelsPerProcess;
	unsigned short uv55; 
	DWORD *pOut=(DWORD*)pOutput;
	if (DitherCount++ & 1)
	{
		for (int col=0;col<docols;col++)
		{
			uv55=(RoundClipShift3[pInput[1]] <<5) | RoundClipShift3[pInput[3]];
			*pOut++ = Table[ ((pInput[0]  & 0xfc) << 8) | uv55 ] | (Table[ (YDitherTable[pInput[2]] << 8) | uv55 ] << 16);
			pInput += sizeof(DWORD);
		}
	}
	else
	{
		for (int col=0;col<docols;col++)
		{
			uv55=(RoundClipShift3[pInput[1]] <<5) | RoundClipShift3[pInput[3]];
//			uv55=(pInput[0] & 0xf8) <<2 | (pInput[2] & 0xf8) >> 3;
			*pOut++ = Table[ (YDitherTable[pInput[0]] << 8) | uv55 ] | (Table[ (((pInput[2] ) & 0xfc) << 8) | uv55 ] << 16);
			pInput += sizeof(DWORD);
		}
	}
}

FromYUY2ToRGBBF16::FromYUY2ToRGBBF16()
{
	lastxres = 0;
	mCanBlank = TRUE;
	mBlankLine = NULL;
}

void FromYUY2ToRGBBF16::InitBlankLine()
{
	if (mBlankLine != NULL)
	{
		if (lastxres == xres)
			return;
		else
			delete [] mBlankLine;
	}

	mBlankLine = new UCHAR[xres*sizeof(WORD)];
	WORD *p = (WORD *)mBlankLine;
	for (int i=0;i<xres;i++)
		*(p++) = 0x8000;
}

void FromYUY2ToRGBBF16::ProcessLine(UCHAR *pOutput,const UCHAR *pInput)
{
	int docols=xres>>1; // /PixelsPerProcess;
	unsigned short uv55; 
	DWORD *pOut=(DWORD*)pOutput;
	if (DitherCount++ & 1)
	{
		for (int col=0;col<docols;col++)
		{
			uv55=(RoundClipShift3[pInput[1]] <<5) | RoundClipShift3[pInput[3]];
			*pOut++ = Table[ ((pInput[0]  & 0xfc) << 8) | uv55 ] | (Table[ (YDitherTable[pInput[2]] << 8) | uv55 ] << 16);
			pInput += sizeof(DWORD);
		}
	}
	else
	{
		for (int col=0;col<docols;col++)
		{
			uv55=(RoundClipShift3[pInput[1]] <<5) | RoundClipShift3[pInput[3]];
//			uv55=(pInput[0] & 0xf8) <<2 | (pInput[2] & 0xf8) >> 3;
			*pOut++ = Table[ (YDitherTable[pInput[0]] << 8) | uv55 ] | (Table[ (((pInput[2] ) & 0xfc) << 8) | uv55 ] << 16);
			pInput += sizeof(DWORD);
		}
	}
}

void FromYuvToRGBBF16::BuildConversionTable()
{
    PWORD pRgb;
	UINT w;

    Table=new WORD[64 * 1024];
	pRgb = (PWORD)Table;
#ifdef TWO16BITTABLES
    PWORD pRgbA;
    TableA=new WORD[64 * 1024];
	pRgbA = (PWORD)TableA;
#endif
	//OutputDebugString("Building Bitfield dither table\n");
    /*
     * build a 16-bit yuv lookup table by stepping through each entry,
     * converting the yuv index to rgb and storing at that index. The index
     * to this table is a 16-bit value with the (6-bit) y in bits 15..10,
     * u in bits 9..5 and v in bits 4..0. All three components are unsigned.
     */
    for (w = 0; w < 64*1024; w++) {

		/*
		 * the YUVtoRGB55 conversion function takes values 0..255 for y,
		 * and -128..+127 for u and v. Pick out the relevant bits of the
		 * index for this cell, and shift to get values in this range.
		 * Subtract 128 from u and v to shift from 0..255 to -128..+127
		 */
 		
		*pRgb++ = YUVToRGB16F(
			   	    (w &  0xfc00) >> 8,
				    ((w & 0x3e0) >> 2) ,
					((w & 0x1f) << 3) );
#ifdef TWO16BITTABLES
		*pRgbA++ = YUVToRGB16FA(
			   	    (w &  0xfc00) >> 8,
				    ((w & 0x3e0) >> 2) ,
					((w & 0x1f) << 3) );
#endif
    }
}

 FromYuvToRGBBF16::FromYuvToRGBBF16() 
	 :FromYuvToRGB16(FALSE)
{
	BuildConversionTable();
}

void FrameConverter::ProcessLine(UCHAR *pOutput,const UCHAR *pInput)
{
	int docols=xres/PixelsPerProcess;
	int isize=PixelsPerProcess*InputBytesPerPixel;
	int osize=PixelsPerProcess*OutputBytesPerPixel;
	for (int col=0;col<docols;col++)
	{
		ProcessPixel(pOutput,pInput);
		pOutput+=osize;
		pInput+=isize;
	}
}

void FrameConverter::StretchLineUYVY(UCHAR * DestCPtr, const UCHAR *SourceCPtr)

/*
stretch or shrink the specified input line.  Writes to the specified output line.
*/
{
	long dx,dy,e,d,dx2;
	short *DestPtr=(short*)DestCPtr;
	short *SourcePtr=(short*)SourceCPtr;
	dx=dxDst-1;
	dy=xres-1;
	e=(dy<<1)-dx;
	dx2=dx<<1;
	dy<<=1;
	long y1 = 0;
	for(d=0;d<=dx;d+=2)
	{
		long FirstY1 = y1;

		//find input pixel for next point
		while(e>=0)	{
			y1++;
			e-=dx2;
		}
		e+=dy;

		//copy a pair of points
		if (FirstY1 & 1) {
			DestPtr[d] = (0xff00 & SourcePtr[FirstY1]) + (0x00ff & SourcePtr[FirstY1-1]);
			DestPtr[d+1] =(0xff00 & SourcePtr[y1]) + (0x00ff & SourcePtr[FirstY1]);
		}
		else {
			DestPtr[d] = SourcePtr[FirstY1];
			DestPtr[d+1] =(0xff00 & SourcePtr[y1]) + (0x00ff & SourcePtr[FirstY1+1]);
		}

		//find input pixel for next point
		while(e>=0)	{
			y1++;
			e-=dx2;
		}
		e+=dy;
	}
}




void FromRGB32ToYuv::ProcessLine(UCHAR *pOutput,const UCHAR *pInput)
{
	int docols=xres/PixelsPerProcess;
	int isize=PixelsPerProcess*InputBytesPerPixel;
	int osize=PixelsPerProcess*OutputBytesPerPixel;
	for (int col=0;col<docols;col++)
	{
		rgb2yuv422((const unsigned char*)pInput, pOutput);
		pOutput+=osize;
		pInput+=isize;
	}
}



/* this function not finnished !!!!!!!!!!!!!!!! 
void FromRGB16ToYuv::ProcessLine(UCHAR *pOutput,const UCHAR *pInput)
{
	int docols=hdisplay/PixelsPerProcess;
	int isize=PixelsPerProcess*InputBytesPerPixel;
	int osize=PixelsPerProcess*OutputBytesPerPixel;
	for (int col=0;col<docols;col++)
	{
   struct {UCHAR r,g,b;} p[2];
	unsigned short *in=(unsigned short *)Input; 
	for (int i=0;i<PixelsPerProcess;i++)
	{
		p[i].r=(*in &0x7c00) >> 7;
		p[i].g=(*in & 0x3e0) >> 2;
		p[i].b=(*in++ & 0x1f) << 3;
	}
		rgb2yuv422((const unsigned char*)pInput, (unsigned short*)pOutput);
		pOutput+=osize;
		pInput+=isize;
	}
}
*/