/* getpic.c, picture decoding                                               */

/* Copyright (C) 1994, MPEG Software Simulation Group. All Rights Reserved. */

/*
 * Disclaimer of Warranty
 *
 * These software programs are available to the user without any license fee or
 * royalty on an "as is" basis.  The MPEG Software Simulation Group disclaims
 * any and all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and user's
 * customers, employees, agents, transferees, successors, and assigns.
 *
 * The MPEG Software Simulation Group does not represent or warrant that the
 * programs furnished hereunder are free of infringement of any third-party
 * patents.
 *
 * Commercial implementations of MPEG-1 and MPEG-2 video, including shareware,
 * are subject to royalty fees to patent holders.  Many of these patents are
 * general enough such that they are unavoidable regardless of implementation
 * design.
 *
 */

#include <stdio.h>
#include <stdlib.h>

#include "config.h"
#include "global.h"

/* private prototypes*/
__inline static void clearblock _ANSI_ARGS_((int comp));



/* decode all macroblocks of the current picture */

void getIFrameMBs( int framenum)
{
  int comp;
  int MBA, MBAmax, MBAinc, mb_type;
  int slice_vert_pos_ext;
  int bx, by;
  unsigned int code;
  int dc_dct_pred[3];
  int qs;

  /* number of macroblocks per picture */
  MBAmax = mb_width*mb_height;

  MBA = 0; /* macroblock address */
  MBAinc = 0;

  fault=0;

  for (;;)
  {
    ld = &base;
    if (MBAinc==0)
    {
      if (!showbits(23) || fault) /* startcode or fault */
      {
resync: /* if fault: resynchronize to next startcode */
        fault = 0;
        ld = &base;

        if (MBA>=MBAmax)
          return; /* all macroblocks decoded */

        startcode();
        code = showbits(32);

        if (code<SLICE_MIN_START || code>SLICE_MAX_START)
        {
          /* only slice headers are allowed in picture_data */
          return;
        }

        flushbits(32);

        /* decode slice header (may change quant_scale) */
        slice_vert_pos_ext = getslicehdr();

        /* decode macroblock address increment */
        MBAinc = getMBA();

        if (fault) goto resync;

        /* set current location */
        MBA = ((slice_vert_pos_ext<<7) + (code&255) - 1)*mb_width + MBAinc - 1;
        MBAinc = 1; /* first macroblock in slice: not skipped */

        /* reset all DC coefficient and motion vector predictors */
        dc_dct_pred[0]=dc_dct_pred[1]=dc_dct_pred[2]=0;
      }
      else /* neither startcode nor fault */
      {
        if (MBA>=MBAmax)
        {
          return;
        }

        /* decode macroblock address increment */
        MBAinc = getMBA();

        if (fault) goto resync;
      }
    }

    if (MBA>=MBAmax)
    {
      /* MBAinc points beyond picture dimensions */
      return;
    }

    if (getbits1())
		mb_type = MB_INTRA;
	else {
	  mb_type = MB_INTRA | MB_QUANT;
	  if (!getbits1())
	    goto resync;  //resync if bit stream error
	}

    if (mb_type & MB_QUANT)
    {
      qs = getbits(5);
      ld->quant_scale = qs;
    }

    /* decode blocks */
    for (comp=0; comp<blk_cnt; comp++)
    {
      clearblock(comp);
      FastGetIntraBlock(comp,dc_dct_pred);
      if (fault) goto resync;
    }

    /* pixel coordinates of top left corner of current macroblock */
    bx = 16*(MBA%mb_width);
    by = 16*(MBA/mb_width);

    //inverse DCT on each block
    for (comp=0; comp<blk_cnt; comp++)
    {
      idct(ld->block[comp]);
    }

	//convert decoded blocks to RGB and copy to output buffer
	CovertToRGB(bx,by);

    /* advance to next macroblock */
    MBA++;
    MBAinc--;

  }
}

/* set block to zero */

__inline static void clearblock(comp)
int comp;
{
  long *bp;
  int i;

  bp = (long *)(ld->block[comp]);

  for (i=0; i<32; i++)
    *bp++ = 0;
}


