/* global.h, global variables                                               */

/* Copyright (C) 1994, MPEG Software Simulation Group. All Rights Reserved. */

/*
 * Disclaimer of Warranty
 *
 * These software programs are available to the user without any license fee or
 * royalty on an "as is" basis.  The MPEG Software Simulation Group disclaims
 * any and all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and user's
 * customers, employees, agents, transferees, successors, and assigns.
 *
 * The MPEG Software Simulation Group does not represent or warrant that the
 * programs furnished hereunder are free of infringement of any third-party
 * patents.
 *
 * Commercial implementations of MPEG-1 and MPEG-2 video, including shareware,
 * are subject to royalty fees to patent holders.  Many of these patents are
 * general enough such that they are unavoidable regardless of implementation
 * design.
 *
 */

#include "mpeg2dec.h"

/* choose between declaration (GLOBAL undefined)
 * and definition (GLOBAL defined)
 * GLOBAL is defined in exactly one file (mpeg2dec.c)
 */

#ifndef GLOBAL
#define EXTERN extern
#else
#define EXTERN
#endif

/* prototypes of global functions */

/* getblk.c */
void FastGetIntraBlock _ANSI_ARGS_((int comp, int dc_dct_pred[]));
int BuildDCTTable(void);

/* gethdr.c */
int getheader _ANSI_ARGS_((void));
void startcode _ANSI_ARGS_((void));
int getslicehdr _ANSI_ARGS_((void));

/* getpic.c */
void getIFrameMBs( int framenum);
void InitRGBTabels(void);

/* getvlc.c */
int getMBA _ANSI_ARGS_((void));
int getDClum _ANSI_ARGS_((void));
int getDCchrom _ANSI_ARGS_((void));

/* idct.c */
void idct _ANSI_ARGS_((short *block));
void init_idct _ANSI_ARGS_((void));

/* mpeg2dec.c */
void error _ANSI_ARGS_((char *text));
void warning _ANSI_ARGS_((char *text));
void printbits _ANSI_ARGS_((int code, int bits, int len));

/* torgb.c */
void CovertBlockToRGBBF16(int bx, int by);
void CovertBlockToRGB16(int bx, int by);
void CovertBlockToRGB24(int bx, int by);
void CovertBlockToUYVY(int bx, int by);

/* zig-zag scan */
EXTERN unsigned char zig_zag_scan[64]
#ifdef GLOBAL
=
{
  0,1,8,16,9,2,3,10,17,24,32,25,18,11,4,5,
  12,19,26,33,40,48,41,34,27,20,13,6,7,14,21,28,
  35,42,49,56,57,50,43,36,29,22,15,23,30,37,44,51,
  58,59,52,45,38,31,39,46,53,60,61,54,47,55,62,63
}
#endif
;

/* default intra quantization matrix */
EXTERN unsigned char default_intra_quantizer_matrix[64]
#ifdef GLOBAL
=
{
  8, 16, 19, 22, 26, 27, 29, 34,
  16, 16, 22, 24, 27, 29, 34, 37,
  19, 22, 26, 27, 29, 34, 34, 38,
  22, 22, 26, 27, 29, 34, 37, 40,
  22, 26, 27, 29, 32, 35, 40, 48,
  26, 27, 29, 32, 35, 40, 48, 58,
  26, 27, 29, 34, 38, 46, 56, 69,
  27, 29, 35, 38, 46, 56, 69, 83
}
#endif
;

/* non-linear quantization coefficient table */
EXTERN unsigned char non_linear_mquant_table[32]
#ifdef GLOBAL
=
{
   0, 1, 2, 3, 4, 5, 6, 7,
   8,10,12,14,16,18,20,22,
  24,28,32,36,40,44,48,52,
  56,64,72,80,88,96,104,112
}
#endif
;


//tables that are initialized in code:

EXTERN short CrToR[256];
EXTERN short CrToG[256];
EXTERN short CbToB[256];
EXTERN short CbToG[256];

EXTERN unsigned char ClipTable[1024];
EXTERN unsigned char ClipTableRGB16[1024];
EXTERN unsigned char *clp,*ClpRGB16;


/*many many globals*/
EXTERN void (*CovertToRGB)(int bx, int by);  //pointer to output conversion function
EXTERN int FrameNum;
EXTERN short *Output16Frame;
typedef struct {
	unsigned char blue;
	unsigned char green;
	unsigned char red;
} RGB24_PIXEL;
EXTERN RGB24_PIXEL *RGB24Frame;

EXTERN int quiet;
EXTERN int trace;
EXTERN char errortext[256];
EXTERN unsigned char *newframe[3];
EXTERN int profile, level;
EXTERN int horizontal_size,vertical_size,mb_width,mb_height;
EXTERN int coded_picture_width, coded_picture_height;
EXTERN int chroma_format,chrom_width,chrom_height,blk_cnt;
EXTERN int pict_type;
EXTERN int forw_r_size,back_r_size;
EXTERN int full_forw,full_back;
EXTERN int fault;
EXTERN int verbose;
EXTERN int prog_seq;
EXTERN int h_forw_r_size,v_forw_r_size,h_back_r_size,v_back_r_size;
EXTERN int dc_prec,pict_struct,topfirst,frame_pred_dct,conceal_mv;
EXTERN int intravlc,repeatfirst,prog_frame;
EXTERN int secondfield;
EXTERN int stwc_table_index,llw,llh,hm,hn,vm,vn;
EXTERN int lltempref,llx0,lly0,llprog_frame,llfieldsel;
EXTERN short *lltmp;
EXTERN char *llinputname;
EXTERN int twostreams,sflag;
EXTERN int refidct;
EXTERN int matrix_coefficients;
/* output */
EXTERN int framestoreflag;
EXTERN char *outputname;
EXTERN int outtype, hiQdither;
#define T_YUV   0
#define T_SIF   1
#define T_TGA   2
#define T_PPM   3
#define T_X11   4
#define T_X11HIQ 5

/* layer specific variables (needed for SNR and DP scalability) */
EXTERN struct layer_data {
  /* bit input */
  int infile;
  unsigned char rdbfr[2048];
  unsigned char *rdptr;
  //unsigned char inbfr[16];  DT mode
  unsigned  char *inbfr;
  int incnt;
  int bitcnt;
  /* sequence header */
  int intra_quantizer_matrix[64],non_intra_quantizer_matrix[64];
  int chroma_intra_quantizer_matrix[64],chroma_non_intra_quantizer_matrix[64];
  int mpeg2;
  /* sequence scalable extension */
  int scalable_mode;
  /* picture coding extension */
  int qscale_type,altscan;
  /* picture spatial scalable extension */
  int pict_scal;
  /* slice/macroblock */
  int pri_brk;
  int quant_scale;
  int intra_slice;
  short block[12][64];
} base, enhan, *ld;

/* some of these (currently unused) variables are layer dependent! */
EXTERN int low_delay;
EXTERN int video_format;  /* initialize to -1 for compliance test */
EXTERN int last_temp_ref; /* compliance test variable */
EXTERN int temp_ref, vbv_delay;
EXTERN int picture_rate, aspect_ratio, vbv_buffer_size;
EXTERN int last_pict_type;  /* compliance test variable */
EXTERN int colour_primaries;
EXTERN int transfer_characteristics;
EXTERN int display_horizontal_size, display_vertical_size;
EXTERN int bit_rate_value; 
EXTERN float bit_rate;

/*
get bits stuff
*/

/* to mask the n least significant bits of an integer */


EXTERN unsigned int msk[33]
#ifdef GLOBAL
={
  0x00000000,0x00000001,0x00000003,0x00000007,
  0x0000000f,0x0000001f,0x0000003f,0x0000007f,
  0x000000ff,0x000001ff,0x000003ff,0x000007ff,
  0x00000fff,0x00001fff,0x00003fff,0x00007fff,
  0x0000ffff,0x0001ffff,0x0003ffff,0x0007ffff,
  0x000fffff,0x001fffff,0x003fffff,0x007fffff,
  0x00ffffff,0x01ffffff,0x03ffffff,0x07ffffff,
  0x0fffffff,0x1fffffff,0x3fffffff,0x7fffffff,
  0xffffffff
}
#endif
;


/* return next bit (could be made faster than getbits(1)) */
EXTERN unsigned long m_CurrentBits;
EXTERN int m_ValidBits;
EXTERN unsigned char *m_FrameData;
EXTERN int m_UsedFrameBytes;
EXTERN int m_TotalFrameBytes;



__inline unsigned int showbits(int n)
/*
return next n bits (right adjusted)
*/
{
  return (m_CurrentBits>>(m_ValidBits-n)) & msk[n];
}


__inline void flushbits(int n)
/*
advance by n bits, don't check for errors.  We handle exceptions in DecodeFrame()
*/
{
	m_ValidBits -= n;
	while (m_ValidBits < 25) {
		m_CurrentBits = (m_CurrentBits << 8) | m_FrameData[m_UsedFrameBytes++];
		m_ValidBits += 8;
	}
}

__inline void InitBitsToFrame(unsigned char * FrameData, int FrameBytes)
/*
DT added routine to make bit fetching work with a frame, and not a file.
Simply set the input buffer to the start of the frame data, and set the 
input bit count.  This routine should be called once before each
frame is processed.
*/
{
	// load buffer info
	m_FrameData = FrameData;
	m_UsedFrameBytes = 0;
	m_TotalFrameBytes = FrameBytes;

	//init current bits
	m_ValidBits = 0;
	flushbits(0);

}

__inline unsigned int getbits(int n)
{
  unsigned int l;

  l = showbits(n);
  flushbits(n);

  return l;
}

__inline unsigned int getbits1()
{
  return getbits(1);
}


