#ifndef _COPYBITS_
#define _COPYBITS_
/*
copybits.h
This file defines and implements a completly inline class that is used to copy and
pack bits into an output byte stream.  This class sets the target output
pointer in the constructor.  It has a CopyBits method that can be used to output 
the specified number of bits (up to 16) to the memery
*/

#ifdef BIG_ENDIAN
//if big endian we can output in chunks of 16 bits
#define CB_OUTSIZE 16
#define CB_OUTTYPE unsigned short

#else
//if little endian we must output in bytes
#define CB_OUTSIZE 8
#define CB_OUTTYPE unsigned char

#endif

#include <stdio.h>

class CCopyBits {
	private:
	long acc;
	int NumBits;
	CB_OUTTYPE *outptr;
	
	public:
	//constructor
	CCopyBits(unsigned char *buffer) {
		NumBits = acc = 0;
		outptr = (CB_OUTTYPE *)buffer;
	};

	//copy method, up to 16 bits
	void CopyBits(int data, int size) {
        //printf("%4x-%2d ",data,size);
		acc = (acc << size) + data;	 //shift up old data and copy input data
		NumBits += size;			 //update bit count

		int rem;
        while ( (rem = NumBits - CB_OUTSIZE) >= 0) { //do we have enough bits to output?
			*outptr++ = (CB_OUTTYPE)(acc >> rem); //if so, output data
			NumBits = rem;		 //adjust bit count
		}
	};

	//flush any remaining bits in acc
	void Flush() {
		if (NumBits) {
			acc <<= CB_OUTSIZE - NumBits;  //zero unused bits
			*outptr++ = (CB_OUTTYPE)acc; //output data
		}
		NumBits = acc = 0;			 //re-init data
	};
			
};



#endif	 //#ifndef _COPYBITS_


