#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include "cirbuf.h"
#include "stream\basepin.h"

/*cirbuf.h
This is the implementation of a very basic circular buffer class
*/

int CCircBuffer::FlushToFile(HANDLE hFile)
/*
this function moves all valid data in the cir buffer to the
specified file.  The function returns the number of points
written.
*/
{
    int size1,size2;
    CBT *ptr1, *ptr2;

    GetDataPointers(&ptr1, &size1, &ptr2, &size2);

    DWORD OutSize;
    DWORD bytes = size1 * sizeof(CBT);
    WriteFile(hFile, ptr1, bytes, &OutSize, NULL);
    //ASSERT(bytes == OutSize);
    if (size2) {
        bytes = size2 * sizeof(CBT);
        WriteFile(hFile, ptr2, bytes, &OutSize, NULL);
        //ASSERT(bytes == OutSize);
    }

    m_ReadLoc = m_WriteLoc;

    return size1+size2;

}




int CCircBuffer::FlushToPin(CBasePin *Pin)
/*
this function moves all valid data in the cir buffer to the
specified file.  The function returns the number of points
written.
*/
{
	int TotalSize=0;
    int size1,size2;
    CBT *ptr1, *ptr2;

	while (ValidData()>0)
	{
		GetDataPointers(&ptr1, &size1, &ptr2, &size2);
		SBuff *buff = Pin->GetBuffer();

		int bytes = min(size1 * (int)sizeof(CBT),(int)buff->DataSize);
		buff->ValidBytes = (int)bytes;
		CopyMemory(buff->data ,ptr1,bytes);
		m_ReadLoc+=bytes/sizeof(CBT);
		if (m_ReadLoc>m_Size) m_ReadLoc=0;
		//ASSERT(bytes == OutSize);
		if (size2  && buff->ValidBytes<buff->DataSize) {
			bytes = min(size2 * (int)sizeof(CBT),(int)(buff->DataSize-buff->ValidBytes));
			CopyMemory(buff->data+buff->ValidBytes ,ptr2,bytes);
			buff->ValidBytes += (int)bytes;
			m_ReadLoc = bytes/sizeof(CBT);
		}
		size1=buff->ValidBytes/sizeof(CBT);
		Pin->PutBuffer(buff);
		TotalSize+=size1;
	}
    return TotalSize ;
}



int CCircBuffer::FreeSpace()
{
	if (m_ReadLoc<m_WriteLoc)
		return m_Size-(m_WriteLoc-m_ReadLoc);
	else
		return (m_ReadLoc-m_WriteLoc);
}

void CCircBuffer::Initialize(CBT *Buffer,int Size)
{
	m_Buffer=Buffer;
	m_Size=Size;
	m_ReadLoc=0;
	m_WriteLoc=0;
}
