/* rcuser.c
c interface defs for recode classes
*/
#define BUILD_RECODE_DLL
#include "recode.h"
#include "video.h"
#include "errors.h"
#include "audio\encode.h"
#include "stream\stream.h"
#include "stream\videoCD.h"
#include "resource.h"
#include "convaud.h"
#include "bwuser.h"
#include "rcuser.h"


//alloc recode object, return pointer as handle
HANDLE RECODE_INTERFACE AllocRecode ( 
    DEVID DevID,			//handle to VT_access board object
    PVOID DMALinearAddr,    //dma buffer linear address   
    DWORD DMAPhysAddr,		//dma buffer physical address
    int DMASize				//dma buffer size
)
{
	return (HANDLE)new	CIBPRecode(DevID);
}





//free recode object
void RECODE_INTERFACE FreeRecode(HANDLE RecodeObj)
{
    delete ((CIBPRecode *)RecodeObj);
}

//compress input file to output file
ECODE RECODE_INTERFACE Compress(
    //interface parameters
    HANDLE pObject,				// handle to recode object
    const char *AviFileName, 	//full path name of input avi file
    const char *MpgFileName, 	//full path name of output mpg file
    HWND hWnd,		     		//window handle of parrent window for progress dialog

    //audio re-compression params
    int AudioMode,         		//none, mono, stereo, daul channel, joint stereo (see bwuser.h)
    int AudioDataRate,	    	//output audio data rate in kbits/second (1000 bits/second)

    //video re-compression params
    int VideoDataRate,			//output video rate in kbits/second	(1000 bits/second)
    BOOL UseQSIF,				//TRUE to select QSIF resolution, FALSE to select SIF

    //stream mix parameters
    EStreamMixType StreamMixType, //type of mixing, normal, none, or VideoCD
    double StreamDataRate,  	//total stream data rate in bytes/second, 0 = use optimum rate
    int PackSize,				//in bytes
    int PacketsPerPack,			//number of data packets per pack
								// 0 => use 1 audio packet and 1 video packet in each pack
    double AudioDelay			//delay audio form video (in seconds, can be negative)
)
{
	return ((CIBPRecode *)pObject)->Compress(AviFileName
											,MpgFileName
											,hWnd
											,AudioMode
											,AudioDataRate
											,VideoDataRate
											,UseQSIF
											,StreamMixType
											,StreamDataRate
											,PackSize
											,PacketsPerPack
											,AudioDelay
											);
}


//compress input file to output file
ECODE RECODE_INTERFACE CompressStreams(
    //interface parameters
    HANDLE pObject,				// handle to recode object
	PAVISTREAM pAudioStream,	//input audio stream
	PAVISTREAM pVideoStream,	//input video stream
    const char *MpgFileName, 	//full path name of output mpg file
    HWND hWnd,		     		//window handle of parrent window for progress dialog

    //audio re-compression params
    int AudioMode,         		//none, mono, stereo, daul channel, joint stereo (see bwuser.h)
    int AudioDataRate,	    	//output audio data rate in kbits/second (1000 bits/second)

    //video re-compression params
    int VideoDataRate,			//output video rate in kbits/second	(1000 bits/second)
    BOOL UseQSIF,				//TRUE to select QSIF resolution, FALSE to select SIF

    //stream mix parameters
    EStreamMixType StreamMixType, //type of mixing, normal, none, or VideoCD
    double StreamDataRate,  	//total stream data rate in bytes/second, 0 = use optimum rate
    int PackSize,				//in bytes
    int PacketsPerPack,			//number of data packets per pack
								// 0 => use 1 audio packet and 1 video packet in each pack
    double AudioDelay			//delay audio form video (in seconds, can be negative)
)
{
	return ((CIBPRecode *)pObject)->Compress(pAudioStream
										    ,pVideoStream
											,MpgFileName
											,hWnd
											,AudioMode
											,AudioDataRate
											,VideoDataRate
											,UseQSIF
											,StreamMixType
											,StreamDataRate
											,PackSize
											,PacketsPerPack
											,AudioDelay
											);
}


// display compression configuration dialog
ECODE RECODE_INTERFACE CompressConfigure(HWND hwnd)
{
	return BW_ConfigRecompress(hwnd);
}


	
//create an audio encoder object, return pointer as handle
HANDLE RECODE_INTERFACE CreateAudioEncoder( 
    ESamplingFreq Freq,  
    EStereoType Type,
    int BitRate	/*(in KHz)*/
)
{
    return (HANDLE)new CAudioEncode(Freq, Type, BitRate);
}


//remove audio encoder object
void RECODE_INTERFACE DeleteAudioEncoder(HANDLE AudioObj)
{
    delete (CAudioEncode *)AudioObj;
}


//return size of next audio frame output in bytes
int RECODE_INTERFACE GetNextAudioFrameSize(HANDLE AudioObj)
{
    return ((CAudioEncode *)AudioObj)->GetNextFrameSize();
}


//return size of max audio frame output in bytes
int RECODE_INTERFACE GetMaxAudioFrameSize(HANDLE AudioObj)
{
    return ((CAudioEncode *)AudioObj)->GetMaxFrameSize();
}

//process a frame of audio data
void RECODE_INTERFACE ProcessFrame(  
    HANDLE AudioObj,
    int combined_smr,
    int *maxsb,
    short *FrameInput,		//1152 16 bit samples or 2*1152 16 samples for stereo
    BYTE  *FrameOutput		//(1152 * (bit rate / 8) / sample rate) bytes on 
)
{	
	((CAudioEncode *)AudioObj)->ProcessFrame(FrameInput, FrameOutput, combined_smr,maxsb);
}


//create a stream mixer object, return pointer as handle
HANDLE RECODE_INTERFACE CreateStreamMixer(void)
{
	return (HANDLE)new CStreamMix;
}


//create a video CD mixer object, return pointer as handle
HANDLE RECODE_INTERFACE CreateVideoCDMixer(void)
{
	return (HANDLE)new CVideoCDMix;
}

//remove stream mixer object
void RECODE_INTERFACE DeleteStreamMixer(HANDLE StreamObj)
{
	delete (CStreamMix *)StreamObj;
}

//initialize stream mixer object
int RECODE_INTERFACE InitializeStream( 
    HANDLE StreamObj,     // stream object
    HANDLE OutputFile,    //win32 file handle
    int PackSize,         //in bytes
    int PacketsPerPack,   //if 0, we will use 2 packets per pack
                          //the first will be an video packet, the 
                          //second an audio packet.  If this is non-zero
                          //we use the specified number of same
                          //size packets in just-in-time audio mode.
                          //in this case audio is sent just before the
                          //audio buffers underflow.

    double StreamRate,    //rate of output stream in bytes/second
                          //(if set to zero, min required rate
                          //will be used.  Ignored for non-fixed rate video)
    double AudioDelay,    //delay audio form video presentation in seconds
                          //(can be negative)
    int    VBVPadding,    //extra buffer padding size in bytes
    double VideoFrameRate //in frames/second, 0.0 => use rate as specified
                          //video stream
)
{

	return ((CStreamMix *)StreamObj)->Initialize (OutputFile
												  ,PackSize
												  ,PacketsPerPack
												  ,StreamRate
												  ,AudioDelay
												  ,VBVPadding
												  ,VideoFrameRate
												  );
}


//get stream mixer video pin, return pointer as handle
HANDLE RECODE_INTERFACE GetStreamVideoPin(HANDLE StreamObj)
{
	return (HANDLE)((CStreamMix *)StreamObj)->GetVideoPin();
}
 

//get stream mixer audio pin, return pointer as handle
HANDLE RECODE_INTERFACE GetStreamAudioPin(HANDLE StreamObj)
{
	return (HANDLE)((CStreamMix *)StreamObj)->GetAudioPin();
}


//get stream mixer pin buffer, return pointer as handle
HANDLE RECODE_INTERFACE GetPinBuffer(HANDLE PinObj)
{
	return (HANDLE)((CBasePin *)PinObj)->GetBuffer();
}

//put buffer to stream mixer pin
void RECODE_INTERFACE PutPinBuffer(HANDLE PinObj, HANDLE buff)
{
	((CBasePin *)PinObj)->PutBuffer((SBuff *)buff);
}


// return true if buffer is ready
BOOL RECODE_INTERFACE PinBufferReady(HANDLE PinObj)
{
	return ((CBasePin *)PinObj)->BufferReady();
}


//flush all stream data to output file
void RECODE_INTERFACE FlushStreamData(HANDLE StreamObj)
{
	((CStreamMix *)StreamObj)->FlushData();
}


#include "video.h"
#include "stream\critsec.h"
#include "audio\encode.h"
#include "rcuser.h"

#ifdef BUILD_RECODE_DLL
#define RECODE_INTERFACE __declspec(dllexport) __stdcall
#else
#define RECODE_INTERFACE __declspec(dllimport) __stdcall
#endif

class CAudioEncode;

class  CRecodeExternal{
	public:

	//constructor, takes board and dma buffer info 
	RECODE_INTERFACE CRecodeExternal (			// dma info no longer needed or used
                 DEVID DevID);				//handle to VT_access board object

	
	//distructor
	RECODE_INTERFACE ~CRecodeExternal(void);

	/*
	worker function, takes parametes as specifed.	This funciton returns
	the appropriate error code, VT_NO_ERROR (0) if success
	*/
	ECODE RECODE_INTERFACE Compress(
	//interface parameters
		const char *AviFileName, //full path name of input avi file
		const char *MpgFileName, //full path name of output mpg file
		HWND hWnd,				 //window handle of parrent window for progress dialog

		//audio re-compression params
	    int AudioMode,          //none, mono, stereo, daul channel, joint stereo (see bwuser.h)
		int AudioDataRate,		//output audio data rate in kbits/second (1000 bits/second)

		//video re-compression params
		int VideoDataRate,		//output video rate in kbits/second	(1000 bits/second)
		BOOL UseQSIF,			//TRUE to select QSIF resolution, FALSE to select SIF

		//stream mix parameters
        EStreamMixType StreamMixType,  //type of mixing, normal, none, or VideoCD
		double StreamDataRate,  //total stream data rate in bytes/second, 0 = use optimum rate
		int PackSize,			//in bytes
		int PacketsPerPack,		//number of data packets per pack:
								// 0 => use 1 audio packet and 1 video packet in each pack
		double AudioDelay);		//delay audio form video (in seconds, can be negative)

	//wait for compression to finish, return compleation status
	ECODE RECODE_INTERFACE WaitForCompress(void);
    
//	private:

    //helper functions
    ECODE SetupAudioEncoding(int AudioDataRate, BOOL UseJointStereo);
//    static DWORD WINAPI AudioEncodeThread(LPVOID ThrdInfo);
  //  static DWORD WINAPI StreamMix(LPVOID ThrdInfo);
//	static DWORD WINAPI CompressThread(LPVOID ThrdInfo);	   //outputs an audio & a video file
//	static DWORD WINAPI CompressThreadMixed(LPVOID ThrdInfo);  //outputs a single system stream file
//	void KillThreads(void);


	//progress dialog functions
//	static BOOL CALLBACK ProgressDialogProc(HWND hwndDlg, UINT uMsg,
//												WPARAM wParam, LPARAM lParam);
//	BOOL OnInitDialog(HWND hwndDlg);
//	void UpdateProgress(HWND hwndDlg);
//	void OnCancel(HWND hwndDlg);


    //member data
	ECODE mStatus;					//compleation status of last compresion
    CRecodeVideo mRecodeVideo;      //object to recode the video
    PAVIFILE mAviFile;              //pointer to the AviFile object
	PAVISTREAM mAudioStream;        //pointer to audio stream object
    CAudioEncode *mAudioEncode;     //pointer to audio encoder object
    HANDLE mhMpgFile;               //file handle to output mpeg system stream file
	HANDLE mAudioThread;			//handle of audio compression thread
	HANDLE mStreamThread;			//handle of stream compression thread
	HANDLE mCompressThread;			//handle of main active compression thread (video is
									//compressed in this thread)
    CCritSec mCritSec;				//critical section object
	int mStreamMixTime;				//estimate of stream mixing time in seconds

	BOOL mAbort;					//TRUE if processing should be aborted
	HCURSOR mOldCursor;				//saves orig cursor when we show wait cursor
	char mAudioFileName[300];		//full path name of output audio mpg file (if \0, don't
									//use temp files)


	//compress params
	char mAviFileName[300]; //full path name of input avi file
	char mMpgFileName[300]; //full path name of output mpg file
    BOOL mEnableAudio;      //TRUE if audio processing is enabled
	int mAudioDataRate;		//output audio data rate in kbits/second (1000 bits/second)
	EStereoType mAudioType;	//audio stereo mode
	BOOL mUseJointStereo;	//if input is stereo, encode in joint-stereo mode if TRUE
	int mVideoDataRate;		//output video rate in bits/second
	BOOL mUseQSIF;			//TRUE to select QSIF resolution, FALSE to select SIF
    EStreamMixType mStreamMixType; //type of mixing, normal, none, or VideoCD
	double mStreamDataRate;  //total stream data rate in bytes/second, 0 = use optimum rate
	int mPackSize;			//in bytes
	int mPacketsPerPack;		//number of data packets per pack
	double mAudioDelay;		//delay audio form video (in seconds, can be negative)
	DEVID    mDevID;  
	CBasePin *mVideoPin;	// direct stream mix pin
	CBasePin *mAudioPin;	// direct stream mix pin

};

CRecodeExternal::CRecodeExternal ( 
                 DEVID DevID)		//handle to VT_access board object
                 : mRecodeVideo(DevID)
{
//	InitCommonControls();
//	AVIFileInit();
	mDevID=DevID;
	mCompressThread = NULL;
	mAudioThread = NULL;
	mStreamThread = NULL;
	mStatus = VT_NO_ERROR;
	mOldCursor = NULL;
} 
 

//distructor
CRecodeExternal::~CRecodeExternal ( void)
{
//	KillThreads();
//   	AVIFileExit();
}


//alloc recode object, return pointer as handle
HANDLE RECODE_INTERFACE OpenRecode( 
    DEVID DevID			//handle to VT_access board object
)
{
	return (HANDLE)new	CRecodeExternal(DevID);
}


// begin frame based compression
ECODE RECODE_INTERFACE CompressBegin(
    HANDLE pObject,				// handle to recode object
	LPSTR szMpegFile,			// output file name
	UINT FrameRate,				// video frame rate
	UINT * lpAudioBufferSize,	// reqired audio buffer size
    BITMAPINFOHEADER *lpbi,     // desrcibes video input stream     
    WAVEFORMATEX *lpwf	        // desrcibes audio input stream     
)
{
	return (0);
}

// put a video frame to the recoder 
ECODE RECODE_INTERFACE CompressPutVideo(
    HANDLE pObject,			// handle to recode object
    LPVIDEOHDR lpFrame		// video header
)
{
	return (0);
}


// put a video buffer to the recoder 
ECODE RECODE_INTERFACE CompressPutAudio(
    HANDLE pObject,		// handle to recode object
    LPWAVEHDR lpWave	// audio header
)
{
	CRecodeExternal *pRC=(CRecodeExternal *)pObject;
	int Bytes=lpWave->dwBufferLength;
	while (Bytes)
	{
		SBuff *Buff=pRC->mAudioPin->GetBuffer();
		int size=min(Bytes,Buff->DataSize);
		memcpy(Buff->data,lpWave->lpData,size);
		Buff->ValidBytes=size;
		Bytes-=size;
		pRC->mAudioPin->PutBuffer(Buff);
	}
	return 0;
}

// query recoder to see if ready for a video frame 

BOOL RECODE_INTERFACE CompressQueryVideoReady (HANDLE pObject)
{
	CRecodeExternal *pRC=(CRecodeExternal *)pObject;
	return pRC->mVideoPin->BufferReady();
}


// query recoder to see if ready for an audio buffer 
BOOL RECODE_INTERFACE CompressQueryAudioReady (HANDLE pObject)
{
	CRecodeExternal *pRC=(CRecodeExternal *)pObject;
	return pRC->mAudioPin->BufferReady();
}
