#ifndef __RCUSER__
#define __RCUSER__

/* rcuser.h
c interface defs for recode classes
*/

#ifdef BUILD_RECODE_DLL
#define RECODE_INTERFACE __declspec(dllexport) __stdcall
#else
#define RECODE_INTERFACE __declspec(dllimport) __stdcall
#endif


typedef enum 
{
    StandardMix = 0,
    NoMix = 1,
    VideoCDMix = 2
}EStreamMixType;


#ifndef BUILD_RCUSER

typedef enum 
{
	STEREO, 
	JOINT_STEREO, 
	DUAL_CHANNEL, 
	MONO
} EStereoType;

typedef enum 
{
	SF_44100_HZ, 
	SF_48000_HZ, 
	SF_32000_HZ, 
	SF_22050_HZ, 
	SF_11025_HZ
} ESamplingFreq;

typedef struct _SBuff_ 
{
    struct _SBuff_ *next;   //linked list pointer
    int DataSize;			//total number of data bytes
    int ValidBytes;			//current number of valid bytes
    char data[2];

} SBuff;

#define MONO_INPUT_SIZE 1152
#define STEREO_INPUT_SIZE (2*MONO_INPUT_SIZE)

#endif


#ifdef __cplusplus
extern "C" {
#endif

//alloc recode object, return pointer as handle
HANDLE RECODE_INTERFACE AllocRecode ( 
    DEVID DevID,			//handle to VT_access board object
    PVOID DMALinearAddr,    //dma buffer linear address   
    DWORD DMAPhysAddr,		//dma buffer physical address
    int DMASize);			//dma buffer size

HANDLE RECODE_INTERFACE OpenRecode ( 
    DEVID DevID);		//handle to VT_access board object


//free recode object
void RECODE_INTERFACE FreeRecode(HANDLE RecodeObj);

//compress input file to output file
ECODE RECODE_INTERFACE Compress(
    //interface parameters
    HANDLE pObject,				// handle to recode object
    const char *AviFileName, 	//full path name of input avi file
    const char *MpgFileName, 	//full path name of output mpg file
    HWND hWnd,		     		//window handle of parrent window for progress dialog

    //audio re-compression params
    int AudioMode,         		//none, mono, stereo, daul channel, joint stereo (see bwuser.h)
    int AudioDataRate,	    	//output audio data rate in kbits/second (1000 bits/second)

    //video re-compression params
    int VideoDataRate,			//output video rate in kbits/second	(1000 bits/second)
    BOOL UseQSIF,				//TRUE to select QSIF resolution, FALSE to select SIF

    //stream mix parameters
    EStreamMixType StreamMixType, //type of mixing, normal, none, or VideoCD
    double StreamDataRate,  	//total stream data rate in bytes/second, 0 = use optimum rate
    int PackSize,				//in bytes
    int PacketsPerPack,			//number of data packets per pack
								// 0 => use 1 audio packet and 1 video packet in each pack
    double AudioDelay);			//delay audio form video (in seconds, can be negative)


//compress input streams to output file
ECODE RECODE_INTERFACE CompressStreams(
    //interface parameters
    HANDLE pObject,				// handle to recode object
	PAVISTREAM pAudioStream,	//input audio stream
	PAVISTREAM pVideoStream,	//input video stream
    const char *MpgFileName, 	//full path name of output mpg file
    HWND hWnd,		     		//window handle of parrent window for progress dialog

    //audio re-compression params
    int AudioMode,         		//none, mono, stereo, daul channel, joint stereo (see bwuser.h)
    int AudioDataRate,	    	//output audio data rate in kbits/second (1000 bits/second)

    //video re-compression params
    int VideoDataRate,			//output video rate in kbits/second	(1000 bits/second)
    BOOL UseQSIF,				//TRUE to select QSIF resolution, FALSE to select SIF

    //stream mix parameters
    EStreamMixType StreamMixType, //type of mixing, normal, none, or VideoCD
    double StreamDataRate,  	//total stream data rate in bytes/second, 0 = use optimum rate
    int PackSize,				//in bytes
    int PacketsPerPack,			//number of data packets per pack
								// 0 => use 1 audio packet and 1 video packet in each pack
    double AudioDelay);			//delay audio form video (in seconds, can be negative)


// display compression configuration dialog
ECODE RECODE_INTERFACE CompressConfigure(HWND hwnd);

// begin frame based compression
ECODE RECODE_INTERFACE CompressBegin(
    HANDLE pObject,				// handle to recode object
	LPSTR szMpegFile,			// output file name
	UINT FrameRate,				// video frame rate
	UINT * lpAudioBufferSize,	// reqired audio buffer size
    BITMAPINFOHEADER *lpbi,     // desrcibes video input stream     
    WAVEFORMATEX *lpwf);        // desrcibes audio input stream     

// put a video frame to the recoder 
ECODE RECODE_INTERFACE CompressPutVideo(
    HANDLE pObject,			// handle to recode object
    LPVIDEOHDR lpFrame);	// video header

// put a video buffer to the recoder 
ECODE RECODE_INTERFACE CompressPutAudio(
    HANDLE pObject,			// handle to recode object
    LPWAVEHDR lpWave);		// audio header

// query recoder to see if ready for a video frame 
BOOL RECODE_INTERFACE CompressQueryVideoReady (HANDLE pObject);

// query recoder to see if ready for an audio buffer 
BOOL RECODE_INTERFACE CompressQueryAudioReady (HANDLE pObject);

//create an audio encoder object, return pointer as handle
HANDLE RECODE_INTERFACE CreateAudioEncoder( 
    ESamplingFreq Freq,  
    EStereoType Type,
    int BitRate/*(in KHz)*/);

//remove audio encoder object
void RECODE_INTERFACE DeleteAudioEncoder(HANDLE AudioObj);

//return size of next audio frame output in bytes
int RECODE_INTERFACE GetNextAudioFrameSize(HANDLE AudioObj);

//return size of max audio frame output in bytes
int RECODE_INTERFACE GetMaxAudioFrameSize(HANDLE AudioObj);

//process a frame of audio data
void RECODE_INTERFACE ProcessFrame(  
    HANDLE AudioObj,
    int combined_smr,
    int *maxsb,
    short *FrameInput,    //1152 16 bit samples or 2*1152 16 samples for stereo
    BYTE  *FrameOutput); //(1152 * (bit rate / 8) / sample rate) bytes on 


//create a stream mixer object, return pointer as handle
HANDLE RECODE_INTERFACE CreateStreamMixer(void);

//create a video CD mixer object, return pointer as handle
HANDLE RECODE_INTERFACE CreateVideoCDMixer(void);

//remove stream mixer object
void RECODE_INTERFACE DeleteStreamMixer(HANDLE StreamObj);

//initialize stream mixer object
int RECODE_INTERFACE InitializeStream( 
    HANDLE StreamObj,     // stream object
    HANDLE OutputFile,    //win32 file handle
    int PackSize,         //in bytes
    int PacketsPerPack,   //if 0, we will use 2 packets per pack
                          //the first will be an video packet, the 
                          //second an audio packet.  If this is non-zero
                          //we use the specified number of same
                          //size packets in just-in-time audio mode.
                          //in this case audio is sent just before the
                          //audio buffers underflow.

    double StreamRate,    //rate of output stream in bytes/second
                          //(if set to zero, min required rate
                          //will be used.  Ignored for non-fixed rate video)
    double AudioDelay,    //delay audio form video presentation in seconds
                          //(can be negative)
    int    VBVPadding,    //extra buffer padding size in bytes
    double VideoFrameRate); //in frames/second, 0.0 => use rate as specified
                            //video stream

//get stream mixer video pin, return pointer as handle
HANDLE RECODE_INTERFACE GetStreamVideoPin(HANDLE StreamObj); 

//get stream mixer audio pin, return pointer as handle
HANDLE RECODE_INTERFACE GetStreamAudioPin(HANDLE StreamObj);

//get stream mixer pin buffer, return pointer as handle
HANDLE RECODE_INTERFACE GetPinBuffer(HANDLE PinObj);

//put buffer to stream mixer pin
void RECODE_INTERFACE PutPinBuffer(HANDLE PinObj, HANDLE buff);

// return true if buffer is ready
BOOL RECODE_INTERFACE PinBufferReady(HANDLE PinObj);

//flush all stream data to output file
void RECODE_INTERFACE FlushStreamData(HANDLE StreamObj);


#ifdef __cplusplus
}
#endif

#endif //#ifndef __RCUSER__
