/* VideoCD.cpp
   implementation of video cd stream mixer
*/

#include "VideoCD.h"
#define STEADY_AUDIO_RATE   //only rate control video

#ifdef _DEBUG
#define TRACEWND_CLASSNAME "MfxTraceWindow"
#define TRACEWND_MESSAGE   "*WM_TRACE_MSG"

extern "C" void __cdecl dprintf(LPSTR szFormat, ...)
{
    char ach[128];

    vsprintf(ach,szFormat,(LPSTR)(&szFormat+1));

	HWND TraceWnd = FindWindow(TRACEWND_CLASSNAME, NULL);
	if (TraceWnd) {
		static UINT WM_TRACE_MSG = RegisterWindowMessage(TRACEWND_MESSAGE);

		// Found Trace window: send message there as a global atom.
		// Delete atom after use.
		//
		ATOM atom = GlobalAddAtom((LPCSTR)ach);
		SendMessage(TraceWnd,WM_TRACE_MSG, (WPARAM)atom,NULL);
		GlobalDeleteAtom(atom);
	}
	else
		OutputDebugString(ach);
}

#endif


BOOL CVideoCDMix::StartUpStream(void)
/*
This function is called to start up the stream.  It initializes all required
data.  The function returns TRUE if successfull, FALSE if more input data is 
required form the audio or video pins
*/
{
    //insure input params are set to match video CD
/*
    mPackSize = VIDEO_CD_PACK_SIZE;
    mStreamRate = 0.0;
    mJustInTimeAudio = TRUE;
    mPacketsPerPack = 1;
*/
    //find audio and video frame rates
	//(must call, so AudioFrameToTime() & VideoFrameToTime() are valid)
    if (!InitRateData(0.0))
        return FALSE;

	//init object data
	mStreamRate = VIDEO_CD_PACK_SIZE / VIDEO_CD_PACK_TIME;
	mPackCount = VIDEO_CD_START_DELAY;     //init pack count to match 1st pack that has data
	mAdjustedStreamTime = 0.0;
	mAudioTimeAhead = VIDEO_CD_AUDIO_FRAMES_AHEAD / mAudioFrameRate; 
	mVBVPadding=VIDEO_CD_TARGET_VBV_FULLNESS;
	mVBVPaddingTime = mVBVPadding / mStreamRate;
    mVideoStuffSize = VIDEO_STAMP_SIZE;

	//init start times
	mVideoStartTime = (VIDEO_CD_INITIAL_VBV_FULLNESS)/mStreamRate + (2+VIDEO_CD_START_DELAY)*VIDEO_CD_PACK_TIME; 
										//(above accounts for 2 system header packs)
    mAudioStartTime = mVideoStartTime + mAudioDelay;


    //allocate packet data storage
    mPacketData = new char[VIDEO_CD_PACK_SIZE];    //packsize is more then enough

    OutputSystemHeaders();

    return TRUE;
}


void CVideoCDMix::ProcessData(void)
/*
This is the main processing loop of the MPEG-1 stream mixing class.  It builds and outputs
data packets and packs until it runs out of input data.
*/
{
    CAutoLock AutoLock(&mCritSec);       //don't forget to make thread safe

    if (!mStreamStarted) {
        mStreamStarted = StartUpStream();
        if (!mStreamStarted)
            return;      //exit if inputs are not yet ready
    }
    while (1) {
        //output next pack
        if (NeedAudio()) {
            if (mAudioPin.ValidData() < VIDEO_CD_AUDIO_PACKET_DATA_SIZE)
                return;           //return if more audio data needed
            OutputAudioPack();
        }
        else {
            if (mVideoPin.ValidData() < VIDEO_CD_VIDEO_PACKET_MAX_DATA_SIZE)
                return;           //return if more video data needed
            OutputVideoPack();
        }
    }

}


void CVideoCDMix::OutputVideoPack(BOOL DisableTimeStamp/*=FALSE*/)
/*
This function formats and outputs the next pack of video data
*/
{

	//set data size params
	const int Packetsize = VIDEO_CD_VIDEO_PACK_DATA_SIZE;
	int DataSize; //= MIN_VIDEO_CD_VIDEO_PACKET_DATA_SIZE;
    char TimeStamp[VIDEO_STAMP_SIZE] ; // holds max time stamp size
	int TsBytes=0;
	// check for video start code
	int FrameBytes=mVideoPin.RemainingFrameSize();
	//output pack header;
	OutputPackHeader();

    int StuffBytes= mVideoStuffSize;
	BOOL InsertStamp =FALSE; //= (FrameBytes <= VIDEO_CD_VIDEO_PACKET_MAX_DATA_SIZE) && !DisableTimeStamp;

    if (FrameBytes <= VIDEO_CD_VIDEO_PACKET_MAX_DATA_SIZE) 
		// disable is used in the end case 
		// checking for disable here allows verifacation that there is sufficienat
		// space to insert end code in the end condition
	{
		if ((VIDEO_CD_VIDEO_PACKET_MAX_DATA_SIZE-FrameBytes)<=VIDEO_STAMP_SIZE )
		{
			StuffBytes=MAX(VIDEO_CD_VIDEO_PACKET_MAX_DATA_SIZE-FrameBytes+2,StuffBytes); 
			InsertStamp=FALSE;
		}
		else
			if (DisableTimeStamp && FrameBytes>=mVideoPin.ValidData())
				InsertStamp=FALSE;
			else
				InsertStamp=TRUE;
	}

    //copy video packet header to packet buffer
    char *PacketPtr = mPacketData;
    BuildPacketHeader(PacketPtr, VIDEO_STREAM, Packetsize);
    PacketPtr += PACKET_HEADER_SIZE;
    
    // add time stamp / stuffing
    if (InsertStamp) {
        
		AdjustStreamTime();
        TsBytes=AddVideoTimeStamp(TimeStamp,FALSE);

    }
	else
	{
		TsBytes=1;
		TimeStamp[0]=(char)0x0f;

	}
	// now calc stuffing size


	if ((StuffBytes+TsBytes) & 1)  // if size is odd
	{
		StuffBytes++;
		if (StuffBytes>MAX_VIDEO_STUFFING) StuffBytes-=2;
	}

	//calc data size
    DataSize = Packetsize - StuffBytes - TsBytes -PACKET_HEADER_SIZE;

	if (InsertStamp && DataSize<=FrameBytes) 
	{
		// too much stuffing was inserted
		int delta=(FrameBytes-DataSize+2) & 0xfffffffe;
		DataSize+=delta;
		StuffBytes-=delta; 
		// stuff size should still be legal because of initial insert stamp calulation
	}
#ifdef _DEBUG
	if (DataSize>=FrameBytes && !InsertStamp)
	{
		printf("Omitted Stamp\n");
	}
	if (DataSize<=FrameBytes && InsertStamp)
	{
		printf("Extra Stamp\n");
	}
#endif

	if (StuffBytes>3)
	{
        AddStuffing(PacketPtr, StuffBytes-2, FALSE);
        LoadStdBuffSize(&PacketPtr[StuffBytes-2], VIDEO_STREAM);
        
 	}
	else
	{
        AddStuffing(PacketPtr, StuffBytes, FALSE);
	}
    PacketPtr += StuffBytes;

	// now deal with time code
	for (int i=0;i<TsBytes;i++) *PacketPtr++=TimeStamp[i];

	


    //copy video data to packet buffer
    int BytesMoved = mVideoPin.MoveData(PacketPtr, DataSize);
   
    //zero fill to end of packet if not enough data
    int fillsize = DataSize - BytesMoved;
    if (fillsize)
	{
        ZeroMemory(PacketPtr + BytesMoved-4, fillsize+4); //kill old stop
		if (fillsize>sizeof(EndCode)+4)
		{
			*(DWORD*)(PacketPtr + DataSize-4-sizeof(EndCode))=0XB7010000;  // video stop remember byte ordering
			BuildPacketHeader(mPacketData, VIDEO_STREAM, Packetsize-sizeof(EndCode));
			memcpy(PacketPtr+DataSize-sizeof(EndCode),EndCode,sizeof(EndCode));
			mLastVideoPacketDone=TRUE;
		}
		else
			*(DWORD*)(PacketPtr + DataSize-4)=0XB7010000;
	}

	//output packet buffer
    OutputData(mPacketData, Packetsize);

    IncrementPackCount();

}


void CVideoCDMix::OutputAudioPack(void)
/*
This function formats and outputs the next pack of audio data
*/
{
	//set data size params
	int Packetsize = VIDEO_CD_AUDIO_PACK_DATA_SIZE;
	int DataSize = Packetsize - AUDIO_PACKET_HEADER_SIZE;
    
	//output pack header;
	OutputPackHeader();

    //copy audio packet header to packet buffer
    char *PacketPtr = mPacketData;
    BuildPacketHeader(PacketPtr, AUDIO_STREAM, Packetsize);
    PacketPtr += PACKET_HEADER_SIZE;
    
    // add time stamp / stuffing
    if (mAudioPin.RemainingFrameSize() < DataSize) {
	    //if new audio frame starts in this packet, output std buffer size & timestamp
        LoadStdBuffSize(PacketPtr, AUDIO_STREAM);
        int FrameNum = mAudioPin.GetNextFrameNumber();
        AddTimeStamp(&PacketPtr[STD_BUFF_DATA_SIZE], SINGLE_STAMP, AudioFrameToTime(FrameNum));
	}
    else {
		//no audio frame start, output stuffing
        AddStuffing(PacketPtr, TIME_STAMP_SIZE + STD_BUFF_DATA_SIZE);
	}
    PacketPtr += TIME_STAMP_SIZE + STD_BUFF_DATA_SIZE;

    //copy audio data to packet buffer
    int BytesMoved = mAudioPin.MoveData(PacketPtr, DataSize);

    //zero fill to end of packet if not enough data
    int fillsize = DataSize - BytesMoved + VIDEO_CD_AUDIO_FILL_SIZE;
    ZeroMemory(PacketPtr + BytesMoved, fillsize);

	//output packet buffer
    OutputData(mPacketData, Packetsize + VIDEO_CD_AUDIO_FILL_SIZE);


    IncrementPackCount();

}
#define MIN_PADDING_BYTES ((PACKET_HEADER_SIZE+2) & 0Xfffffffe)
void CVideoCDMix::OutputLastAudioPack(BOOL OutputLastFrameOnly /*=FALSE*/)
/*
This function formats and outputs the next pack of audio data
*/
{
	//set data size params
	int Packetsize = VIDEO_CD_AUDIO_PACK_DATA_SIZE;
	int DataSize = Packetsize - AUDIO_PACKET_HEADER_SIZE;
    int AudioSize = mAudioPin.ValidData();
	int PadSize,AudioPacketHeaderSize;
	if (AudioSize>DataSize && !OutputLastFrameOnly)
	{
		OutputAudioPack();
		return ;
	}
	if (OutputLastFrameOnly)
		AudioSize=mAudioPin.BytesToCopy(); 
	// max stuffing is 16 +1 end byte = 17  
	if (AudioSize+sizeof(EndCode)+16+PACKET_HEADER_SIZE >= VIDEO_CD_AUDIO_PACK_DATA_SIZE)
	{
		// stuffing is sufficent to pad out buffer
		PadSize=0;
	}
	else
	{   // calc with padding packet
		PadSize=VIDEO_CD_AUDIO_PACK_DATA_SIZE-AudioSize-sizeof(EndCode)-AUDIO_PACKET_HEADER_SIZE;
		PadSize&=0xfffffffe;// even number
		PadSize=MAX(PadSize,MIN_PADDING_BYTES);
	}
	// calc header size
	AudioPacketHeaderSize=VIDEO_CD_AUDIO_PACK_DATA_SIZE-AudioSize-sizeof(EndCode)-PadSize;
    DataSize=AudioSize;
    
	//output pack header;
	OutputPackHeader();

    //copy audio packet header to packet buffer
    char *PacketPtr = mPacketData;
    BuildPacketHeader(PacketPtr, AUDIO_STREAM, AudioSize+AudioPacketHeaderSize);
    PacketPtr += PACKET_HEADER_SIZE;
    
    // add time stamp / stuffing
    if (mAudioPin.RemainingFrameSize() < DataSize && AudioPacketHeaderSize>=AUDIO_PACKET_HEADER_SIZE) 
	{
	    //if new audio frame starts in this packet, output std buffer size & timestamp
		AddStuffing(PacketPtr,AudioPacketHeaderSize-AUDIO_PACKET_HEADER_SIZE,FALSE);
		PacketPtr+=AudioPacketHeaderSize-AUDIO_PACKET_HEADER_SIZE;
		LoadStdBuffSize(PacketPtr, AUDIO_STREAM);
		int FrameNum = mAudioPin.GetNextFrameNumber();
		AddTimeStamp(&PacketPtr[STD_BUFF_DATA_SIZE], SINGLE_STAMP, AudioFrameToTime(FrameNum));
		PacketPtr+=STD_BUFF_DATA_SIZE+TIME_STAMP_SIZE;
	}
    else {
		//no audio frame start, output stuffing
        AddStuffing(PacketPtr, AudioPacketHeaderSize-PACKET_HEADER_SIZE);
		PacketPtr += AudioPacketHeaderSize-PACKET_HEADER_SIZE;
	}

    //copy audio data to packet buffer
    int BytesMoved = mAudioPin.MoveData(PacketPtr, AudioSize); 

 //   _ASSERT(AudioSize ==BytesMoved) ;  

	//output packet buffer
    OutputData(mPacketData, AudioSize+AudioPacketHeaderSize);

	OutputPadding(PadSize);
    OutputData(EndCode, sizeof(EndCode));

    ZeroMemory(mPacketData, VIDEO_CD_AUDIO_FILL_SIZE);
    OutputData(mPacketData, VIDEO_CD_AUDIO_FILL_SIZE);
	mLastAudioPacketDone=TRUE;
    IncrementPackCount();

}

void CVideoCDMix::OutputData(void const * data, int DataSize)
/*
This function sends the specified data to the objects output file
*/
{
    unsigned long OutSize;
    WriteFile(mOutputFile, data, DataSize, &OutSize, NULL);
    if ((int)OutSize != DataSize)
        ProcessError(ERROR_FILE_WRITE);
}

void CVideoCDMix::AdjustStreamTime(void)
/*
This function is called when a new video frame is encountered.  It computes what 
the current stream time should be based on the video frame number and the VBV
buffer fullness that is specifed for the next frame. The function then adjusts the
stream time based on this calculation

This function insures that the stream time tracks the video frame time dispite any error
in the video data rate as reported by the video stream header. This is important, because
the NeedAudio() function uses stream time to sync audio and video packets.

new stream time =  frame (presentation) time of next frame
				 - stream time till next frame
				 - desired delta between stream time and presentation time (VBV delay time)
*/
{
    int FrameNum = mVideoPin.GetNextFrameNumber();
//	double TargetDelay = mVideoPin.GetNextVBVDelay() + mVBVPaddingTime;
    
#ifndef STEADY_AUDIO_RATE
	double VidioTime = VideoFrameToTime(FrameNum);
    mAdjustedStreamTime =   VidioTime
                    -    mVideoPin.RemainingFrameSize()/mStreamRate
                    -    TargetDelay;
#ifdef _DEBUG
    dprintf("frame %3d, time %8.3f, adjusted time %8.3f, diff %5.3f\n",
            FrameNum, VidioTime, mAdjustedStreamTime, VidioTime-mAdjustedStreamTime); 
#endif

#endif
    //update amount of video stuffing to mantain rate
    if (FrameNum > 10) {
        double TimeError =  VideoFrameToTime(FrameNum) -mVideoPin.GetNextVBVDelay()
                            - (mPackCount * VIDEO_CD_PACK_TIME);
        double DataError = TimeError * mStreamRate;
        if (DataError > VIDEO_CD_TARGET_VBV_FULLNESS) {
            //if buffer is overflowing, add more stuffing / send less data
            mVideoStuffSize = min(mVideoStuffSize+1, MAX_VIDEO_STUFFING);
        }
        else {
            //if buffer is underflowing, add less stuffing / send more data
            mVideoStuffSize = max(mVideoStuffSize-1, MIN_VIDEO_STUFFING);
        }
        ///*
#ifdef _DEBUG
        if ((FrameNum & 0x007f) == 0)
            dprintf("%4d %2d  %f\n",FrameNum, mVideoStuffSize, DataError);
#endif
        //*/
    }


    //end of test code*/
}


int CVideoCDMix::NeedAudio(void)
/*
This function returns true (1) if an audio packet is required.  It
returns false (0) if a video packet is required.
*/
{
	// if audio input is done, and all video buffers all full, force video processing
	if (mAudioPin.IsInputFinished() && !mVideoPin.BufferReady())
		return FALSE;
 
 	// if video input is done, and all audio buffers all full, stop audio processing
	if (mVideoPin.IsInputFinished() && !mAudioPin.BufferReady())
	{
		 return FALSE; // THIS IS BAD BUT NEEDED to prevent full buffer lockup.
	}

/*test code - stop all audio after 30 secs
    if (mAudioPin.GetNextFrameNumber() > 1148)
        return FALSE;
//end of test code*/

    double TranTime = AudioFrameToTime(mAudioPin.GetNextFrameNumber()) - mAudioTimeAhead;

#ifdef STEADY_AUDIO_RATE
    return ( (mPackCount * VIDEO_CD_PACK_TIME) >= TranTime);
#else
    return (mAdjustedStreamTime >= TranTime);
#endif
}

//the audio system header, see white book spec
const unsigned char CVideoCDMix::AudioSystemHeader[15] = {
	0x00, 0x00, 0x01, 0xbb, //start code
	0, 9,					//length of remaining data
	0x80, 0x1b, 0x91,		//padding bit, 22 rate bits = 2352*75/50, padding bit
	0x05,					//6 bits audio bound =1, fixed flag = 0, CSPS flag = 1
	0xe0,					//audio lock = 1, marker bit,video lock = 1, 5 video bound bits = 0
	0xff,					//reserved byte
	0xc0,					//audio stream id
	0xc0,					//2 marker bits, STD buffer bound scale bit =0
	32						//STD buffer bound
};

//the video system header, see white book spec
const unsigned char CVideoCDMix::VideoSystemHeader[15] = {
	0x00, 0x00, 0x01, 0xbb, //start code
	0, 9,					//length of remaining data
	0x80, 0x1b, 0x91,		//padding bit, 22 rate bits = 2352*75/50, padding bit
	0x01,					//6 bits audio bound =0, fixed flag = 0, CSPS flag = 1
	0xe1,					//audio lock = 1, marker bit,video lock = 1, 5 video bound bits = 1
	0xff,					//reserved byte
	0xe0,					//video stream id
	0xe0,					//2 marker bits, STD buffer bound scale bit =1
	46						//STD buffer bound
};

const unsigned char CVideoCDMix::EndCode[4] = {0x00, 0x00, 0x01, 0xb9};

void CVideoCDMix::OutputSystemHeaders(void)
/*
This function outputs a pack containing the video system header, and a pack
containing the audio system header
*/
{
	//output video system header
	OutputPackHeader();
	OutputData(VideoSystemHeader, sizeof(VideoSystemHeader));
	OutputPadding(VIDEO_CD_VIDEO_PACK_DATA_SIZE - sizeof(VideoSystemHeader));
    IncrementPackCount();

	//output audio system header
	OutputPackHeader();
	OutputData(AudioSystemHeader, sizeof(AudioSystemHeader));
	OutputPadding(VIDEO_CD_AUDIO_PACK_DATA_SIZE - sizeof(AudioSystemHeader));
    IncrementPackCount();
}


void CVideoCDMix::OutputPackHeader(void)
/*
This function writes a pack header to the object's stream output file.
*/
{
    SPackHeader header;

    //load sync code
    header.PackStartCode[0] = 0;
    header.PackStartCode[1] = 0;
    header.PackStartCode[2] = 1;
    header.PackStartCode[3] = (char)0xba;

    //fill in time stamp Most Burner apps seem to over write this
    AddTimeStamp(header.StreamTime, SINGLE_STAMP, mPackCount * VIDEO_CD_PACK_TIME );

    //set mux rate = 2352 * video_CD_pack_rate / 50
    header.MuxRate[0] = (char)0x80;
    header.MuxRate[1] = (char)0x1b;
    header.MuxRate[2] = (char)0x91;

    //write header to file
	OutputData(&header, PACK_HEADER_SIZE);

}

void CVideoCDMix::FlushData(void)
/*
This function is called when all input data is in the audio and video pin buffers.
This routine roemoves and precesses all remaining data to build the last packets and
packs of the mpeg-1 stream.
*/
{
    //flush video pin input and process "full packets" of data
    mVideoPin.flush();
    ProcessData();

    if (!mStreamStarted)
        return;     //can't flush if we haven't started

    //process remaining data
   int VideoSize = mVideoPin.ValidData();
   while (!mLastAudioPacketDone || VideoSize) {
        //fetch data for next packet
        if (!mLastAudioPacketDone && (NeedAudio() ))
            OutputLastAudioPack();
        else if (VideoSize)
            OutputVideoPack(TRUE);
		else 
			OutputLastAudioPack(TRUE); // Force OUTPUT
		VideoSize = mVideoPin.ValidData();
    }
#if 0
    //output audio end of stream pack
	OutputPackHeader();
    OutputPadding(VIDEO_CD_AUDIO_PACK_DATA_SIZE - sizeof(EndCode));
    OutputData(EndCode, sizeof(EndCode));
    ZeroMemory(mPacketData, VIDEO_CD_AUDIO_FILL_SIZE);
    OutputData(mPacketData, VIDEO_CD_AUDIO_FILL_SIZE);
    IncrementPackCount();
#endif
    //output video end of stream pack
	if (!mLastVideoPacketDone)
	{
		OutputPackHeader();
		OutputPadding(VIDEO_CD_VIDEO_PACK_DATA_SIZE - sizeof(EndCode));
		OutputData(EndCode, sizeof(EndCode));
		IncrementPackCount();
	}
}

void CVideoCDMix::OutputZeros(int OutSize)
/*
This function writes the specified number of bytes of zeros to the output file
*/
{
    static const char zeros[100] = {0};

    int size = min(OutSize, sizeof(zeros));
    while (size) {
        OutputData(zeros, OutSize);
        OutSize  -= size;
        size = min(OutSize, sizeof(zeros));
    }
}
