#include "stdincs.h"

#include "dtdefs.h"
#include "stdio.h"

#include "4000_reg.h"
#include "mcode.h"
#include "codec.h"
#include "math.h"


#ifdef _DEBUG
extern "C" void cdecl dprintf(LPSTR szFormat, ...);

static char* DebugMessage=__FILE__ "error %d returned from line %d\n";
static char DebugBuffer[120];

#define ReturnIfError(__func) {ECODE __ecode=__func; \
if (__ecode) {dprintf(DebugMessage,__ecode,__LINE__);return __ecode;} } 
#else
#define dprintf / ## /
#define ReturnIfError(__func) {ECODE __ecode=__func; if (__ecode) return __ecode;} 
#endif

#define CODEC
#define VIDA_BUS_16_WIDE
#define VIDB_BUS_16_WIDE
#include "IE_dmm.h"
#include "IE_api.h"

#define SIF
#include "IE_def.h"
#include "IE_vid.h"

#define VBV_SIZE		327680	     /* total VBV size */
#define INIT_VBV_FULLVAL	327680	     /* initial VBV fullness */

#define DMA_MODE_9_SCV	0x01000000
#define DMA_MODE_8_CHP	0x00000000
#define DMA_MODE_9_CHP	0x00000040

#define CODEC_STATUS_REGISTER (0x7fb0c)

extern int gMasterID;
extern int gSlaveID;

static int polarity_switch = 0;
static int dmem_clr        = TRUE;
static int pts_reg_value   = PTS_DISABLED;
static int pts_offset_value = 0;
static int pts_format_value = 0;
static int pes_data_value  = PES_DISABLED;
static int as_reg_value    = DA_DUMMY;
static int cl950_value     = FALSE;
static int interrupt_enable_value = 0;
static int freq_filt_enable = 0;
static int decimation_flag_value;
static int wide_vida_flag_value = VIDA_BUS_WIDTH / 0x10;
static int wide_vidb_flag_value = VIDB_BUS_WIDTH / 0x10;
static int scd_flag_value = 1;
static int start_gop_reset_flag_value = 1;
static int cmd_discard_thres_val = 0x01000000;	/* one hour command discard  */
static int bit_dist_clip_ctl_val = 8;		/* 1/4 bit dist clipping     */
#ifdef CODEC
static int no_stuffing_value = TRUE;    /* enable bit stuffing is default */
#endif 
static int decoder_value = 2;    /* Decode/Encode switch for I-CODEC:
				    initial value set to 2 => neutral */
/***********************************************************************
 *
 * Function to generate Quantization Matrices.   
 *
 ***********************************************************************/
#pragma warning (disable:4244) // possible data loss on double to float conversion
void CMicrocode::qmatrix(ulong qmatrix_addr, int quant_flag,
	     int intra_flag)
    {
    int	a, b, i, j;
    int	Quant[8][8];
    int	Qout[8][8];
    int	Qloop;
    static int	Q_scale[8][8];
    float	coeff1[8], coeff2[8];
  
    for(Qloop=1; Qloop <= 31; Qloop++)
	{
	for(i = 0; i <= 7; i++)
	    {
	    for(j = 0; j <= 7; j++)
		Q_scale[i][j] = Qloop;
	    }
	if (intra_flag == TRUE)
	    Q_scale[0][0] = 8;     /* Force DC Term to 8 */
    
	for(i = 0; i <= 7; i++)
	    {
	    for(j = 0; j <= 7; j++)
		{
		if (intra_flag == TRUE)
		    Quant[i][j] = IntraQuant[i][j];
		else
		    Quant[i][j] = NonIntraQuant[i][j];
		}
	    } 
    
	if (quant_flag == TRUE)
	    {
	    coeff1[0] = cos(M_PI/4);        /*  FDCT Pass 1 Cos. Multiplies. */
	    coeff1[1] = cos(M_PI/16);
	    coeff1[2] = cos(M_PI/8);
	    coeff1[3] = cos(3*M_PI/16);
	    coeff1[4] = cos(M_PI/4);
	    coeff1[5] = 2*(cos(5*M_PI/16));
	    coeff1[6] = 2*(cos(3*M_PI/8));
	    coeff1[7] = 4*(cos(7*M_PI/16));
      
	    coeff2[0] = cos(M_PI/4);        /*  FDCT Pass 2 Cos. Multiplies. */
	    coeff2[1] = 2*(cos(M_PI/16));
	    coeff2[2] = cos(M_PI/8);
	    coeff2[3] = cos(3*M_PI/16);
	    coeff2[4] = cos(M_PI/4);
	    coeff2[5] = cos(5*M_PI/16);
	    coeff2[6] = cos(3*M_PI/8);
	    coeff2[7] = 2*(cos(7*M_PI/16));
	    }

	for(a = 0; a <= 7; a++)
	    {
	    for(b = 0; b <= 7; b++)
		{
		if (quant_flag == TRUE)
		    {
		    Qout[a][b] = (int)(((8*coeff1[a]*coeff2[b]) /
			((Quant[a][b])*Q_scale[a][b]))*((1<<16)+0.5));
		    }
		else
		    {
		    Qout[a][b] = (int)(((2*Quant[a][b]*Q_scale[a][b]) *
			(256/16)) >> 2);
		    } 
		}
	    }
    
	for(i = 0; i <= 7; i=i+2)
	    {
	    for(j = 0; j <= 7; j++)
		{
		dramWrite(Dev->Regs, qmatrix_addr++,
		    (((Qout[i][j]&0xFFFF)<<16) | (Qout[i+1][j]&0xFFFF)));
		}
	    }
	}

    for(i = 0; i <= 7; i=i+2)
	{
	for(j = 0; j <= 7; j++)
	    {
	    if ((i == 0) && (j == 0))
		{
		if (intra_flag == TRUE)
		    {
		    dramWrite(Dev->Regs, qmatrix_addr++,
			(((Qout[i][j]&0xFFFF)<<16) | (Qout[i+1][j]&0xFFFF)));
		    }
		else
		    {
		    dramWrite(Dev->Regs, qmatrix_addr++,
			(((Qout[i][j]&0xFFFF)<<16) | 0x0));
		    }
		}
	    else
		dramWrite(Dev->Regs, qmatrix_addr++, 0x0);
	    }
	}
    }

CCodec::CCodec(DEVSTRUCT *dwDevice):CMicrocode(dwDevice)
{
	m_DA_CLIPX=DA_CLIPX;
	m_DA_CLIPY=DA_CLIPY;
	m_DA_PTB=DA_PTB;
	m_DA_BTB=DA_BTB;
	m_ME1_CLIP_TABLE_P=ME1_CLIP_TABLE_P;
	m_ME1_CLIP_TABLE_B=ME1_CLIP_TABLE_B;
	m_CodecMode=YUV_DECODE;
	pOpenCount=(int*)Dev->pDeviceInfo->pUserInfo->cbReserved;
	pHwConfig=(COMPRESSINFO *)(pOpenCount+1);
	if (Dev->CodecInfo.DataRate==0)
		Dev->CodecInfo.DataRate=4000000;
	if (*pOpenCount==0)
	{
		// if first open init struct size
		pHwConfig->StructSize=sizeof(COMPRESSINFO);

	}
	else 
	{
		// copy current config in 
	}
    (*pOpenCount)++;
}
 
CCodec::~CCodec()
{
	(*pOpenCount)--;
	if (*pOpenCount==0)
	{
		// last codec closed
		dprintf("Closeing Last codec\n");
		Dev->pDeviceInfo->pUserInfo->State=INITIALIZED;
	}
}


/***********************************************************************
 *
 *
 ***********************************************************************/

ECODE CCodec::Load()
{
  PREGS Regs=GET_STRUCT_PTR(Dev);

  uwide temp36;
  // special code for codec to prevent reloading too often
  if (Dev->pDeviceInfo->pUserInfo->State == CODEC_INITIALIZED)
  {
	  // check loaded mode to see if new load needed
	  return VT_NO_ERROR;
  }
  if (VIDA_FILT_MODE == 0x1)
      decimation_flag_value = TRUE;
  else
      decimation_flag_value = FALSE;
  
  VT_ResetVRP(Dev);
  VT_ResetVRP(Dev);

  /* chip counting is tricky - we use only the chips in the use_chips array,
   * but file and chip numbering for loading purposes starts at 2 and goes
   * up from there (Scorpio does not allow a chip #0 or #1) - so this count
   * is called chip_scorp.  Finally, chip_count is a counting index that goes
   * from 0 to the number of chips used in the encoding session. 
   */
  
    

    
    /* reset chip & clear IMEM - set up for initialization sequence */
    
    reset(Regs);
    imemClear(Regs);
    
    imemWrite(Regs,0x10,0x8c0);
    run_inst(Regs,32);
    imemWrite(Regs,0x10,0);
    reset(Regs);
    smemClear(Regs);
    
    /*  Enable DMA Unit (Normal, Interleaved, Refr Count = 0x100). */
    /*-------------------------------------------------------------
     *  Configure dma_mode register of VRP to set memory type
     *  8x10, 9x9, 10x8, memory organization (interleaved), and
     *  refresh count = 0x100.
     *  This code checks a software specified state
     *  to determine dram type and sets VRP
     *  accordingly.  Note customers can hard-code this if necessary.
     */
    GBusWrite(Regs,GBUS_DMAMODE,VIDTOOL_MEMCONFIG); 
    /*-------------------------------------------------------------*/

    /*  Reset Vlc_Start register. */
    GBusWrite(Regs,GBUS_VLCSTART,0x0);
    
    if(cl950_value)
    {
	/* turn on VLC unit (Px64, Intra, empty PACK & ZZ) */
	GBusWrite(Regs,GBUS_VLCCTL,0x0008010D);
	GBusWrite(Regs,GBUS_VLCCTL,0xc008010D);
    }
    else
    {
	/* turn on VLC unit (MPEG, Intra, empty PACK & ZZ) */
	GBusWrite(Regs,GBUS_VLCCTL,0x00080105);
	GBusWrite(Regs,GBUS_VLCCTL,0xc0080105);
    }

    /*  Enable CPU module (reset Ghalt, Greset), set PIO as input. */
    WBusWrite(Regs,WBUS_CPUCTL,0xc0000000);
    
    /*  Dummy VLD transfer to disable VLD. */
    WBusWrite(Regs,WBUS_CHMEM5,0x07840002);
    
    /*
     * set a breakpoint
     * run for awhile to initialize the VLD
     * then reset the Regs. 
     */
    GBusWrite(Regs,0x24,0x20);
    GBusWrite(Regs,0x24,0x0);
    
    GBusWrite(Regs,GBUS_VLDDATA,0x0);
    GBusWrite(Regs,GBUS_VLDDATA,0x0);
    GBusWrite(Regs,GBUS_VLDDATA,0x0);
    GBusWrite(Regs,GBUS_VLDDATA,0x0);

    GBusWrite(Regs,GBUS_VLDDATA,0x0);
    GBusWrite(Regs,GBUS_VLDDATA,0x0);
    GBusWrite(Regs,GBUS_VLDDATA,0x0);
    GBusWrite(Regs,GBUS_VLDDATA,0x0);
    
    GBusWrite(Regs,GBUS_VLDDATA,0x0);
    GBusWrite(Regs,GBUS_VLDDATA,0x0);
    GBusWrite(Regs,GBUS_VLDDATA,0x0);
    GBusWrite(Regs,GBUS_VLDDATA,0x0);
    
    GBusWrite(Regs,GBUS_VLDDATA,0x0);
    GBusWrite(Regs,GBUS_VLDDATA,0x0);
    GBusWrite(Regs,GBUS_VLDDATA,0x0);
    GBusWrite(Regs,GBUS_VLDDATA,0x0);
    

    /*  VLD initialized: start again. */
    reset(Regs);                             

    rmemClear(Regs);

    if(dmem_clr == TRUE) {
      dramClear(Regs);
    }
    
 
    /*-------------------------------------------------------------
     *  Initialize the ctl register on Gaia to configure P2
     *  interface which enables videoA input over P2
     *  and videoB as output over P2 (port mode 1).
     *
     *  Although scv is non-existent on Gaia, this code uses the variable,
scv_val.
     *
     *  This code is board specific!
     */
/*      gaia_rd_ctl(&scv_val);
      if(gaia_wr_ctl(scv_val | PORT_MODE_2) == -1) {
        printf("ERROR: ctl failed\n");
        exit(-1);
      }
*/
    /*-------------------------------------------------------------*/

    /* initialize rmem locations - MUST BE DONE BEFORE INITIALIZING IMEM! */
    
    temp36.loword = 0xc00000;
    temp36.hiword = 0;
    rmemWrite(Regs,22,&temp36);
    CpuWritePC(Regs, 0, 0, 0);
    
    temp36.loword = 0x800000;
    temp36.hiword = 0;
    rmemWrite(Regs,23,&temp36);
    CpuWritePC(Regs, 0, 0, 0);
    
    temp36.loword = 0;
    temp36.hiword = 0;
    rmemWrite(Regs,30,&temp36);
    CpuWritePC(Regs, 0, 0, 0);
   
    /* for E2 initialize this register to run CL4000 emulation mode */
    GBusWrite(Regs, GBUS_DMAREF, 0);

    /* Initialize initial IMEM image */
    
    /* printf("loading %s in imem # %d\n",imem_file,Regs); */
    VT_LoadFile(Dev,VR_IMEM,"ie.imb");
    
    /* Initialize initial DMEM image */
    
    /* printf("loading %s in dram # %d\n",dmem_file,Regs); */
    VT_LoadFile(Dev,VR_DRAM,"ie.dmb");
    
    /* Load the tables and do the remainder of the initialization.  If SIF 
       resolution, make sure to initialize the rc and bitrate parameters! */
    
    init_SIF();   
    if (polarity_switch) 
      imemWrite(Regs, 0xA9, 0xFC1F0321);

    
    /*  set the Regsid */
    
    GBusWrite(Regs,GBUS_HCTL,0x50001000 + (CHIP_ID << 4));
    
    /* start the Regs executing if it is not the last one in the array to be
       loaded. */
        

  /* start the last chip executing, then exit this program. */
  GBusWrite(Regs, GBUS_HCTL, 0xf0000000);
  Sleep(10);
//  DWORD data[2];
//  data[0] = 1; data[1] = 2;
//  LoadParams(TC_NOW, 2, 2, data); 
    CrChange(Dev->Regs,CTRL1,0xff,0x10);
	CrChange(Dev->Regs,CTRL2,FIFO_MODE_MASK,FIFO_MODE_RECODE);
	// set up mode to match configuration
	pHwConfig->DataRate=Dev->CodecInfo.DataRate;
	pHwConfig->FrameType=0;
	pHwConfig->Quality=0;
	pHwConfig->FrameHeight=240;
	pHwConfig->FrameWidth=352;
	pHwConfig->Padding=no_stuffing_value;
	
	// finish configuration
    UpdateSettings();

    Dev->pDeviceInfo->pUserInfo->State = CODEC_INITIALIZED;
    return(VT_NO_ERROR);
}


/***********************************************************************
 *
 * Initialize data arrays in DRAM
 *
 ***********************************************************************/
 ECODE CCodec::init_SIF()
{
  PREGS Regs=GET_STRUCT_PTR(Dev);
    ulong bps_scaled;
    int i;

#ifdef CODEC /*------------------------------------------------------*/  
    /* load default DQ tables */
    VT_LoadFileBin(Dev,VR_DRAM,"dqtables.bii", D_IQM_Tbls>>2, 
		 (D_IQM_Tbls>>2) + 2*32*31-1, 0);


#endif /* CODEC -----------------------------------------------------*/

#ifdef CLIP_TABLES_FROM_FILE
   /*
    *-------------------------------------------------------------------------
    * The default is now to generate the clipping tables internally with
    * existing code.  If it is more advantageous in your system to load from
    * a file, #define CLIP_TABLES_FROM_FILE.
    *-------------------------------------------------------------------------
    */
    loadDram_bin(Regs, "tables/BtabSIF.dt",  DA_BTB,
		 DA_BTB + DA_BTB_SIZE - 1, 0);
    loadDram_bin(Regs, "tables/PtabSIF.dt",  DA_PTB,
		 DA_PTB + DA_PTB_SIZE - 1, 0);
    loadDram_bin(Regs, "tables/clpxSIF.dt",  DA_CLIPX,
		 DA_CLIPX + DA_CLIPX_SIZE - 1, 0);
    loadDram_bin(Regs, "tables/clpySIF.dt",  DA_CLIPY,
		 DA_CLIPY + DA_CLIPY_SIZE - 1, 0);
    loadDram_bin(Regs, "tables/clpBSIF.bin", ME1_CLIP_TABLE_B,
		 ME1_CLIP_TABLE_B + ME1_CLIP_TABLE_SIZE - 1, 0);
    loadDram_bin(Regs, "tables/clpPSIF.bin", ME1_CLIP_TABLE_P,
		 ME1_CLIP_TABLE_P + ME1_CLIP_TABLE_SIZE - 1, 0);
#else 	/* no clause #ifdef CLIP_TABLES_FROM_FILE	*/
   /*
    *-------------------------------------------------------------------------
    * Generate default clipping table for NTSC.
    *-------------------------------------------------------------------------
    */
    loadClippingTables(22, 15);    /* load NTSC clipping tables */
#endif	/* end #ifdef CLIP_TABLES_FROM_FILE	*/

   /*
    *-------------------------------------------------------------------------
    * Generate or load quant tables and send them to the encoder.
    * Frequency-domain-filtering tables (used rarely) must be loaded from a
    * file, but regular quant tables can be generated internally.
    *-------------------------------------------------------------------------
    */
    if (freq_filt_enable)
        {
	printf("loading frequency domain filtering tables \n");
	Dev->LoadFileBinSwap(VR_DRAM, "qf_SIF.int", QUANT_TABLES_INTRA,
		     QUANT_TABLES_INTRA + TOTAL_QUANT_TABLE_SIZE - 1, 0);
	Dev->LoadFileBinSwap(VR_DRAM, "qf_SIF.non", QUANT_TABLES_NONINTRA,
		     QUANT_TABLES_NONINTRA + TOTAL_QUANT_TABLE_SIZE - 1, 0);
	Dev->LoadFileBinSwap(VR_DRAM, "dq_SIF.int",  DQUANT_TABLES_INTRA,
		     DQUANT_TABLES_INTRA + TOTAL_QUANT_TABLE_SIZE - 1, 0);
	Dev->LoadFileBinSwap(VR_DRAM, "dq_SIF.non",  DQUANT_TABLES_NONINTRA,
		     DQUANT_TABLES_NONINTRA + TOTAL_QUANT_TABLE_SIZE - 1, 0);
        }
    else
        {
#ifdef QUANT_TABLES_FROM_FILE
       /*
	*---------------------------------------------------------------------
	* The default is now to generate the quant tables internally with
	* existing code.  If it is more advantageous in your system to load
	* from a file, #define QUANT_TABLES_FROM_FILE.
	*---------------------------------------------------------------------
	*/
	loadDram_bin(Regs, "tables/q_SIF.int", QUANT_TABLES_INTRA,
		     QUANT_TABLES_INTRA + TOTAL_QUANT_TABLE_SIZE - 1, 0);
	loadDram_bin(Regs, "tables/q_SIFnf.non", ALT_QUANT_TABLES_NONINTRA,
		     ALT_QUANT_TABLES_NONINTRA + TOTAL_QUANT_TABLE_SIZE - 1, 0);
	loadDram_bin(Regs, "tables/dq_SIF.int",  DQUANT_TABLES_INTRA,
		     DQUANT_TABLES_INTRA + TOTAL_QUANT_TABLE_SIZE - 1, 0);
        loadDram_bin(Regs, "tables/dq_SIFnf.non", ALT_DQUANT_TABLES_NONINTRA,
		     ALT_DQUANT_TABLES_NONINTRA + TOTAL_QUANT_TABLE_SIZE - 1, 0);
#else	/* no clause #ifdef QUANT_TABLES_FROM_FILE	*/
       /*
	*---------------------------------------------------------------------
	* Generate default quant tables and send to chip.  Non-intra tables
	* are not the MPEG standard, and so are sent to the ping-ponged
	* buffer.  The encoder is flagged via a command sent through the API
	* in main().
	*---------------------------------------------------------------------
	*/
	qmatrix( QUANT_TABLES_INTRA, TRUE, TRUE);
	qmatrix( DQUANT_TABLES_INTRA, FALSE, TRUE);
	qmatrix( ALT_QUANT_TABLES_NONINTRA, TRUE, FALSE);
	qmatrix( ALT_DQUANT_TABLES_NONINTRA, FALSE, FALSE);
#endif	/* end #ifdef QUANT_TABLES_FROM_FILE	*/
        }

   /*
    *-------------------------------------------------------------------------
    * Load mathematical lookup tables.
    *-------------------------------------------------------------------------
    */
    VT_LoadFileBin(Dev,VR_DRAM,"cos_SIF.tai", COS_TABLE,
		 COS_TABLE + COS_TABLE_SIZE - 1, 0);
    VT_LoadFileBin(Dev,VR_DRAM, "fvbfvbs.bii", DA_F_VBS_VBF,
                 DA_F_VBS_VBF + DA_F_VBS_VBF_SIZE - 1, 0);

   /*
    *-------------------------------------------------------------------------
    * Load VLC table.
    *-------------------------------------------------------------------------
    */
    VT_LoadFileBin(Dev,VR_DRAM, "vlc_SIF.tai", VLC_TABLES,
		 VLC_TABLES + VLC_TABLES_SIZE - 1, 0);

   /*
    *-------------------------------------------------------------------------
    * Write bitrate parameters into DRAM  - MUST be done after code is loaded!
    *-------------------------------------------------------------------------
    */
	if (Dev->CodecInfo.DataRate==0)
		bps_scaled=8000000/400;
	else
		bps_scaled = Dev->CodecInfo.DataRate/400;
    dramWrite(Regs , BITS_PER_SEC_DIV_400 >> 2, bps_scaled);
    dramWrite(Regs , INIT_VBV_FULLNESS    >> 2, INIT_VBV_FULLVAL);
    dramWrite(Regs , HDR_VBV_SIZE_LOC     >> 2, VBV_SIZE);

    /* load the polarity bit with the desired polarity. */
    if (polarity_switch)
      dramWrite(Regs, POLARITY_BIT, 1);   /* use the alternate polarity */
    else
      dramWrite(Regs, POLARITY_BIT, 0);   /* use the default polarity */
    
    /* Initialize the interrupt enable locations */
    for (i=0; i< INTERRUPT_ENABLE_SIZE; i++) 
    	dramWrite(Regs , (INTERRUPT_ENABLE_BLOCK + i), interrupt_enable_value); 
	
    /* Setup Memory locations for audio_start_register */
    dramWrite(Regs , DA_AUDIO_START_REG>>2, as_reg_value); 

    /* Setup Memory location for pts register */
    dramWrite(Regs , DA_PTS_REG_ADDR>>2, pts_reg_value); 

    /* Setup Memory location for pts offset   */
    dramWrite(Regs, DA_PTS_OFFSET >> 2, pts_offset_value);

    /* Setup Memory location for pts format	*/
    dramWrite(Regs, DA_PTS_FORMAT >> 2, pts_format_value);

    /* Setup Memory location for PES data	*/
    dramWrite(Regs, DA_PES_DATA >> 2, pes_data_value);

    /* Setup Memory location for decimation flag	*/
    dramWrite(Regs, DA_DECIMATION_FLAG >> 2, decimation_flag_value);

    /* Setup Memory location for wide video A flag	*/
    dramWrite(Regs, DA_WIDE_VIDA_FLAG >> 2, wide_vida_flag_value);

    /* Setup Memory location for wide video B flag	*/
    dramWrite(Regs, DA_WIDE_VIDB_FLAG >> 2, wide_vidb_flag_value);

    /* Setup Memory location for scene change detection control	*/
    dramWrite(Regs, DA_SCENE_CHANGE_CONTROL >> 2, 
        (start_gop_reset_flag_value << 1) | scd_flag_value);

    /* Setup Memory location for command discard threshold */
    dramWrite(Regs, DA_DISCARD_THRESHOLD >> 2, cmd_discard_thres_val);

    /* Setup Memory location for bit distribution clipping control */
    dramWrite(Regs, DA_BIT_DIST_CLIP_CONTROL >> 2, bit_dist_clip_ctl_val);

#ifdef CODEC
    /* Setup Memory location for stuffing flag */ 
    dramWrite(Regs, DA_STUFFING_FLAG >> 2, no_stuffing_value);    
#endif
    /* Setup Memory location of I-Codec mode flag */
    dramWrite(Regs, DA_DECODER_FLAG >> 2, decoder_value);

   /*
    *-------------------------------------------------------------------------
    * Initialize any DRAM loactions indicated by the config.sys file.  This
    * will override command-line settings.  Config.sys is not required to
    * exist.
    *-------------------------------------------------------------------------
    */
//    switch(InitDramFromFile(Regs, "config.sys")) {
//    case 2:
//      printf ("Syntax error in config.sys\n");
//      break;
      
//    case 3:
//      printf ("Dram address out of range in config.sys\n");
//      break;
//    }
    
    /* Initialize ME MemCtl */
    GBusWrite(Regs, GBUS_MEMEMCTL, 0xc0000040L);    

    /* Clear Host semaphore bit */
    GBusWrite(Regs, GBUS_SEMA, 0x0L);               

    /* Disable all INTs. */    
    WBusWrite(Regs, WBUS_INTCTL, 0xfffefffeL);      
   
    /* Reset Mul_Cntl Reg (clip 8) */
    /* Note: the setting of MULCTL will be overwritten by the ucode */
    /* so the setting here is don't care */
    WBusWrite(Regs,WBUS_MULCTL,0x4);             
    
    /*-------------------------------------------------------------
     *  Configure dma_mode register of VRP to set memory type
     *  8x10, 9x9, 10x8, memory organization (interleaved), and
     *  refresh count = 0x100.
     *  This code checks a software specified state
     *  to determine dram type and sets VRP
     *  accordingly.  Note customers can hard-code this if necessary.
     */
     GBusWrite(Regs,GBUS_DMAMODE,VIDTOOL_MEMCONFIG); 
   /*-------------------------------------------------------------*/

    /* Reset Vlc_Start register. */
    GBusWrite(Regs,GBUS_VLCSTART,0x0);

    if(cl950_value)
    {
	/* turn on VLC unit (Px64, Intra, empty PACK & ZZ) */
	GBusWrite(Regs,GBUS_VLCCTL,0x0008010D);
	GBusWrite(Regs,GBUS_VLCCTL,0xc008010D);
    }
    else
    {
	/* turn on VLC unit (MPEG, Intra, empty PACK & ZZ) */
	GBusWrite(Regs,GBUS_VLCCTL,0x00080105);
	GBusWrite(Regs,GBUS_VLCCTL,0xc0080105);
    }

    /* Zero out VLC Pack register, 
     * enable CPU module (reset Ghalt, Greset), set PIO as input, 
     */
    WBusWrite(Regs,WBUS_CPUPACK,0x0);
    WBusWrite(Regs,WBUS_CPUCTL,0xc0000000);

    /* enable smem, refresh and VLC */
    WBusWrite(Regs,WBUS_INTCTL,0x00010201);
 
    /* setup VLC DMA */
    WBusWrite(Regs,WBUS_CHMEM5,0xff980003); 
	return (VT_NO_ERROR);
}

ECODE CCodec::UpdateSettings()
  // will update all settings if needed on running microcode
{
/* 
	pHwConfig->DataRate=Dev->CodecInfo.DataRate;
	pHwConfig->FrameType=0;
	pHwConfig->Quality=0;
	pHwConfig->FrameHeight=240;
	pHwConfig->FrameWidth=352;
	pHwConfig->Padding=no_stuffing_value;
*/
  if (pHwConfig->FrameType!=Dev->CodecInfo.FrameType)
	{
		VRP_COMMAND Cmd={2,3,0xffffffff,0};
		
		Cmd.params[1]=Dev->CodecInfo.FrameType;
		ReturnIfError(VT_SendCommand(Dev,&Cmd));
		dprintf("Frametype changed to %d\n",Dev->CodecInfo.FrameType);
		pHwConfig->FrameType=Dev->CodecInfo.FrameType;
	}
	// QUALITY	and bitrate for encode only
	if (m_CodecMode==YUV_DECODE || m_CodecMode==RGB_DECODE) 
		return VT_NO_ERROR;
	if  (pHwConfig->Quality!=Dev->CodecInfo.Quality || pHwConfig->DataRate!=Dev->CodecInfo.DataRate)
	{
			VRP_COMMAND Cmd={8,7,0xffffffff,3,1,8000000/400,4,0};
			if (Dev->CodecInfo.DataRate==0)
			{// use quality
				Cmd.params[5]=CalcQuant(pHwConfig->Quality);
			}
			else
				Cmd.params[3]=Dev->CodecInfo.DataRate/400;
			ReturnIfError(VT_SendCommand(Dev,&Cmd));
			dprintf("Quality set to %d Datarate %d\n",Dev->CodecInfo.Quality,Dev->CodecInfo.DataRate);
			pHwConfig->Quality=Dev->CodecInfo.Quality;
			pHwConfig->DataRate=Dev->CodecInfo.DataRate;
	}
	return VT_NO_ERROR;

}

ECODE CCodec::Start()
{
	VRP_COMMAND Cmd={0,3,TC_NOW,0xffffffff};
	UpdateSettings();
	//	VRP_COMMAND Cmd={0,2,0xffffffff};
	switch (m_CodecMode)
	{
	case YUV_ENCODE:
		Cmd.command=3;
		break;
	case YUV_DECODE:
		Cmd.command=4;
		break;
	case RGB_ENCODE:
	case RGB_DECODE:
	default :
		return VT_ERR_UNSUPPORTED;
	}
//    CrChange(Dev->Regs,CTRL1,0xff,0x10);
//	CrChange(Dev->Regs,CTRL2,FIFO_MODE_MASK,FIFO_MODE_RECODE);
	// disable master until data written
//	Change32(&Dev->Regs->Csr,MSTR_MODE_MASK | STOP_1DONE_IN, 0);
///	Write32(&Dev->Regs->Status,DONE_FLAG_1| DONE_FLAG_2);

//	ECODE ec=VT_SendPriorityCommand(Dev,&Cmd);
	ECODE ec=VT_SendCommand(Dev,&Cmd);
//	Sleep(5);
//	GBusChange(Dev->Regs,GBUS_VACTL,0X10,0X10);
	return ec;

}

ECODE CCodec::Stop()
{
	Change32(&Dev->Regs->Csr,MSTR_INPUT , 0);
	Change32(&Dev->Regs->Csr,STOP_1DONE_IN, 0);
	return VT_NO_ERROR;
}

ECODE CCodec::SetParam(DWORD param,long  value)
{
	switch (param)
	{
	case CODECMODE:
	
		switch (value)
		{
		case YUV_ENCODE:
		case YUV_DECODE:
			m_CodecMode=value;
			break;
		case RGB_ENCODE:
		case RGB_DECODE:
		default :
			return VT_ERR_UNSUPPORTED;
		}
		break;
	case DATARATE:
		{
			Dev->CodecInfo.DataRate=value;
//			Dev->CodecInfo.Quality
		}
		break;
	case QUALITY:
		{
			Dev->CodecInfo.Quality=value;
			Dev->CodecInfo.DataRate=0;
		}
		break;

	default: return CMicrocode::SetParam(param,value);
	}
	return VT_NO_ERROR;
}
//#define PadSize 0x2000
#define BLOCK_SIZE 0x400
#ifdef _DEBUG
ECODE CheckMasterErr(volatile PREGS Regs)
{
	if (Regs->Csr & 0x10)
	{
		OutputDebugString("Master Abort\n");
		return 0xDEADDEAD;
	}
	else 
		return VT_NO_ERROR;
}
#else
#define CheckMasterErr(_x) VT_NO_ERROR
#endif

#define STOP_CODE 0xb7010000
#if 1
ECODE CCodec::WriteData(const DWORD *SrcPtr, DWORD Words, DWORD *WordsWritten)
{
//    CrChange(Dev->Regs,CTRL1,0xff,0x10);
// 	CrChange(Dev->Regs,CTRL2,FIFO_MODE_MASK,FIFO_MODE_RECODE);

//   Disable dma and clear Done Flags
	DWORD PadSize;
	ReturnIfError(CheckMasterErr(Dev->Regs));
	Change32(&Dev->Regs->Csr,MSTR_MODE_MASK ,0);
	ReturnIfError(CheckMasterErr(Dev->Regs));
	// empty out fifo
	while ((Dev->Regs->Csr & FIFO_EMPTYN)) 
		PadSize=Dev->Regs->FifoData;
	Write32(&Dev->Regs->Status,DONE_FLAG_1| DONE_FLAG_2 |FIFO_OVERFLOW);
	ReturnIfError(CheckMasterErr(Dev->Regs));
	// init input buffer
	*WordsWritten=Words;
	if (Dev->FB_LinAddr!=SrcPtr)
		CopyMemory(Dev->FB_LinAddr,SrcPtr,Words*sizeof(DWORD));	
	if (m_CodecMode==YUV_DECODE)
	{
		m_Buffer1Size=(Words<0x700? 0x800 :((Words+32)/BLOCK_SIZE+1)*BLOCK_SIZE);
		Dev->FB_LinAddr[Words++]=STOP_CODE; // add one word
	}
	else
	{
		m_Buffer1Size=Words;// +0x100; //(Words/BLOCK_SIZE+1)*BLOCK_SIZE;
	}
	PadSize=m_Buffer1Size-Words;
	memset(Dev->FB_LinAddr+Words,0,PadSize*sizeof(DWORD));
	VT_LoadDma(Dev,MASTER1,m_Buffer1Size,Dev->FB_PhysAddr);
	m_Buffer2Phys=Dev->FB_PhysAddr+m_Buffer1Size*sizeof(DWORD);
	VT_LoadDma(Dev,MASTER2,Dev->FB_Size-m_Buffer1Size,m_Buffer2Phys);
	// now enable the master
	//Clear master 1 done
	// Wait for local ?
//	while (Dev->Regs->Csr & VRP_LOCAL)
//		Sleep(1);
#if 0 // don't bother waiting here 
	if (Dev->Regs->Csr &  VRP_LOCAL)
	{
		for (int i=0;i<200;i++)
		{
			Sleep(2);
			if (!(Dev->Regs->Csr &  VRP_LOCAL))
				break;
		}
		if (Dev->Regs->Csr &  VRP_LOCAL)
			return VT_ERR_LOCAL_TIMEOUT;
	}
#endif
	Change32(&Dev->Regs->Csr,MSTR_MODE_MASK | STOP_1DONE_IN,MSTR_INPUT | MSTR_OUTPUT |STOP_1DONE_IN);
//	Change32(&Dev->Regs->Csr, STOP_1DONE_IN,STOP_1DONE_IN); // now enable stop
	ReturnIfError(CheckMasterErr(Dev->Regs));
//	Write32(&Dev->Regs->Status,DONE_FLAG_1| DONE_FLAG_2);
//	ReturnIfError(CheckMasterErr(Dev->Regs));
//	Write32(&Dev->Regs->Status,DONE_FLAG_1);
	return VT_NO_ERROR;
}
#else
// version of WriteData that does not use master input
ECODE CCodec::WriteData(const DWORD *SrcPtr, DWORD Words, DWORD *WordsWritten)
//   Disable dma and clear Done Flags
{
	DWORD PadSize;
	ReturnIfError(CheckMasterErr(Dev->Regs));
	Change32(&Dev->Regs->Csr,MSTR_MODE_MASK ,0);
	ReturnIfError(CheckMasterErr(Dev->Regs));
	Write32(&Dev->Regs->Status,DONE_FLAG_1| DONE_FLAG_2 |FIFO_OVERFLOW);
	ReturnIfError(CheckMasterErr(Dev->Regs));
	// init input buffer
	*WordsWritten=Words;
	if (m_CodecMode==YUV_DECODE)
	{
		m_Buffer1Size=((Words+32)/BLOCK_SIZE+1)*BLOCK_SIZE;
//		Dev->FB_LinAddr[Words++]=STOP_CODE; // add one word
	}
	else
	{
		m_Buffer1Size=((Words-1)/BLOCK_SIZE+1)*BLOCK_SIZE;
	}
	PadSize=m_Buffer1Size-Words;
	m_Buffer2Phys=Dev->FB_PhysAddr+m_Buffer1Size*sizeof(DWORD);
	VT_LoadDma(Dev,MASTER2,Dev->FB_Size-m_Buffer1Size,m_Buffer2Phys);
//	while (Dev->Regs->Csr & VRP_LOCAL)
//		Sleep(1);
	Change32(&Dev->Regs->Csr,MSTR_MODE_MASK, MSTR_OUTPUT |STOP_1DONE_IN);
	ReturnIfError(CheckMasterErr(Dev->Regs));
	// Now Write in the Data

    PREGS bd_addr=GET_STRUCT_PTR(Dev);
	DWORD i;
	int timeout;
	if (bd_addr->Csr &  VRP_LOCAL)
	{
		for (i=0;i<100;i++)
		{
			Sleep(1);
			if (!(bd_addr->Csr &  VRP_LOCAL))
				break;
		}
		if (bd_addr->Csr &  VRP_LOCAL)
			return VT_ERR_LOCAL_TIMEOUT;
	}
	for (i=0;i<Words;i++)
	{
		bd_addr->PortAIn=*SrcPtr++;
		timeout=0;
		while (bd_addr->Csr & (PORTA_FLAG | VRP_LOCAL))
		{
			if (bd_addr->Csr &  VRP_LOCAL)
				Sleep(1);
		    if (timeout++>IREG_WRITE_TIMEOUT) 
				return VT_ERR_LOCAL_TIMEOUT;
		}
			
	}
	if (m_CodecMode==YUV_DECODE)
	{
		PadSize--;
		bd_addr->PortAIn=STOP_CODE;
		timeout=0;
		while (bd_addr->Csr & (PORTA_FLAG | VRP_LOCAL))
		{
			if (bd_addr->Csr &  VRP_LOCAL)
				Sleep(1);
		    if (timeout++>IREG_WRITE_TIMEOUT) 
				return VT_NO_ERROR;
		}
	}
	for (i=0;i<10000;i++)  // pad untill local sets
	{
		bd_addr->PortAIn=0;
		timeout=0;
		while (bd_addr->Csr & (PORTA_FLAG | VRP_LOCAL))
		{
			if (bd_addr->Csr &  VRP_LOCAL)
				Sleep(0);
		    if (timeout++>3) 
				return VT_NO_ERROR;
		}
			
	}
	return VT_NO_ERROR;
}
#endif

ECODE CCodec::ReadData(void **DestPtr, DWORD *WordsRead)
{
	DWORD tmp;
	if (Dev->Regs->Status & FIFO_OVERFLOW)
		return VT_ERR_FIFOOVERFLOW;
	dramRead(Dev->Regs,CODEC_STATUS_REGISTER,&tmp);
	if (tmp !=0) 
	{
		// current codec busy statement
		*WordsRead=0;
		return VT_NO_ERROR;
	}
	*DestPtr=Dev->FB_LinAddr+m_Buffer1Size;
	VT_GetCurrentTransferAddress(Dev,MASTER2,&tmp);
	*WordsRead=(tmp-m_Buffer2Phys) >> 2;
	if (*WordsRead==0) return VT_NO_ERROR;
#if 0
	char *sp=(char*)*DestPtr;
	char *cp=sp;
	*WordsRead>>=1;
	for (DWORD i=0;i<*WordsRead;i++)
	{
		sp++;
		*cp++=*sp++;
	}
#endif	
	
	return VT_NO_ERROR;
}
