/* Copyright C-Cube Microsystems 1995	*/
/*	@(#)video_def.h	35.3 8/2/95 	*/
/*************************************************************************
 *
 * Filename:  video_def.h
 * Author:  Stewart Kelly
 * Purpose:  definitions for video ucode
 * Date:  1/5/93
 *
 * V. Wilburn	24-May-95	Added defines for source format.
 *************************************************************************/

#ifdef VIDA_BUS_16_WIDE
#define VIDA_BUS_WIDTH  0x10
#else	/* 8 bit wide video = SCORPIO */
#define VIDA_BUS_WIDTH  0x0
#endif

#ifdef VIDB_BUS_16_WIDE
#define VIDB_BUS_WIDTH  0x10
#else	/* 8 bit wide video = SCORPIO */
#define VIDB_BUS_WIDTH  0x0
#endif

/* VACTL bits */
#define VA_LST_RUN	(3 << 30)
#define VIE		(1 << 29)
#define LUMA_ONLY	(0x4)

/*
 *----------------------------------------------------------------------------
 * Defines for source format.
 *----------------------------------------------------------------------------
 */
#define PAL_FORMAT 	(0)
#define NTSC_FORMAT	(1)

#ifdef	R601
#if CHIP_NUM == 3
#define	SCHED_SWITCH_LINE	86
#else
#define	SCHED_SWITCH_LINE	85
#endif
#endif

#ifdef SIF
#define	SCHED_SWITCH_LINE	1028
#endif

#define	NUM_VB_GUARD_LINES	3
#define	VLINE_OBITS_THRESH	(SCHED_SWITCH_LINE - NUM_VB_GUARD_LINES)

/*
 * these values are the number of video lines per field, per chip
 */

#ifdef CODEC
#define VIDLINES_PER_CHIP       (15 * 16)	/* Grab whole field */
#define PAL_VIDLINES_PER_CHIP	(18 * 16)	/* MB_ROWS */
#else /* CODEC */
#if CHIP_NUM == 0
#define VIDLINES_PER_CHIP       (7 * 16)	/* MB_ROWS */
#else  /* CHIP_NUM == 1 */
#define VIDLINES_PER_CHIP       (8 * 16)
#endif
#define PAL_VIDLINES_PER_CHIP	(9 * 16)	/* MB_ROWS */
#endif /* CODEC */


#ifdef SINGLE_CHIP
#define	START_VIDEO_LINE	(0)	/* Grab whole field */
#else
#define	START_VIDEO_LINE	((CHIP_NUM) * (7) * (16))
#endif

/* video line # at first Vsync */
#define	F1_VSYNC_LINE		4

/* According to "new" numbering scheme:
 *	Event	Old#	New#
 *	VSYNC	4	1008
 *	Act V	20	0
 *	VSYNC	266	246
 *	ACT V	283	263
 *	END V	525	505
 * The last visible line is about 503.
 * The Master acquires through the last visible line, so it
 * needs to wait to the end, but the slave is done acquiring
 * half way through the 2nd field, so it can leave sooner.
 */
#if CHIP_NUM == 1	/* BOTTOM CHIP */
#define LAST_VISIBLE_LINE	503
#else
#define LAST_VISIBLE_LINE	480
#endif

/*
 * these two values create an interlaced frame in DRAM
 * as used by 601
 *
 * the values 4 for LINE_TO_LINE_INC is necessary (but
 * not sufficient) to create two fields stored in DRAM
 */
#define	MB_ROW_INC_MASK		((64 / LINE_TO_LINE_INC) - 1)
#define	MB_ROW_INC		((NUM_WDS_PER_LINE * 16 * 4) - 64)

#define	MB_ODD_ROW_INC		(((32+352)*16) - 64)
#define	MB_EVEN_ROW_INC		((2*352*16) - 64)
#define ODD_FIELD_NTSC_JUNK_INCREMENT   (128 << 2)
#define EVEN_FIELD_NTSC_JUNK_INCREMENT  (0)
#define	ODD_FIELD_NTSC_VAFIFO_LINE_VAL	(32 << 12) /* 32 bytes = 8 words */
#define	EVEN_FIELD_NTSC_VAFIFO_LINE_VAL	(0 << 12)   /* 0 bytes = 0 words */

#define	MB_ODD_23WD_ROW_INC		(((16+368)*16) - 64)
#define	MB_EVEN_23WD_ROW_INC		(((32+2*368)*16) - 64)
#define ODD_FIELD_23WD_JUNK_INCREMENT   (64 << 2)
#define EVEN_FIELD_23WD_JUNK_INCREMENT  (128 << 2)
#define	ODD_FIELD_23WD_VAFIFO_LINE_VAL	(16 << 12) /* 16 bytes = 4 words */
#define	EVEN_FIELD_23WD_VAFIFO_LINE_VAL	(32 << 12)   /* 32 bytes = 8 words */

#define NTSC_ABS_VIDEO_COUNTER_INIT	(1008)
#define PAL_ABS_VIDEO_COUNTER_INIT	(1002)


#ifdef	TESTBED
#define	sched_done		r4
#define tmp_reg			r10
#define jpc_reg			r11
#define	err_cnt			r12
#define cnt			r14
#define tmp_reg4		r15
#define tmp_reg2		r16
#define tmp_reg3		r20
#define loop_cnt		r21
#endif

#define	vb_ctl_r24		r24
#define	hi_tmp_r24		r24
#define	vbi_tmp_r24		r24
#define	vai_tmp_r24		r24
#define	irq0_tmp_r24		r24
#define	hi_tmp_r25		r25
#define	vbi_tmp_r25		r25
#define	vai_tmp_r25		r25
#define	irq0_tmp_r25		r25
#define	hi_tmp_r26		r26
#define	vbi_tmp_r26		r26
#define	vai_tmp_r26		r26
#define	irq0_tmp_r26		r26
#define	hi_tmp_r27		r27
#define	vbi_tmp_r27		r27
#define	vai_tmp_r27		r27
#define	irq0_tmp_r27		r27
#define	hi_tmp_r31		r31
#define	vbi_tmp_r31		r31
#define	vai_tmp_r31		r31
#define	irq0_tmp_r31		r31

#define MAX_XFER_BITS		9
#define MAX_XFER_SIZE		(1 << MAX_XFER_BITS)

#define WORDS	(10*MAX_XFER_SIZE + 16)

#define	MIN_XFER_SZ	0x010
#define	XFER_SZ_MASK	0x00000fff
#define	OUTPUT_SRC_ADDR	(0x00010 << 2)
#define	INPUT_DST_ADDR	(0x04010 << 2)

/*************************************************************************
 *
 * C-Cube Microsystems
 *
 *************************************************************************/
