// CapAbout.cpp : implementation file
//

#include "stdafx.h"
#include "bwdlgs.h"
#include "bwdialog.h"
#include "CapAbout.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define MAX_PATHNAME 256
#define MAX_REGISTRY_PATH 256

#define STR_BROADWAY_HARDWARE_SUBKEY "SOFTWARE\\DataTranslation\\Broadway\\HardwareOptions"
#define STR_AUDIO_CAL_FACTOR        "AudioCalFactor"
#define STR_DRIVER_INSTALL			"DriverInstallComplete"


/////////////////////////////////////////////////////////////////////////////
// CCapAbout dialog


CCapAbout::CCapAbout(CWnd* pParent, BW_PROPERTYOPTIONS *pOptions)
	: CDialog(CCapAbout::IDD, pParent)
{
	//{{AFX_DATA_INIT(CCapAbout)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT

	m_pOptions = pOptions;
}


void CCapAbout::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CCapAbout)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CCapAbout, CDialog)
	//{{AFX_MSG_MAP(CCapAbout)
	ON_WM_CTLCOLOR()
	ON_BN_CLICKED(IDC_CALIBRATE, OnCalibrate)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCapAbout message handlers

BOOL CCapAbout::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	if (m_pOptions != NULL) {

		CString	rString;

		GetDlgItemText(IDC_SERIALNUMBER, rString);
		rString = rString + "  " + m_pOptions->szSerialNumber; 
		SetDlgItemText(IDC_SERIALNUMBER, rString);

		GetDlgItemText(IDC_DRIVERVERSION, rString);
		rString = rString + "  " + m_pOptions->szFileVersion; 
		SetDlgItemText(IDC_DRIVERVERSION, rString);

		GetDlgItemText(IDC_BOARDREVISION, rString);
		rString = rString + "  " + m_pOptions->szHardwareRevision; 
		SetDlgItemText(IDC_BOARDREVISION, rString);

		SetDlgItemText(IDC_FILEDESCRIPTION, m_pOptions->szFileDescription);
   		
		rString = "About ";
		rString += m_pOptions->szFileDescription; 
   		SetWindowText (rString);
	}

#if 0
	
    DWORD dwResult = 0, Type, Length, lRet;
    HKEY hkOptions;

    /* See if we can get a registry handle to our settings data */
    lRet = RegOpenKey(HKEY_LOCAL_MACHINE, STR_BROADWAY_HARDWARE_SUBKEY, &hkOptions);
    if (lRet  == ERROR_SUCCESS) {

        /* try to get the current driver install setting */
        Length = sizeof(dwResult);  
        lRet = RegQueryValueEx(hkOptions
                               ,STR_DRIVER_INSTALL
                               ,NULL
                               ,&Type
                               ,(PUCHAR)&dwResult
                               ,&Length
                               );

        RegCloseKey(hkOptions);

	    if (lRet  != ERROR_SUCCESS || Type != REG_DWORD) {
			dwResult = 0;
		}
	}


	/* 
	 * if the driver has not yet been installed, do 
	 * audio/video calibration and mark it as installed
	 */
	if (!dwResult) {	
		
		(GetDlgItem(IDC_CALIBRATE_GROUP))->ShowWindow(SW_HIDE);
		(GetDlgItem(IDC_CALIBRATE_TEXT))->ShowWindow(SW_HIDE);
		(GetDlgItem(IDC_CALIBRATE))->ShowWindow(SW_HIDE);

		DoCalibration();
	
	    lRet = RegCreateKey(HKEY_LOCAL_MACHINE, STR_BROADWAY_HARDWARE_SUBKEY, &hkOptions);
		if (lRet == ERROR_SUCCESS) {

			dwResult = 1;
			lRet = RegSetValueEx(
				    hkOptions,
					STR_DRIVER_INSTALL,
					0,
					REG_DWORD,
					(const unsigned char *)&dwResult,
					sizeof(dwResult));

			RegCloseKey(hkOptions);
		}
	}


#endif

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

HBRUSH CCapAbout::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) 
{
	//don't erase the bitmap when drawing controls.
	pDC->SetBkMode( OPAQUE );

	pDC->SetTextColor( RGB( 255, 255, 255 ) );	
	pDC->SetBkColor( RGB( 0, 0, 0 ) );

	return( (HBRUSH)::GetStockObject( BLACK_BRUSH	));

}


void CCapAbout::OnCalibrate() 
{
	int ret = MessageBox (
		"Calibration for audio/video syncronization may "
		"take several seconds. All open programs should "
		"be closed before beginning calibration. Are you "
		"sure you want to continue?", "Calibration", 
		MB_YESNO|MB_ICONQUESTION);

	if (ret == IDNO) {
		return;
	}
	
	DoCalibration();
}



void CCapAbout::DoCalibration() 
{
	HCURSOR hCursor = AfxGetApp()->LoadStandardCursor(IDC_WAIT);
	HCURSOR hOld = SetCursor (hCursor);

	double dfCal = BW_CalibrateAudio (m_hWnd);
    
	SetCursor (hOld);
	
	DWORD dwCal = (DWORD)(dfCal * (double)1000000000.0);

    DWORD lRet;
    HKEY hkOptions;

    lRet = RegCreateKey(HKEY_LOCAL_MACHINE, STR_BROADWAY_HARDWARE_SUBKEY, &hkOptions);
    if (lRet == ERROR_SUCCESS) {

        lRet = RegSetValueEx(
                hkOptions,
                STR_AUDIO_CAL_FACTOR,
                0,
                REG_DWORD,
                (const unsigned char *)&dwCal,
                sizeof(dwCal));

		dwCal = 1;
		lRet = RegSetValueEx(
			    hkOptions,
				STR_DRIVER_INSTALL,
				0,
				REG_DWORD,
				(const unsigned char *)&dwCal,
				sizeof(dwCal));

        RegCloseKey(hkOptions);
	}
}


