#include "stdafx.h"
#include "bwdialog.h"
#include "captures.h"
#include "captvid.h"
#include "captaud.h"
#include "captcolr.h"
#include "captqual.h"
#include "capabout.h"
#include "encdlg.h"
#include "progress.h"
#include "capappearance.h"
#include "captureCompression.h"
#include "CalErrorDlg.h"
#include "bwhelper.h"

// Hide a dialog box control
static void HideControl(int controlId,CWnd *pParentWnd) {
	CWnd *pWnd = pParentWnd->GetDlgItem( controlId);
	pWnd->ModifyStyle(0,0,SWP_HIDEWINDOW);
}

// Show a dialog box control
static void ShowControl(int controlId,CWnd *pParentWnd) {
	CWnd *pWnd = pParentWnd->GetDlgItem( controlId);
	pWnd->ModifyStyle(0,0,SWP_SHOWWINDOW);
}

BOOL WINAPI 
BW_GetCaptureOptions(
	BW_BOARDINFO *pInfo, 
	BW_CAPTUREOPTIONS *pCaptureOptions, 
	BW_RECODEOPTIONS *pRecodeOptions,
	int nFirstPage)
{
   	CBwdialogApp *pApp = (CBwdialogApp *)AfxGetApp();
	BW_CAPTURE_PREFERENCES *pCapPref = pCaptureOptions->lpPreferences;
	BW_RECODE_PREFERENCES *pRecPref = pRecodeOptions->lpPreferences;
	DWORD Caps;

    //fill in the current settings
    pApp->m_Settings.CaptureInitialize(pInfo, pCaptureOptions->lpProf);

#if 0
	//only rev 2 and higher hardware support realtime mpeg capture
	if (BW_HardwareRevision (&HardwareRevision)) {
		if (HardwareRevision < 2) {
			pApp->m_Settings.SetCaptureMode(CAPMODE_IFRAME);
		}
	}
#endif

	if (BW_HardwareCaps (&Caps) ) 
		if (!(Caps & BROADWAY_CANCAPTURE_MPEG)) 
			pApp->m_Settings.SetCaptureMode(CAPMODE_IFRAME);
	


    //we NOW care about this mode - preference files!!!                                        
    if (pCaptureOptions->nMode == MODE_MODIFY_CURRENT) {

		//verify the capture/compression settings/
		pApp->m_Settings.UpdateEncodeType (pRecPref->EncodePlaybackTarget);
		pApp->m_Settings.ValidateCapturePreferences(pCapPref);
		pApp->m_Settings.ValidateCompressionPreferences(pRecPref);

		//set the AVI capture options
		pApp->m_Settings.SetAutoDetect(pCapPref->nAutoDetect);	
        pApp->m_Settings.SetNTSC(pCapPref->nNTSC);			
        pApp->m_Settings.SetComposite(pCapPref->nComposite);	 

		pApp->m_Settings.SetAudioEnable(pCapPref->nAudioEnable);
		pApp->m_Settings.SetAudioMode(pCapPref->nAudioMode);
		pApp->m_Settings.SetAudioRate(pCapPref->nAudioRate);
        
		pApp->m_Settings.SetHue(pCapPref->nHue);		 
        pApp->m_Settings.SetSaturation(pCapPref->nSaturation);			
        pApp->m_Settings.SetIntensity(pCapPref->nIntensity);	
        pApp->m_Settings.SetContrast(pCapPref->nContrast);	 
        pApp->m_Settings.SetFilterValue(pCapPref->nFilterValue);		
        pApp->m_Settings.SetBlankLines(pCapPref->nBlankLines);		

        pApp->m_Settings.SetBitRate(pCapPref->nBitRate);		
        pApp->m_Settings.SetCaptureMode(pCapPref->nCaptureMode);		

		//set the MPG capture options 
		pApp->m_Settings.SetEncodePlayBackTarget(pRecPref->EncodePlaybackTarget);
		
		pApp->m_Settings.SetEncodeVideoEnable(pRecPref->nEncodeVideoEnable);
		pApp->m_Settings.SetEncodeVideoBitRate(pRecPref->nEncodeVideoBitRate);
		pApp->m_Settings.SetEncodeSquarePixels(0);
		pApp->m_Settings.SetEncodeSofteningLevel(0);
		pApp->m_Settings.SetLargeImage(pRecPref->nLargeImage);
		
		pApp->m_Settings.SetEncodeAudioEnable(pRecPref->nEncodeAudioEnable);
		pApp->m_Settings.SetEncodeAudioBitRate(pRecPref->nEncodeAudioBitRate);
		pApp->m_Settings.SetEncodeAudioMode(pRecPref->nEncodeAudioMode);

    	pApp->m_Settings.SetCompressionEstimate(pRecPref->nCompressionEstimate);
		
		CString Target = pRecPref->EncodePlaybackTarget;
		CString iniFile = pApp->m_Settings.GetIniFile();

		// These options are fixed based on the playbaxck target 
		pApp->m_Settings.SetEncodeUseOptimumRate(GetPrivateProfileInt(Target,"OptimumStreamRate",1,iniFile));
		pApp->m_Settings.SetEncodeStreamDataRate(GetPrivateProfileInt(Target,"StreamDataRate",5600000,iniFile));
		pApp->m_Settings.SetEncodePackSize(GetPrivateProfileInt(Target,"StreamPackSize",2048,iniFile));
		pApp->m_Settings.SetEncodePacketsPerPack(GetPrivateProfileInt(Target,"PacketsPerPack",1,iniFile));
		pApp->m_Settings.SetStreamMixType(GetPrivateProfileInt(Target,"StreamMixType",0,iniFile));

		//all done - now write them to the registry
		pApp->m_Settings.Terminate();

		return TRUE;
	}

	//get current capture settings 
    pCapPref->nAutoDetect   =  pApp->m_Settings.AutoDetect();	
    pCapPref->nNTSC         =  pApp->m_Settings.Ntsc();			
    pCapPref->nComposite    =  pApp->m_Settings.Composite();	 
    
	pCapPref->nAudioEnable	=  pApp->m_Settings.AudioEnable();
	pCapPref->nAudioRate    =  pApp->m_Settings.AudioRate();
	pCapPref->nAudioMode    =  pApp->m_Settings.AudioMode();
   
	pCapPref->nHue          =  pApp->m_Settings.Hue();		 
    pCapPref->nSaturation   =  pApp->m_Settings.Saturation();			
    pCapPref->nIntensity    =  pApp->m_Settings.Intensity();	
    pCapPref->nContrast     =  pApp->m_Settings.Contrast();	 
    pCapPref->nFilterValue  =  pApp->m_Settings.FilterValue();		
    pCapPref->nBlankLines   =  pApp->m_Settings.BlankLines();		

    pCapPref->nBitRate      =  pApp->m_Settings.BitRate();		
    pCapPref->nCaptureMode  =  pApp->m_Settings.CaptureMode();		

	//get current mpeg capture settings 	
	lstrcpy (pRecPref->EncodePlaybackTarget, pApp->m_Settings.EncodePlayBackTarget());

	pRecPref->nEncodeVideoEnable	= pApp->m_Settings.EncodeVideoEnable();
	pRecPref->nEncodeVideoBitRate	= pApp->m_Settings.EncodeVideoBitRate();
	pRecPref->nEncodeSquarePixels	= pApp->m_Settings.EncodeSquarePixels();
	pRecPref->nEncodeSofteningLevel	= pApp->m_Settings.EncodeSofteningLevel();
	pRecPref->nLargeImage			= pApp->m_Settings.LargeImage();
	
	pRecPref->nEncodeAudioEnable	= pApp->m_Settings.EncodeAudioEnable();
	pRecPref->nEncodeAudioBitRate	= pApp->m_Settings.EncodeAudioBitRate();
	pRecPref->nEncodeAudioMode		= pApp->m_Settings.EncodeAudioMode();

    pRecPref->nCompressionEstimate	= pApp->m_Settings.CompressionEstimate();

	pRecodeOptions->nEncodeUseOptimumRate	= pApp->m_Settings.EncodeUseOptimumRate();
	pRecodeOptions->nEncodeStreamDataRate	= pApp->m_Settings.EncodeStreamDataRate();
	pRecodeOptions->nEncodePackSize			= pApp->m_Settings.EncodePackSize();
	pRecodeOptions->nEncodePacketsPerPack	= pApp->m_Settings.EncodePacketsPerPack();
    pRecodeOptions->nStreamMixType			= pApp->m_Settings.StreamMixType();

    //If a configure dialog is requested display it. Otherwise
    //we're just being queried for the current registry
    //settings. so skip it.
    if (pCaptureOptions->nMode == MODE_DISPLAY_DIALOG) {

        CWnd *pParentWnd = CWnd::FromHandle(pCaptureOptions->hWndParent);

    	//create and init the pages
	    CCaptureVideo 	videoPage;
	    CCaptureColor 	colorPage;
	    CCaptureAudio 	audioPage;
   	    CCaptureQuality qualityPage;
		CCapAppearance  appearancePage;
		CCaptureCompression compressionPage;

		int nId;

		// Force the other condition
		if (pApp->m_Settings.CaptureMode() == CAPMODE_IFRAME) {
			nId = IDS_AVI_CAPTURE_OPTIONS;
		} else {
			nId = IDS_MPG_CAPTURE_OPTIONS;
		}

		//create the property sheet
        //nFirstPage = pCaptureOptions->nIsBroadway ?0:nFirstPage;
        nFirstPage = 0;
        CCaptureSheet propSheet (nId, pParentWnd, nFirstPage);

	    //add the pages to the sheet
	    propSheet.AddPage(&videoPage);
	    propSheet.AddPage(&colorPage);
	    propSheet.AddPage(&appearancePage);

        if (pCaptureOptions->nIsBroadway) {
            propSheet.AddPage(&audioPage);
	    }

        if (pCaptureOptions->nIsBroadway && pApp->m_Settings.CaptureMode() == CAPMODE_RTMPEG) {
            propSheet.AddPage(&compressionPage);
	    } else {
			propSheet.AddPage(&qualityPage);
		}
        
	    // pop up the property sheet
	    int i = 0;
	    CPropertyPage *pPage;
	    if(propSheet.DoModal() == IDOK) {

		    for(i=0; i<propSheet.GetPageCount(); i++) {
			    pPage = propSheet.GetPage(i);
			    pPage->OnOK();
		    }
            
		    pCapPref->nAutoDetect	=  pApp->m_Settings.AutoDetect();	
			pCapPref->nNTSC         =  pApp->m_Settings.Ntsc();			
			pCapPref->nComposite    =  pApp->m_Settings.Composite();	 
    
			pCapPref->nAudioEnable  =  pApp->m_Settings.AudioEnable();
			pCapPref->nAudioMode    =  pApp->m_Settings.AudioMode();
			pCapPref->nAudioRate    =  pApp->m_Settings.AudioRate();
   
			pCapPref->nHue          =  pApp->m_Settings.Hue();		 
			pCapPref->nSaturation   =  pApp->m_Settings.Saturation();			
			pCapPref->nIntensity    =  pApp->m_Settings.Intensity();	
			pCapPref->nContrast     =  pApp->m_Settings.Contrast();	 
			pCapPref->nFilterValue  =  pApp->m_Settings.FilterValue();		
		    pCapPref->nBlankLines   =  pApp->m_Settings.BlankLines();		

            pCapPref->nBitRate		= pApp->m_Settings.BitRate();	
			pCapPref->nCaptureMode  =  pApp->m_Settings.CaptureMode();		
		
			lstrcpy (pRecPref->EncodePlaybackTarget, pApp->m_Settings.EncodePlayBackTarget());

			pRecPref->nEncodeVideoEnable	= pApp->m_Settings.EncodeVideoEnable();
			pRecPref->nEncodeVideoBitRate	= pApp->m_Settings.EncodeVideoBitRate();
			pRecPref->nEncodeSquarePixels	= pApp->m_Settings.EncodeSquarePixels();
			pRecPref->nEncodeSofteningLevel	= pApp->m_Settings.EncodeSofteningLevel();
			pRecPref->nLargeImage			= pApp->m_Settings.LargeImage();
	
			pRecPref->nEncodeAudioEnable	= pApp->m_Settings.EncodeAudioEnable();
			pRecPref->nEncodeAudioBitRate	= pApp->m_Settings.EncodeAudioBitRate();
			pRecPref->nEncodeAudioMode		= pApp->m_Settings.EncodeAudioMode();

			pRecPref->nCompressionEstimate	= pApp->m_Settings.CompressionEstimate();

			pRecodeOptions->nEncodeUseOptimumRate	= pApp->m_Settings.EncodeUseOptimumRate();
			pRecodeOptions->nEncodeStreamDataRate	= pApp->m_Settings.EncodeStreamDataRate();
			pRecodeOptions->nEncodePackSize			= pApp->m_Settings.EncodePackSize();
			pRecodeOptions->nEncodePacketsPerPack	= pApp->m_Settings.EncodePacketsPerPack();
		    pRecodeOptions->nStreamMixType			= pApp->m_Settings.StreamMixType();

		}
    }

    //all done with the registry settings for now
    pApp->m_Settings.Terminate();
    
	//only a broadway app supports mpeg capture
	if (!pCaptureOptions->nIsBroadway) {
		pCapPref->nCaptureMode = CAPMODE_IFRAME;
	}
	
	return (TRUE);
}


int WINAPI 
BW_CapturePropertiesDialog (HWND hWnd, BW_PROPERTYOPTIONS *pOptions)
{
    /* verify structure size */
    if (pOptions == NULL || pOptions->ulStructSize < sizeof(BW_PROPERTYOPTIONS)) {
		return DRVCNF_CANCEL;
    }
   	
	CBwdialogApp *pApp = (CBwdialogApp *)AfxGetApp();
    CWnd *pParentWnd = CWnd::FromHandle(hWnd);

   	CCapAbout AboutDlg(pParentWnd, pOptions);
    return AboutDlg.DoModal();
}


BOOL WINAPI
BW_GetCaptureEncodeType (LPVC_PROFILE_INFO lpProf, LPTSTR lpszType, UINT cbSize)
{
   	CBwdialogApp *pApp = (CBwdialogApp *)AfxGetApp();

    pApp->m_Settings.CaptureInitialize(NULL, lpProf);
    lstrcpyn (lpszType, pApp->m_Settings.EncodeType(), cbSize);

	pApp->m_Settings.Terminate();

	return TRUE;
}


BOOL WINAPI
BW_SetCaptureEncodeType (LPVC_PROFILE_INFO lpProf, LPCTSTR lpszType)
{
   	CBwdialogApp *pApp = (CBwdialogApp *)AfxGetApp();
	BOOL bReturn;

    pApp->m_Settings.CaptureInitialize(NULL, lpProf);
	bReturn = pApp->m_Settings.ReadEncodeType(lpszType);

	pApp->m_Settings.Terminate();

	return bReturn;
}


BOOL WINAPI
BW_GetRecodeType (LPTSTR lpszType, UINT cbSize)
{

   	CBwdialogApp *pApp = (CBwdialogApp *)AfxGetApp();

    pApp->m_Settings.RecodeInitialize();
    lstrcpyn (lpszType, pApp->m_Settings.EncodeType(), cbSize);

	pApp->m_Settings.Terminate();

	return TRUE;
}


BOOL WINAPI
BW_SetRecodeType (LPCTSTR lpszType)
{
   	CBwdialogApp *pApp = (CBwdialogApp *)AfxGetApp();
	BOOL bReturn;

    pApp->m_Settings.RecodeInitialize();

	bReturn = pApp->m_Settings.ReadEncodeType(lpszType);

	pApp->m_Settings.Terminate();

	return bReturn;
}

BOOL WINAPI
BW_GetMpegIniFileName (LPTSTR lpszName, UINT cbSize)
{

	GetSystemDirectory(lpszName,cbSize);
	DWORD Caps;
	if (BW_HardwareCaps (&Caps) ) 
		if ((Caps & BROADWAY_CANUSE_EXTERNAL_TARGETS)) 
			strncat(lpszName,TEXT("\\bwtables\\mpegtype.ini"),cbSize-strlen(lpszName));
		else
			strncat(lpszName,TEXT("\\bwtables\\begin.ini"),cbSize-strlen(lpszName));

	return TRUE;
}


BOOL WINAPI 
BW_GetRecodeOptions(BW_RECODEOPTIONS *pRecodeOptions)
{
	BW_RECODE_PREFERENCES *pRecPref = pRecodeOptions->lpPreferences;
   	CBwdialogApp *pApp = (CBwdialogApp *)AfxGetApp();

    pApp->m_Settings.RecodeInitialize();

	//we NOW care about this mode - preference files!!!                                        
    if (pRecodeOptions->nMode == MODE_MODIFY_CURRENT) {

		//verify the compression settings/
		pApp->m_Settings.ValidateCompressionPreferences(pRecPref);

		//set the compression options 
		pApp->m_Settings.SetEncodePlayBackTarget(pRecPref->EncodePlaybackTarget);
		
		pApp->m_Settings.SetEncodeVideoEnable(pRecPref->nEncodeVideoEnable);
		pApp->m_Settings.SetEncodeVideoBitRate(pRecPref->nEncodeVideoBitRate);
		pApp->m_Settings.SetEncodeSquarePixels(0);
		pApp->m_Settings.SetEncodeSofteningLevel(0);
		pApp->m_Settings.SetLargeImage(pRecPref->nLargeImage);
		
		pApp->m_Settings.SetEncodeAudioEnable(pRecPref->nEncodeAudioEnable);
		pApp->m_Settings.SetEncodeAudioBitRate(pRecPref->nEncodeAudioBitRate);
		pApp->m_Settings.SetEncodeAudioMode(pRecPref->nEncodeAudioMode);

    	pApp->m_Settings.SetCompressionEstimate(pRecPref->nCompressionEstimate);
		
		CString Target = pRecPref->EncodePlaybackTarget;
		CString iniFile = pApp->m_Settings.GetIniFile();

		// These options are fixed based on the playbaxck target 
		pApp->m_Settings.SetEncodeUseOptimumRate(GetPrivateProfileInt(Target,"OptimumStreamRate",1,iniFile));
		pApp->m_Settings.SetEncodeStreamDataRate(GetPrivateProfileInt(Target,"StreamDataRate",5600000,iniFile));
		pApp->m_Settings.SetEncodePackSize(GetPrivateProfileInt(Target,"StreamPackSize",2048,iniFile));
		pApp->m_Settings.SetEncodePacketsPerPack(GetPrivateProfileInt(Target,"PacketsPerPack",1,iniFile));
		pApp->m_Settings.SetStreamMixType(GetPrivateProfileInt(Target,"StreamMixType",0,iniFile));

	    //all done - now write them to the registry
		pApp->m_Settings.Terminate();

		return TRUE;
	}


    //fill in the current settings
	lstrcpy (pRecPref->EncodePlaybackTarget, pApp->m_Settings.EncodePlayBackTarget());

	pRecPref->nEncodeVideoEnable	= pApp->m_Settings.EncodeVideoEnable();
	pRecPref->nEncodeVideoBitRate	= pApp->m_Settings.EncodeVideoBitRate();
	pRecPref->nEncodeSquarePixels	= pApp->m_Settings.EncodeSquarePixels();
	pRecPref->nEncodeSofteningLevel	= pApp->m_Settings.EncodeSofteningLevel();
	pRecPref->nLargeImage			= pApp->m_Settings.LargeImage();
	
	pRecPref->nEncodeAudioEnable	= pApp->m_Settings.EncodeAudioEnable();
	pRecPref->nEncodeAudioBitRate	= pApp->m_Settings.EncodeAudioBitRate();
	pRecPref->nEncodeAudioMode		= pApp->m_Settings.EncodeAudioMode();

    pRecPref->nCompressionEstimate	= pApp->m_Settings.CompressionEstimate();

	pRecodeOptions->nEncodeUseOptimumRate	= pApp->m_Settings.EncodeUseOptimumRate();
	pRecodeOptions->nEncodeStreamDataRate	= pApp->m_Settings.EncodeStreamDataRate();
	pRecodeOptions->nEncodePackSize			= pApp->m_Settings.EncodePackSize();
	pRecodeOptions->nEncodePacketsPerPack	= pApp->m_Settings.EncodePacketsPerPack();
    pRecodeOptions->nStreamMixType			= pApp->m_Settings.StreamMixType();

    //If a configure dialog is requested display it. Otherwise
    //we're just being queried for the current registry
    //settings. so skip it.
    if (pRecodeOptions->nMode == MODE_DISPLAY_DIALOG) {

        CWnd *pParentWnd = CWnd::FromHandle(pRecodeOptions->hWndParent);

    	CEncodeDialog  encodeDlg (pParentWnd);
	    
		encodeDlg.DoModal();

        // fill in the updated settings
		lstrcpy (pRecPref->EncodePlaybackTarget, pApp->m_Settings.EncodePlayBackTarget());

		pRecPref->nEncodeVideoEnable	= pApp->m_Settings.EncodeVideoEnable();
		pRecPref->nEncodeVideoBitRate	= pApp->m_Settings.EncodeVideoBitRate();
		pRecPref->nEncodeSquarePixels	= pApp->m_Settings.EncodeSquarePixels();
		pRecPref->nEncodeSofteningLevel	= pApp->m_Settings.EncodeSofteningLevel();
		pRecPref->nLargeImage			= pApp->m_Settings.LargeImage();
	
		pRecPref->nEncodeAudioEnable	= pApp->m_Settings.EncodeAudioEnable();
		pRecPref->nEncodeAudioBitRate	= pApp->m_Settings.EncodeAudioBitRate();
		pRecPref->nEncodeAudioMode		= pApp->m_Settings.EncodeAudioMode();

		pRecPref->nCompressionEstimate	= pApp->m_Settings.CompressionEstimate();

		pRecodeOptions->nEncodeUseOptimumRate	= pApp->m_Settings.EncodeUseOptimumRate();
		pRecodeOptions->nEncodeStreamDataRate	= pApp->m_Settings.EncodeStreamDataRate();
		pRecodeOptions->nEncodePackSize			= pApp->m_Settings.EncodePackSize();
		pRecodeOptions->nEncodePacketsPerPack	= pApp->m_Settings.EncodePacketsPerPack();
		pRecodeOptions->nStreamMixType			= pApp->m_Settings.StreamMixType();
    }

    //all done with the registry settings for now
    pApp->m_Settings.Terminate();
    return (TRUE);
}


LPVOID WINAPI
BW_CalProgressBarCreate (HWND hParent)
{
    CWnd *pParentWnd = CWnd::FromHandle(hParent);
	CCalibrationProgress *pProgressDlg;

	pProgressDlg = new CCalibrationProgress(pParentWnd, &pProgressDlg, NULL, 100);
	pProgressDlg->Create(CCalibrationProgress::IDD);

	return (LPVOID)pProgressDlg;
}

VOID WINAPI
BW_CalProgressSetText (LPVOID lpPb, LPCTSTR lpstrText) 
{
	CCalibrationProgress *pProgressDlg = (CCalibrationProgress *)lpPb;
	pProgressDlg->m_FileName.SetWindowText( lpstrText );

}

VOID WINAPI
BW_CalProgressBarSetPercentage (LPVOID lpPb, UINT nPercent)
{
	CCalibrationProgress *pProgressDlg = (CCalibrationProgress *)lpPb;
	pProgressDlg->SetPercentage(nPercent);
}


BOOL WINAPI
BW_CalProgressBarContinue (LPVOID lpPb) 
{ 
	CCalibrationProgress *pProgressDlg = (CCalibrationProgress *)lpPb;
	return pProgressDlg->Continue();
}


VOID WINAPI
BW_CalProgressBarDestroy (LPVOID lpPb)
{
	CCalibrationProgress *pProgressDlg = (CCalibrationProgress *)lpPb;
	delete pProgressDlg;
}

int WINAPI
BW_CalErrorDialog (HWND hParent)
{
    CWnd *pParentWnd = CWnd::FromHandle(hParent);

	CCalErrorDlg dlg(pParentWnd);
	
	return dlg.DoModal();
}