/*
 * bwuser.c
 *
 * 32-bit Video Capture driver
 * Broadway App User-mode support library
 *
 * functions providing access to video capture and recompression 
 * hardware, by accessing the kernel-mode device driver. 
 * 
 *
 * Mark Vrabel, October 95:
 */


#include <vcdriver.h>
#include "hardware.h"
#include <bwdlgs.h>
#include <bwuser.h>
#include <rcuser.h>
#include "realtime.h"
#include "resource.h"
#include <bwhelper.h>
#include <errors.h>
#include <stdio.h>

//#define _USE_HIDDEN_APP_


/*
 * needed to bybass subclassed window proc
 */
extern "C" FARPROC gfpOrigCapWndProc; 
extern "C" LPHWINFO glpHwCurrent;


#define MAX_PATHNAME    256
#define MAX_COMMANDLINE (MAX_PATHNAME+1+MAX_PATHNAME+1+MAX_PATHNAME+1+10+1)

/*
 * Init Private Capture Interface
 *
 * Initializes the driver's private capture interface
 * by subclassing the VFW avicap capture window.  It 
 * will fail if the current proccess 
 * does not have the driver open for capture.   
 */
BOOL WINAPI
BW_SetPrivateCaptureInterface (HWND hWndCap, LPCAPCALLBACKS lpCapCalbacks)
{
	return RT_GetCaptureWindow(hWndCap, lpCapCalbacks);
}


/*
 * Release Private Capture Interface
 *
 * Initializes the driver's private capture interface
 * by subclassing the VFW avicap capture window.  It 
 * will fail if the current proccess 
 * does not have the driver open for capture.   
 */
BOOL WINAPI
BW_ReleasePrivateCaptureInterface (HWND hWndCap)
{
	return RT_ReleaseCaptureWindow(hWndCap);
}


#define MODE_MODIFY_CURRENT    0      
#define MODE_FETCH_SETTINGS    1      
#define MODE_DISPLAY_DIALOG    2

/*
 * Capture Configuration.
 *
 * This function displays the capure options property
 * sheet.
 */
BOOL WINAPI BW_ConfigCapture (HWND hWndParent) 
{
    LPHWINFO lpHw = glpHwCurrent;

    BW_CAPTUREOPTIONS CaptureOptions;
    BW_CAPTURE_PREFERENCES CapturePreferences;

    BW_RECODEOPTIONS RecodeOptions;
    BW_RECODE_PREFERENCES RecodePreferences;

	BOOL bReturn;

    /* open the registry profile */
    CaptureOptions.lpProf = VC_OpenProfileAccess("vtool", 0);
    if (CaptureOptions.lpProf == NULL) {
        return FALSE;
    }

	/* init the capture options structure */
	CaptureOptions.ulStructSize		 = sizeof(BW_CAPTUREOPTIONS);
    CaptureOptions.nMode			 = MODE_DISPLAY_DIALOG;
    CaptureOptions.nIsBroadway		 = TRUE;
	CaptureOptions.nPrivateInterface = TRUE;
    CaptureOptions.hWndParent        = hWndParent;

	CapturePreferences.ulStructSize	 = sizeof(BW_CAPTURE_PREFERENCES);
	CaptureOptions.lpPreferences	 = &CapturePreferences;

	/* init the recode options structure */
	RecodeOptions.ulStructSize		 = sizeof(BW_RECODEOPTIONS);
    RecodeOptions.nMode				 = MODE_DISPLAY_DIALOG;
    RecodeOptions.hWndParent		 = hWndParent;

	RecodePreferences.ulStructSize	 = sizeof(BW_RECODE_PREFERENCES);
	RecodeOptions.lpPreferences		 = &RecodePreferences;

	/* pop up the config dialog */
	bReturn = BW_GetCaptureOptions(NULL, &CaptureOptions, &RecodeOptions, 0);

	/* if the hardware is available - configure now */
	if (bReturn && lpHw != NULL) {
	    BW_CAPTURE_CONFIG Config;
	    
		Config.ulStructSize = sizeof(BW_CAPTURE_CONFIG);
		Config.lpProf		= CaptureOptions.lpProf;
		Config.nMode		= MODE_FETCH_SETTINGS;
	    Config.nIsBroadway	= TRUE;
		Config.nWndParent   = (long)NULL;

		bReturn = HW_ConfigFormat(lpHw->vh, (LPCONFIG_INFO)&Config);
		if (bReturn && lpHw->hWndCap != NULL) {
			lpHw->bNoDialog = TRUE;
			bcapDlgVideoFormat (lpHw->hWndCap);
			lpHw->bNoDialog = FALSE;
		}
	} 

	VC_CloseProfileAccess(CaptureOptions.lpProf);
	return bReturn;
}

/*
 * Set overlay blank lines
 */
BOOL WINAPI BW_SetBlankLines(long BlankLines) {

	LPHWINFO lpHw = glpHwCurrent;

	if (lpHw == NULL) {
		return FALSE;
	}

	VOVERLAY_HANDLE voh = lpHw->voh;

	if (voh) {
		return VO_SetOverlayBlankLines(voh,BlankLines);
	} else {
		return FALSE;
	}
}


/*
 * Load Capture Preferences
 *
 * Loads the capture settings in the given 
 * configuration structres to the device 
 * registry.
 */
BOOL WINAPI BW_LoadCapturePreferences (
    BW_CAPTURE_PREFERENCES FAR *lpCapturePreferences,
    BW_RECODE_PREFERENCES *lpRecodePreferences
)
{
    LPHWINFO lpHw = glpHwCurrent;
    
	BW_CAPTUREOPTIONS CaptureOptions;
    BW_RECODEOPTIONS RecodeOptions;

	BOOL bReturn;

    BW_BOARDINFO *lpBoardInfo = NULL;
    BW_BOARDINFO BoardInfo;

    /* fill in the board info */
    if (lpHw != NULL) {
		BoardInfo.vh       = lpHw->vh;
		BoardInfo.DevID    = lpHw->DevID;
		BoardInfo.bOverlay = lpHw->bCapture;
		BoardInfo.bOverlay = lpHw->bOverlay;

		lpBoardInfo = &BoardInfo;
	}

	/* verify pointers */
	if (lpCapturePreferences == NULL) {
		return FALSE;
	}
	if (lpRecodePreferences == NULL) {
		return FALSE;
	}

	/* verify structure sizes */
	if (lpCapturePreferences->ulStructSize < sizeof(BW_CAPTURE_PREFERENCES)) {
		return FALSE;
	}
	if (lpRecodePreferences->ulStructSize < sizeof(BW_RECODE_PREFERENCES)) {
		return FALSE;
	}

    /* open the registry profile */
    CaptureOptions.lpProf = VC_OpenProfileAccess("vtool", 0);
    if (CaptureOptions.lpProf == NULL) {
        return FALSE;
    }

	/* init the capture options structure */
	CaptureOptions.ulStructSize		 = sizeof(BW_CAPTUREOPTIONS);
    CaptureOptions.nMode			 = MODE_MODIFY_CURRENT;
    CaptureOptions.nIsBroadway		 = TRUE;
	CaptureOptions.nPrivateInterface = TRUE;
    CaptureOptions.hWndParent        = NULL;
	CaptureOptions.lpPreferences	 = lpCapturePreferences;

	/* init the recode options structure */
	RecodeOptions.ulStructSize		 = sizeof(BW_RECODEOPTIONS);
    RecodeOptions.nMode				 = MODE_MODIFY_CURRENT;
    RecodeOptions.hWndParent		 = NULL;
	RecodeOptions.lpPreferences		 = lpRecodePreferences;

	/* modify the current settings */
	bReturn = BW_GetCaptureOptions(lpBoardInfo, &CaptureOptions, &RecodeOptions, 0);

	/* if the hardware is available - configure now */
	if (bReturn && lpHw != NULL) {
	    BW_CAPTURE_CONFIG Config;
		HWND hWnd = lpHw->hWndCap;

		Config.ulStructSize = sizeof(BW_CAPTURE_CONFIG);
		Config.lpProf		= CaptureOptions.lpProf;
		Config.nMode		= MODE_FETCH_SETTINGS;
	    Config.nIsBroadway	= TRUE;
		Config.nWndParent   = (long)NULL;

		bReturn = HW_ConfigFormat(lpHw->vh, (LPCONFIG_INFO)&Config);
		if (bReturn && lpHw->hWndCap != NULL) {
			lpHw->bNoDialog = TRUE;
			bcapDlgVideoFormat (lpHw->hWndCap);
			lpHw->bNoDialog = FALSE;
		}

		/* modify the capture file name in case the capture mode changed */
		if (hWnd != NULL) {
			char szFile[MAX_PATH];
		
			capFileGetCaptureFile(hWnd, szFile, sizeof(szFile));
			capFileSetCaptureFile(hWnd, szFile);
		}
	} 

	VC_CloseProfileAccess(CaptureOptions.lpProf);
	return bReturn;
}


/*
 * Get Capture Preferences
 *
 * Retrieves the current capture settings of the device
 * from the registry and stores them in the given 
 * configuration structures.
 */
BOOL WINAPI BW_GetCapturePreferences (
    BW_CAPTURE_PREFERENCES FAR *lpCapturePreferences,
    BW_RECODE_PREFERENCES *lpRecodePreferences
)
{
	BW_CAPTUREOPTIONS CaptureOptions;
    BW_RECODEOPTIONS RecodeOptions;

	BOOL bReturn;

	/* verify pointers */
	if (lpCapturePreferences == NULL) {
		return FALSE;
	}
	if (lpRecodePreferences == NULL) {
		return FALSE;
	}

	/* verify structure sizes */
	if (lpCapturePreferences->ulStructSize < sizeof(BW_CAPTURE_PREFERENCES)) {
		return FALSE;
	}
	if (lpRecodePreferences->ulStructSize < sizeof(BW_RECODE_PREFERENCES)) {
		return FALSE;
	}

    /* open the registry profile */
    CaptureOptions.lpProf = VC_OpenProfileAccess("vtool", 0);
    if (CaptureOptions.lpProf == NULL) {
        return FALSE;
    }

	/* init the capture options structure */
	CaptureOptions.ulStructSize		 = sizeof(BW_CAPTUREOPTIONS);
    CaptureOptions.nMode			 = MODE_FETCH_SETTINGS;
    CaptureOptions.nIsBroadway		 = TRUE;
	CaptureOptions.nPrivateInterface = TRUE;
    CaptureOptions.hWndParent        = NULL;
	CaptureOptions.lpPreferences	 = lpCapturePreferences;

	/* init the recode options structure */
	RecodeOptions.ulStructSize		 = sizeof(BW_RECODEOPTIONS);
    RecodeOptions.nMode				 = MODE_FETCH_SETTINGS;
    RecodeOptions.hWndParent		 = NULL;
	RecodeOptions.lpPreferences		 = lpRecodePreferences;

	/* get the current settings */
	bReturn = BW_GetCaptureOptions(NULL, &CaptureOptions, &RecodeOptions, 0);

	VC_CloseProfileAccess(CaptureOptions.lpProf);
	return bReturn;
}


/*
 * Load Recode Preferences
 *
 * Loads the recode settings in the given 
 * configuration structure to the device 
 * registry.
 */
BOOL WINAPI BW_LoadRecodePreferences (
    BW_RECODE_PREFERENCES *lpRecodePreferences
)
{
    BW_RECODEOPTIONS RecodeOptions;
	BOOL bReturn;

	/* verify pointer */
	if (lpRecodePreferences == NULL) {
		return FALSE;
	}

	/* verify structure size */
	if (lpRecodePreferences->ulStructSize < sizeof(BW_RECODE_PREFERENCES)) {
		return FALSE;
	}

	/* init the recode options structure */
	RecodeOptions.ulStructSize		 = sizeof(BW_RECODEOPTIONS);
    RecodeOptions.nMode				 = MODE_MODIFY_CURRENT;
    RecodeOptions.hWndParent		 = NULL;
	RecodeOptions.lpPreferences		 = lpRecodePreferences;

	/* modify the current settings */
	bReturn = BW_GetRecodeOptions(&RecodeOptions);

	return bReturn;
}


/*
 * Get Recode Preferences
 *
 * Retrieves the current recode settings
 * from the device registry and stores
 * them in the given configuration structure. 
 */
BOOL WINAPI BW_GetRecodePreferences (
    BW_RECODE_PREFERENCES FAR *lpRecodePreferences
)
{
    BW_RECODEOPTIONS RecodeOptions;
	BOOL bReturn;

	/* verify pointer */
	if (lpRecodePreferences == NULL) {
		return FALSE;
	}

	/* verify structure size */
	if (lpRecodePreferences->ulStructSize < sizeof(BW_RECODE_PREFERENCES)) {
		return FALSE;
	}

	/* init the recode options structure */
	RecodeOptions.ulStructSize		 = sizeof(BW_RECODEOPTIONS);
    RecodeOptions.nMode				 = MODE_FETCH_SETTINGS;
    RecodeOptions.hWndParent		 = NULL;
	RecodeOptions.lpPreferences		 = lpRecodePreferences;

	/* modify the current settings */
	bReturn = BW_GetRecodeOptions(&RecodeOptions);

	return bReturn;
}


/*
 * Audio Mode
 *
 * This function reports back the last set Audio capture mode.  
 */
BOOL WINAPI
BW_AudioMode (UINT *pAudioMode) 
{
    BW_CAPTURE_PREFERENCES CapturePreferences;
    BW_RECODE_PREFERENCES RecodePreferences;
    LPHWINFO lpHw = glpHwCurrent;

	if (lpHw != NULL) {
		*pAudioMode = BW_AudioToMode (lpHw->AudioEnable, lpHw->AudioMode);
		return TRUE;
	}

    CapturePreferences.ulStructSize = sizeof(BW_CAPTURE_PREFERENCES);
    RecodePreferences.ulStructSize = sizeof(BW_RECODE_PREFERENCES);

	/* get the current settings */
	if (!BW_GetCapturePreferences (&CapturePreferences, &RecodePreferences)) {
		return FALSE;
	}

	/* return the current setting */
    *pAudioMode = BW_AudioToMode (CapturePreferences.nAudioEnable, CapturePreferences.nAudioMode);        

    return TRUE;
}


/*
 * Audio Rate                                                                  
 *
 * This function reports back the last set Audio capture rate.  
 */
BOOL WINAPI 
BW_AudioRate (UINT *pAudioRate) 
{
    BW_CAPTURE_PREFERENCES CapturePreferences;
    BW_RECODE_PREFERENCES RecodePreferences;
    LPHWINFO lpHw = glpHwCurrent;

	if (lpHw != NULL) {
		*pAudioRate = lpHw->AudioRate;
		return TRUE;
	}

    CapturePreferences.ulStructSize = sizeof(BW_CAPTURE_PREFERENCES);
    RecodePreferences.ulStructSize = sizeof(BW_RECODE_PREFERENCES);

	/* get the current settings */
	if (!BW_GetCapturePreferences (&CapturePreferences, &RecodePreferences)) {
		return FALSE;
	}

	/* return the current setting */
    *pAudioRate = CapturePreferences.nAudioRate;        

    return TRUE;
}


/*
 * Quality
 *
 * This function reports back the last set Quality capture Setting
 * in Megabytes per minute. 
 */
BOOL WINAPI 
BW_Quality (UINT *pQuality)
{
    BW_CAPTURE_PREFERENCES CapturePreferences;
    BW_RECODE_PREFERENCES RecodePreferences;
    LPHWINFO lpHw = glpHwCurrent;

	if (lpHw != NULL) {
		*pQuality = lpHw->Quality;
		return TRUE;
	}

    CapturePreferences.ulStructSize = sizeof(BW_CAPTURE_PREFERENCES);
    RecodePreferences.ulStructSize = sizeof(BW_RECODE_PREFERENCES);

	/* get the current settings */
	if (!BW_GetCapturePreferences (&CapturePreferences, &RecodePreferences)) {
		return FALSE;
	}

	/* return the current setting */
	*pQuality = CapturePreferences.nBitRate;        

    return TRUE;
}


/*
 * Real Time Quality
 *
 * This function reports back the last set real time MPEG quality 
 * capture Setting in in bytes per second. 
 */
BOOL WINAPI 
BW_RealtimeQuality (UINT *pQuality)
{
    BW_CAPTURE_PREFERENCES CapturePreferences;
    BW_RECODE_PREFERENCES RecodePreferences;
    LPHWINFO lpHw = glpHwCurrent;

	if (lpHw != NULL) {
		*pQuality = lpHw->RecodePreferences.nCompressionEstimate / 8;
		return TRUE;
	}

    CapturePreferences.ulStructSize = sizeof(BW_CAPTURE_PREFERENCES);
    RecodePreferences.ulStructSize = sizeof(BW_RECODE_PREFERENCES);

	/* get the current settings */
	if (!BW_GetCapturePreferences (&CapturePreferences, &RecodePreferences)) {
		return FALSE;
	}

	/* return the current setting  - convert from bits to bytes/sec */
    *pQuality = RecodePreferences.nCompressionEstimate / 8;    

    return TRUE;
}


/*
 * Hardware Revision
 *
 * This function reports back the board hardware revision.  
 */
BOOL WINAPI 
BW_HardwareRevision (UINT *pHardwareRevision) 
{
    LPHWINFO lpHw = glpHwCurrent;
    DEVID DevID;
	UCHAR RevisionID;
	BOOL bReturn;

	if (lpHw != NULL) {
		*pHardwareRevision = lpHw->RevisionID;
		return TRUE;
	}

	/* get the hardware revision */
	if (VT_OpenDevice(TEXT("vtool"),0,  &DevID) != VT_NO_ERROR) {
		return FALSE;
	} else {
		bReturn = VT_GetRevisionID(DevID, &RevisionID);
	}

	/* release the device */
	if (DevID != NULL) {
		VT_DeleteDevice(DevID);
	}

	/* return the revision */
    *pHardwareRevision = RevisionID;        

    return bReturn == 0 ?TRUE:FALSE;
}

BOOL WINAPI 
BW_HardwareCaps (DWORD *pHardwareCaps) 
{
    LPHWINFO lpHw = glpHwCurrent;
    DEVID DevID = 0;

	*pHardwareCaps = 0L;

	if (lpHw != NULL) {
		BroadwayGetBoardCaps(lpHw->DevID, pHardwareCaps);
		return TRUE;
	}

	/* get the hardware revision */
	if (BroadwayOpenDevice(&DevID,0) == VT_NO_ERROR) {
		BroadwayGetBoardCaps(DevID, pHardwareCaps);
		BroadwayCloseDevice(DevID);
	}

	return TRUE;
}


/*
 * Capture Mode
 *
 * This function reports back the last set Capture Mode 
 * capture setting. 
 */
BOOL WINAPI 
BW_CaptureMode (UINT *pMode)
{
    BW_CAPTURE_PREFERENCES CapturePreferences;
    BW_RECODE_PREFERENCES RecodePreferences;
    LPHWINFO lpHw = glpHwCurrent;
	DWORD Caps;

	if (!BW_HardwareCaps (&Caps) ) { 
		return FALSE;
	}
	
	if (!(Caps & BROADWAY_CANCAPTURE_MPEG)) {
		*pMode = CAPMODE_IFRAME_ONLY;
		return TRUE;
	}

	if (lpHw != NULL) {
		*pMode = lpHw->CaptureMode;
		return TRUE;
	}

    CapturePreferences.ulStructSize = sizeof(BW_CAPTURE_PREFERENCES);
    RecodePreferences.ulStructSize = sizeof(BW_RECODE_PREFERENCES);

	/* get the current settings */
	if (!BW_GetCapturePreferences (&CapturePreferences, &RecodePreferences)) {
		return FALSE;
	}

	/* return the current setting */
    *pMode = CapturePreferences.nCaptureMode;        

    return TRUE;
}


/*
 * File Type
 *
 * This function reports back the current output 
 * file type(s) generated. 
 */
BOOL WINAPI BW_FileType (UINT *pType) 
{

	BW_RECODEOPTIONS RecodeOptions; 
	BW_RECODEOPTIONS *lpRecodeOptions;

	BW_RECODE_PREFERENCES RecodePreferences; 
	BW_RECODE_PREFERENCES *lpRecodePreferences;

    LPHWINFO lpHw = glpHwCurrent;
	UINT Mode;

	if (!BW_CaptureMode (&Mode))	{
		return FALSE;
	}

	if (Mode != CAPMODE_RTMPEG) {
		*pType = FILETYPE_AVI;
		return TRUE;
	}

	if (lpHw != NULL) {
		lpRecodeOptions = &lpHw->RecodeOptions;
		lpRecodePreferences = &lpHw->RecodePreferences;
	} else {
	    BW_CAPTUREOPTIONS CaptureOptions;
		BW_CAPTURE_PREFERENCES CapturePreferences;
		BOOL bReturn;

		lpRecodePreferences = &RecodePreferences;
		lpRecodeOptions = &RecodeOptions;

	    /* open the registry profile */
		CaptureOptions.lpProf = VC_OpenProfileAccess("vtool", 0);
		if (CaptureOptions.lpProf == NULL) {
			return FALSE;
		}

		/* init the capture options structure */
		CaptureOptions.ulStructSize		 = sizeof(BW_CAPTUREOPTIONS);
		CaptureOptions.nMode			 = MODE_FETCH_SETTINGS;
		CaptureOptions.nIsBroadway		 = TRUE;
		CaptureOptions.nPrivateInterface = TRUE;
		CaptureOptions.hWndParent        = NULL;

		CapturePreferences.ulStructSize	 = sizeof(BW_CAPTURE_PREFERENCES);
		CaptureOptions.lpPreferences	 = &CapturePreferences;

		/* init the recode options structure */
		RecodeOptions.ulStructSize		 = sizeof(BW_RECODEOPTIONS);
		RecodeOptions.nMode				 = MODE_FETCH_SETTINGS;
		RecodeOptions.hWndParent		 = NULL;

		RecodePreferences.ulStructSize	 = sizeof(BW_RECODE_PREFERENCES);
		RecodeOptions.lpPreferences		 = &RecodePreferences;

		/* get the current settings */
		bReturn = BW_GetCaptureOptions(NULL, &CaptureOptions, &RecodeOptions, 0);
		VC_CloseProfileAccess(CaptureOptions.lpProf);
			
		if (!bReturn) {
			return FALSE;
		}
	}

	UINT AudioMode = BW_AudioToMode (lpRecodePreferences->nEncodeAudioEnable, lpRecodePreferences->nEncodeAudioMode);
	EStreamMixType StreamMixType = (EStreamMixType)lpRecodeOptions->nStreamMixType;
	
	BOOL bCaptureAudio = (AudioMode != AUDIO_NONE);
	BOOL bCaptureVideo = lpRecodePreferences->nEncodeVideoEnable;
	BOOL bStreamMix = (StreamMixType != NoMix && bCaptureAudio && bCaptureVideo) ?TRUE:FALSE;

	if (bStreamMix) {
		*pType = FILETYPE_MPG_ONLY;
	} else if (bCaptureAudio && bCaptureVideo) {
		*pType = FILETYPE_MPG_MPA;
	} else if (bCaptureAudio) {
		*pType = FILETYPE_MPA_ONLY;
	} else if (bCaptureVideo) {
		*pType = FILETYPE_MPG_ONLY;
	}

	return TRUE;
}


/*
 * File size
 *
 * This function sets the maximum mpeg file size.
 * file size of 0 indicate no limit to file size.
 * This is a request from PictureTel. File size is
 * given in megabytes. 
 */
BOOL WINAPI BW_SetFileSize (DWORD dwSize) 
{
    LPHWINFO lpHw = glpHwCurrent;

	if (lpHw == NULL) {
		return FALSE;
	}

	if (dwSize == FILE_SIZE_NOFILE) {
		lpHw->dwMaxFileSize = dwSize;
	} else {
		lpHw->dwMaxFileSize = dwSize * (1024*1024);
	}

	return TRUE;
}


/*
 * GetCapturePlayBackTarget
 *
 * This function reports back the current
 * capture play back target
 */
BOOL WINAPI 
BW_GetCapturePlayBackTarget (LPTSTR lpszTarget, UINT cbSize)
{
    BW_CAPTURE_PREFERENCES CapturePreferences;
    BW_RECODE_PREFERENCES RecodePreferences;
    LPHWINFO lpHw = glpHwCurrent;

	if (lpHw != NULL) {
	    lstrcpyn (lpszTarget, lpHw->RecodePreferences.EncodePlaybackTarget, cbSize);
		return TRUE;
	}

    CapturePreferences.ulStructSize = sizeof(BW_CAPTURE_PREFERENCES);
    RecodePreferences.ulStructSize = sizeof(BW_RECODE_PREFERENCES);

	/* get the current settings */
	if (!BW_GetCapturePreferences (&CapturePreferences, &RecodePreferences)) {
		return FALSE;
	}

	/* return the current setting */
    lstrcpyn (lpszTarget, RecodePreferences.EncodePlaybackTarget, cbSize);

    return TRUE;
}


/*
 * SetCapturePlayBackTarget
 *
 * This function sets the current
 * compression play back target and updates
 * the registry seting to reflect the last set
 * options for that target
 */
BOOL 
WINAPI BW_SetCapturePlayBackTarget (LPTSTR lpszTarget)
{
    LPHWINFO lpHw = glpHwCurrent;

    BW_CAPTUREOPTIONS CaptureOptions;
    BW_CAPTURE_PREFERENCES CapturePreferences;

    BW_RECODEOPTIONS RecodeOptions;
    BW_RECODE_PREFERENCES RecodePreferences;

	BOOL bReturn;

    /* open the registry profile */
    CaptureOptions.lpProf = VC_OpenProfileAccess("vtool", 0);
    if (CaptureOptions.lpProf == NULL) {
        return FALSE;
    }

	/* init the capture options structure */
	CaptureOptions.ulStructSize		 = sizeof(BW_CAPTUREOPTIONS);
    CaptureOptions.nMode			 = MODE_MODIFY_PLAYBACKTARGET;
    CaptureOptions.nIsBroadway		 = TRUE;
	CaptureOptions.nPrivateInterface = TRUE;
    CaptureOptions.hWndParent        = NULL;

	CapturePreferences.ulStructSize	 = sizeof(BW_CAPTURE_PREFERENCES);
	CaptureOptions.lpPreferences	 = &CapturePreferences;

	/* init the recode options structure */
	RecodeOptions.ulStructSize		 = sizeof(BW_RECODEOPTIONS);
    RecodeOptions.nMode				 = MODE_DISPLAY_DIALOG;
    RecodeOptions.hWndParent		 = NULL;

	RecodePreferences.ulStructSize	 = sizeof(BW_RECODE_PREFERENCES);
	RecodeOptions.lpPreferences		 = &RecodePreferences;

	lstrcpyn (RecodePreferences.EncodePlaybackTarget, lpszTarget, sizeof(RecodePreferences.EncodePlaybackTarget));

	/* set the new target */
	bReturn = BW_GetCaptureOptions(NULL, &CaptureOptions, &RecodeOptions, 0);

	/* if the hardware is available - configure now */
	if (bReturn && lpHw != NULL) {
	    BW_CAPTURE_CONFIG Config;
	    
		Config.ulStructSize = sizeof(BW_CAPTURE_CONFIG);
		Config.lpProf		= CaptureOptions.lpProf;
		Config.nMode		= MODE_FETCH_SETTINGS;
	    Config.nIsBroadway	= TRUE;
		Config.nWndParent   = (long)NULL;

		bReturn = HW_ConfigFormat(lpHw->vh, (LPCONFIG_INFO)&Config);
		if (bReturn && lpHw->hWndCap != NULL) {
			lpHw->bNoDialog = TRUE;
			bcapDlgVideoFormat (lpHw->hWndCap);
			lpHw->bNoDialog = FALSE;
		}
	} 

	VC_CloseProfileAccess(CaptureOptions.lpProf);
	return bReturn;
}


/*
 * GetCapturePlayBackType
 *
 * This function reports back the current
 * capture play back type
 */
BOOL 
WINAPI BW_GetCapturePlayBackType (LPTSTR lpszType, UINT cbSize)
{
	BOOL bReturn;

    /* open the registry profile */
    LPVC_PROFILE_INFO lpProf = VC_OpenProfileAccess("vtool", 0);
    if (lpProf == NULL) {
        return FALSE;
    }

	bReturn = BW_GetCaptureEncodeType (lpProf, lpszType, cbSize);

	VC_CloseProfileAccess(lpProf);

	return bReturn;
}


/*
 * GetCapturePlayBackType
 *
 * This function sets the current
 * capture play back type
 */
BOOL 
WINAPI BW_SetCapturePlayBackType (LPTSTR lpszType)
{
	BOOL bReturn;

    /* open the registry profile */
    LPVC_PROFILE_INFO lpProf = VC_OpenProfileAccess("vtool", 0);
    if (lpProf == NULL) {
        return FALSE;
    }

	bReturn = BW_SetCaptureEncodeType (lpProf, lpszType);

	VC_CloseProfileAccess(lpProf);

	return bReturn;
}


/*
 * GetCompressPlayBackTarget
 *
 * This function reports back the current
 * compression play back target
 */
BOOL 
WINAPI BW_GetCompressPlayBackTarget (LPTSTR lpszTarget, UINT cbSize)
{
    return BW_GetRecodeType (lpszTarget, cbSize);
}


/*
 * SetCompressPlayBackTarget
 *
 * This function sets the current
 * compression play back target and updates
 * the registry seting to reflect the last set
 * options for that target
 */
BOOL 
WINAPI BW_SetCompressPlayBackTarget (LPTSTR lpszTarget)
{
    return BW_SetRecodeType (lpszTarget);
}


/*
 * Compression Configuration.
 *
 * This function perform device-dependent recompression setup affecting 
 * the target format.
 */
BOOL WINAPI
BW_ConfigRecompress (HWND hWndParent)
{
    BW_RECODEOPTIONS RecodeOptions;
    BW_RECODE_PREFERENCES RecodePreferences;
	BOOL bReturn;

	/* init the recode options structure */
	RecodeOptions.ulStructSize		 = sizeof(BW_RECODEOPTIONS);
    RecodeOptions.nMode				 = MODE_DISPLAY_DIALOG;
    RecodeOptions.hWndParent		 = hWndParent;

	RecodePreferences.ulStructSize	 = sizeof(BW_RECODE_PREFERENCES);
	RecodeOptions.lpPreferences		 = &RecodePreferences;

	/* pop up the config dialog */
	bReturn = BW_GetRecodeOptions(&RecodeOptions);

    return bReturn;
}


/*
 * CompressEstimate
 *
 * This function reports back the total stream data rate in bytes/second
 */
BOOL WINAPI BW_CompressEstimate (DWORD *pEstimate)
{
    BW_RECODEOPTIONS RecodeOptions;
    BW_RECODE_PREFERENCES RecodePreferences;

	/* init the recode options structure */
	RecodeOptions.ulStructSize		 = sizeof(BW_RECODEOPTIONS);
    RecodeOptions.nMode				 = MODE_FETCH_SETTINGS;
    RecodeOptions.hWndParent		 = NULL;

	RecodePreferences.ulStructSize	 = sizeof(BW_RECODE_PREFERENCES);
	RecodeOptions.lpPreferences		 = &RecodePreferences;

    if (!BW_GetRecodeOptions(&RecodeOptions)) {
        return FALSE;
    }

	/* get the current setting */
    *pEstimate = (RecodePreferences.nCompressionEstimate / 8);    

    return TRUE;
}

/*
 * Stream Recompression
 *
 * This function will recompress the input Avi streams to full mpeg. 
 * The resulting mpeg system stream is written to the output MpegFile.
 * The function will not return until the recompression is complete 
 * or an error is encountered.  
 *
 * This function will fail if ther are no idle boards in the system.
 *
 */
DWORD WINAPI BW_RecompressStreams (
	PAVISTREAM pAudioStream,
	PAVISTREAM pVideoStream,
    LPTSTR  pMpegFileName, 
    HWND hWndParent,
    int nAudioDelay
)
{
    BW_RECODEOPTIONS RecodeOptions;
    BW_RECODE_PREFERENCES RecodePreferences;
    DEVID DevID;

	/* init the recode options structure */
	RecodeOptions.ulStructSize		 = sizeof(BW_RECODEOPTIONS);
    RecodeOptions.nMode				 = MODE_FETCH_SETTINGS;
    RecodeOptions.hWndParent		 = hWndParent;

	RecodePreferences.ulStructSize	 = sizeof(BW_RECODE_PREFERENCES);
	RecodeOptions.lpPreferences		 = &RecodePreferences;

    if (!BW_GetRecodeOptions(&RecodeOptions)) {
        return VT_ERR_HARDWARE_NOT_FOUND;
    }

	RTINFO RTInfo;
	FillMemory (&RTInfo, sizeof(RTINFO), 0);

	/* 
	 * Get the available disk space... 
	 */
	long DiskSpaceInKB = RT_GetFreeDiskSpaceInKB(&RTInfo, pMpegFileName);

	long VideoSpaceInKB = 0;
	long AudioSpaceInKB = 0;

	/* 
	 * lets verify we have enough disk space for the compression
	 */
	if (pVideoStream != NULL && RecodePreferences.nEncodeVideoEnable) {
		
		AVIFileInit();
		long time = AVIStreamSampleToTime(pVideoStream, AVIStreamLength(pVideoStream) - 1);
		AVIFileExit();

		VideoSpaceInKB = time * (RecodePreferences.nEncodeVideoBitRate/8000); 
		VideoSpaceInKB /= 1024;
	}

	if (pAudioStream != NULL && RecodePreferences.nEncodeAudioEnable) {
		
		AVIFileInit();
		long time = AVIStreamSampleToTime(pAudioStream, AVIStreamLength(pAudioStream) - 1);
		AVIFileExit();

		AudioSpaceInKB = time * (RecodePreferences.nEncodeAudioBitRate/8000); 
		AudioSpaceInKB /= 1024;
	}

	/* add audio and video together and add 5 for good measure */
	long SpaceRequiredInKB = (AudioSpaceInKB+VideoSpaceInKB) + (1024*5);

	if (DiskSpaceInKB <= SpaceRequiredInKB) {
        return IDS_ERROR_SPACE_COMPRESS;
	}

    /* try to initialize the video tool library for any device */
    if (VT_OpenDevice(TEXT("vtool"), 0, &DevID) != VT_NO_ERROR) {
        return VT_ERR_HARDWARE_NOT_FOUND;
    }


	BOARD_STATE HWState;

	/* 
	 * before we go any further, verify we can take over ownership
	 * of the board cleanly.
	 */
	if (VT_GetHardwareState(DevID, (DWORD*)&HWState) != VT_NO_ERROR) {
        VT_DeleteDevice(DevID);
        return VT_ERR_HARDWARE_NOT_FOUND;
    } else if (!(HWState ==INITIALIZED || HWState == CODEC_INITIALIZED)) {
        VT_DeleteDevice(DevID);
        return VT_ERR_HARDWARE_NOT_FOUND;
	}


	PUCHAR pLinearAddr;
    DWORD pPhysAddr, dwMemoryBytes;

    /* get device memory information */
    if (VT_GetHardwareBuffer(DevID, &pPhysAddr, (void**)&pLinearAddr, NULL, &dwMemoryBytes)) {
        VT_DeleteDevice(DevID);
        return VT_ERR_HARDWARE_NOT_FOUND;
    }

    /* create the recode class */
    HANDLE pRecode = AllocRecode (DevID 
                                  ,pLinearAddr
                                  ,pPhysAddr
                                  ,dwMemoryBytes
                                  );

	/* get all the recode parameters */
    int VideoDataRate = RecodePreferences.nEncodeVideoBitRate;
    BOOL UseQSIF = RecodePreferences.nLargeImage ?FALSE:TRUE;

    int AudioDataRate = RecodePreferences.nEncodeAudioBitRate/1000;
	BOOL AudioMode = BW_AudioToMode (RecodePreferences.nEncodeAudioEnable, RecodePreferences.nEncodeAudioMode);
 
    double StreamDataRate = RecodeOptions.nEncodeUseOptimumRate ?0.0:(double)RecodeOptions.nEncodeStreamDataRate; 
    int PackSize = RecodeOptions.nEncodePackSize;
    int PacketsPerPack = RecodeOptions.nEncodePacketsPerPack;
	EStreamMixType StreamMixType = (EStreamMixType)RecodeOptions.nStreamMixType;
    
    /* delay given in milliseconds - convert to seconds */
    double AudioDelay = (double)nAudioDelay / 1000.0;

    /* do the recode!!! */
	ECODE status = CompressStreams(pRecode,
								   pAudioStream,
								   pVideoStream,
								   pMpegFileName,
								   hWndParent,
								   AudioMode,
								   AudioDataRate,
								   VideoDataRate,
								   UseQSIF,
								   StreamMixType,
								   StreamDataRate,
								   PackSize,
								   PacketsPerPack,
								   AudioDelay); 

	/* clean up and return */
    FreeRecode(pRecode);
    VT_DeleteDevice(DevID);

    return status;
}



/*
 * Recompression
 *
 * This function will recompress the input AviFile to full mpeg. 
 * The resulting mpeg system stream is written to the output MpegFile.
 * The function will not return until the recompression is complete 
 * or an error is encountered.  
 *
 * This function will fail if ther are no idle boards in the system.
 *
 */
DWORD WINAPI BW_Recompress (
    LPTSTR pAviFileName, 
    LPTSTR  pMpegFileName, 
    HWND hWndParent,
    int nAudioDelay
)
{
    BW_RECODEOPTIONS RecodeOptions;
    BW_RECODE_PREFERENCES RecodePreferences;
    DEVID DevID;

	/* init the recode options structure */
	RecodeOptions.ulStructSize		 = sizeof(BW_RECODEOPTIONS);
    RecodeOptions.nMode				 = MODE_FETCH_SETTINGS;
    RecodeOptions.hWndParent		 = hWndParent;

	RecodePreferences.ulStructSize	 = sizeof(BW_RECODE_PREFERENCES);
	RecodeOptions.lpPreferences		 = &RecodePreferences;

    if (!BW_GetRecodeOptions(&RecodeOptions)) {
        return VT_ERR_HARDWARE_NOT_FOUND;
    }

    /* try to initialize the video tool library for any device */
    if (VT_OpenDevice(TEXT("vtool"), 0, &DevID) != VT_NO_ERROR) {
        return VT_ERR_HARDWARE_NOT_FOUND;
    }


	BOARD_STATE HWState;

	/* 
	 * before we go any further, verify we can take over ownership
	 * of the board cleanly.
	 */
	if (VT_GetHardwareState(DevID, (DWORD*)&HWState) != VT_NO_ERROR) {
        VT_DeleteDevice(DevID);
        return VT_ERR_HARDWARE_NOT_FOUND;
    } else if (!(HWState ==INITIALIZED || HWState == CODEC_INITIALIZED)) {
        VT_DeleteDevice(DevID);
        return VT_ERR_HARDWARE_NOT_FOUND;
	}


	PUCHAR pLinearAddr;
    DWORD pPhysAddr, dwMemoryBytes;

    /* get device memory information */
    if (VT_GetHardwareBuffer(DevID, &pPhysAddr, (void**)&pLinearAddr, NULL, &dwMemoryBytes)) {
        VT_DeleteDevice(DevID);
        return VT_ERR_HARDWARE_NOT_FOUND;
    }

    /* create the recode class */
    HANDLE pRecode = AllocRecode (DevID 
                                  ,pLinearAddr
                                  ,pPhysAddr
                                  ,dwMemoryBytes
                                  );

	/* get all the recode parameters */
    int VideoDataRate = RecodePreferences.nEncodeVideoBitRate;
    BOOL UseQSIF = RecodePreferences.nLargeImage ?FALSE:TRUE;

    int AudioDataRate = RecodePreferences.nEncodeAudioBitRate/1000;
	BOOL AudioMode = BW_AudioToMode (RecodePreferences.nEncodeAudioEnable, RecodePreferences.nEncodeAudioMode);
 
    double StreamDataRate = RecodeOptions.nEncodeUseOptimumRate ?0.0:(double)RecodeOptions.nEncodeStreamDataRate; 
    int PackSize = RecodeOptions.nEncodePackSize;
    int PacketsPerPack = RecodeOptions.nEncodePacketsPerPack;
	EStreamMixType StreamMixType = (EStreamMixType)RecodeOptions.nStreamMixType;
    
    /* delay given in milliseconds - convert to seconds */
    double AudioDelay = (double)nAudioDelay / 1000.0;

    /* do the recode!!! */
	ECODE status = Compress(pRecode,
                            pAviFileName,
                            pMpegFileName,
                            hWndParent,
                            AudioMode,
                            AudioDataRate,
                            VideoDataRate,
                            UseQSIF,
                            StreamMixType,
							StreamDataRate,
                            PackSize,
                            PacketsPerPack,
                            AudioDelay); 

	/* clean up and return */
    FreeRecode(pRecode);
    VT_DeleteDevice(DevID);

    return status;
}


/*
 * RecompressBegin
 *
 * This function will spawn a hidden process to recompress the input 
 * AviFile to full mpeg. The resulting mpeg system stream is written 
 * to the output MpegFile. The function will not return until the 
 * recompression is complete or an error is encountered.  
 *
 * This function will fail if ther are no idle boards in the system.
 *
 */

#ifdef _USE_HIDDEN_APP_

BOOL WINAPI BW_RecompressBegin (
    LPTSTR pAviFileName, 
    LPTSTR  pMpegFileName, 
    HWND hWndParent, 
    int nAudioDelay
)
{
    PROCESS_INFORMATION pi = {0};
    STARTUPINFO sui;
    char szText[20];

    LPSTR szPathName = new char [MAX_COMMANDLINE];
	
	/* create the command line */
    GetSystemDirectory (szPathName, MAX_PATHNAME);
    lstrcat (szPathName, "\\BWRECODE.EXE \"");
    lstrcat (szPathName, pAviFileName);
    lstrcat (szPathName, "\",\"");
    lstrcat (szPathName, pMpegFileName);        
    wsprintf (szText, ",\"%ld\"", hWndParent);
    lstrcat (szPathName, szText);
    wsprintf (szText, ",\"%ld\"", nAudioDelay);
    lstrcat (szPathName, szText);

    ZeroMemory(&sui,sizeof (STARTUPINFO));
    sui.cb = sizeof (STARTUPINFO);
    sui.dwFlags = STARTF_FORCEOFFFEEDBACK;


	BOOL bEnable, bIsWindow = FALSE;
	
	/* if we got a valid window handle - disable it now!!! */
	if (hWndParent != NULL && IsWindow (hWndParent)) {
		bIsWindow = TRUE;

		bEnable = IsWindowEnabled (hWndParent);
		EnableWindow(hWndParent, FALSE);
	}

	/* call the hidden app to recompresss */
    if (!CreateProcess (NULL
                        ,szPathName
                        ,NULL
                        ,NULL
                        ,FALSE
                        ,DETACHED_PROCESS
                        ,NULL
                        ,NULL
                        ,&sui
                        ,&pi
                        )) {

		/* if the app doesn't exist - inform the user */
        char szPath[MAX_PATHNAME];
        GetSystemDirectory (szPath, sizeof(szPath));
        wsprintf (szPathName, "Required program file missing: %s\\bwrecode.exe.", szPath);
        
        MessageBox (hWndParent
                    ,szPathName
                    ,"Broadway Compression Error" 
                    , MB_ICONEXCLAMATION|MB_OK
                    );

        delete szPathName;
	
		/* remember to re-enable the window */
		if (bIsWindow) {
			EnableWindow(hWndParent, bEnable);
		}
        
		return FALSE;
    }

    delete szPathName;
    
	/* wait for the process to complete */
    MSG Msg;
    while (WaitForSingleObject (pi.hProcess, 0) != WAIT_OBJECT_0)  {
        if (PeekMessage (&Msg, NULL, 0, 0, PM_REMOVE)) {
            {
            TranslateMessage (&Msg);
            DispatchMessage (&Msg);
            }
        }             
    }

	/* re-enable the window */
	if (bIsWindow) {
		EnableWindow(hWndParent, bEnable);
	}

    return TRUE; 
 }

#else

BOOL WINAPI BW_RecompressBegin (
    LPTSTR pAviFileName, 
    LPTSTR  pMpegFileName, 
    HWND hWndParent, 
    int nAudioDelay
)
{
    char  szCaption[MAX_PATH];
    char  szMessage[MAX_PATH*2];
    DWORD status;

	/* get the caption for the message boxes */
    LoadString (ghDLLInst, IDS_BROADWAY_ERROR, szCaption, sizeof(szCaption));

    /* verify the input file name */
    if (pAviFileName == NULL) {
	    LoadString (ghDLLInst, IDS_INVALID_AVIFILE, szMessage, sizeof(szMessage));
	    MessageBox (NULL, szMessage, szCaption , MB_ICONEXCLAMATION|MB_OK);
		return FALSE;
    }                           

	/* verify the output file name */
    if (pMpegFileName == NULL) {
	    LoadString (ghDLLInst, IDS_INVALID_MPEGFILE, szMessage, sizeof(szMessage));
	    MessageBox (NULL, szMessage, szCaption , MB_ICONEXCLAMATION|MB_OK);
		return FALSE;
	}

	/* 
	 * lets verify we have a valid avi file of greater than one second.  
	 * if the avi file does not exist, fall through and let the software
	 * further down the pipe handle it. If it's les than one second, 
	 * dislay a warning and fail now.
	 */
	if (lstrlen (pAviFileName) > 0)	{
		AVIFILEINFO fi;
		double filesize, fps;
		BOOL bFailed = FALSE;
		PAVIFILE pfile = NULL;

		/* init the avifile dll */
		AVIFileInit();

		/* open the file */
		if (AVIFileOpen(&pfile, pAviFileName, OF_READ, NULL) == 0) {

			/* get the file information */
			if (AVIFileInfo(pfile, &fi, sizeof(AVIFILEINFO)) == 0) {
	
				/* calculate the file size in seconds */
				fps = (double)((double)fi.dwRate/(double)fi.dwScale);
				filesize =  (double)( (double)fi.dwLength/fps);
				
				/* file size must be one second or greater */
				if (filesize < 1.0) {
					bFailed = TRUE;
				    LoadString (ghDLLInst, IDS_FILE_TOO_SHORT, szMessage, sizeof(szMessage));
				    MessageBox (NULL, szMessage, szCaption , MB_ICONEXCLAMATION|MB_OK);
				}
			}
		}
		
		if (pfile != NULL) {
			AVIFileRelease(pfile);
		}

		AVIFileExit();

		if (bFailed) {
			return 0;
		}
	}

    /* Do the compression!!! */
    status = BW_Recompress (pAviFileName, pMpegFileName, hWndParent, nAudioDelay);

    /* any errors??? */
    if (status != 0) {
        char szError[MAX_PATH];

        switch (status) {
            case IDS_VT_ERR_CANT_OPEN_AVI_FILE:
                LoadString (ghDLLInst, status, szError, sizeof(szError));
                wsprintf (szMessage, szError, pAviFileName);
                break;

            case IDS_VT_ERR_CANT_OPEN_OUTPUT_FILE:
                LoadString (ghDLLInst, status, szError, sizeof(szError));
                wsprintf (szMessage, szError, pMpegFileName);
                break;

            default:
                if (LoadString (ghDLLInst, status, szMessage, sizeof(szMessage)) == 0) {
                    LoadString (ghDLLInst, IDS_UNKNOWNERROR, szError, sizeof(szError));
                    wsprintf (szMessage, szError, status);
                }
        }

        MessageBox (hWndParent ,szMessage ,szCaption , MB_ICONEXCLAMATION|MB_OK);
    }
        
    return TRUE; 
 }

#endif


/*
 * Stream Recompression
 *
 * This function will recompress the input Avi streams to full mpeg. 
 * The resulting mpeg system stream is written to the output MpegFile.
 * The function will not return until the recompression is complete 
 * or an error is encountered.  
 *
 * This function will fail if ther are no idle boards in the system.
 *
 */
BOOL WINAPI BW_RecompressStreamsBegin (
	PAVISTREAM pAudioStream,
	PAVISTREAM pVideoStream,
    LPTSTR  pMpegFileName, 
    HWND hWndParent, 
    int nAudioDelay
)
{
    char  szCaption[MAX_PATH];
    char  szMessage[MAX_PATH*2];
    DWORD status;

	/* get the caption for the message boxes */
    LoadString (ghDLLInst, IDS_BROADWAY_ERROR, szCaption, sizeof(szCaption));

    /* verify the we hav at least on input stream */
    if (pAudioStream == NULL && pVideoStream == NULL) {
	    LoadString (ghDLLInst, IDS_INVALID_AVIFILE, szMessage, sizeof(szMessage));
	    MessageBox (NULL, szMessage, szCaption , MB_ICONEXCLAMATION|MB_OK);
		return FALSE;
    }                           

	/* verify the output file name */
    if (pMpegFileName == NULL) {
	    LoadString (ghDLLInst, IDS_INVALID_MPEGFILE, szMessage, sizeof(szMessage));
	    MessageBox (NULL, szMessage, szCaption , MB_ICONEXCLAMATION|MB_OK);
		return FALSE;
	}

	/* 
	 * lets verify we have a valid video stream of greater than one second.  
	 * if the video stream does not exist, fall through and let the software
	 * further down the pipe handle it. If it's les than one second, 
	 * dislay a warning and fail now.
	 */
	if (pVideoStream != NULL)	{

		BOOL bFailed = FALSE;

		AVIFileInit();
		long time = AVIStreamSampleToTime(pVideoStream, AVIStreamLength(pVideoStream) - 1);
		AVIFileExit();

		/* file size must be one second or greater */
		if (time < 1000) {
			bFailed = TRUE;
		    LoadString (ghDLLInst, IDS_FILE_TOO_SHORT, szMessage, sizeof(szMessage));
		    MessageBox (NULL, szMessage, szCaption , MB_ICONEXCLAMATION|MB_OK);
		}
		
		if (bFailed) {
			return 0;
		}
	}

    /* Do the compression!!! */
    status = BW_RecompressStreams (pAudioStream, pVideoStream, pMpegFileName, hWndParent, nAudioDelay);

    /* any errors??? */
    if (status != 0) {
        char szError[MAX_PATH];

        switch (status) {
			/*
            case IDS_VT_ERR_CANT_OPEN_AVI_FILE:
                LoadString (ghDLLInst, status, szError, sizeof(szError));
                wsprintf (szMessage, szError, pAviFileName);
                break;
			*/
            case IDS_VT_ERR_CANT_OPEN_OUTPUT_FILE:
                LoadString (ghDLLInst, status, szError, sizeof(szError));
                wsprintf (szMessage, szError, pMpegFileName);
                break;

            default:
                if (LoadString (ghDLLInst, status, szMessage, sizeof(szMessage)) == 0) {
                    LoadString (ghDLLInst, IDS_UNKNOWNERROR, szError, sizeof(szError));
                    wsprintf (szMessage, szError, status);
                }
        }

        MessageBox (hWndParent ,szMessage ,szCaption , MB_ICONEXCLAMATION|MB_OK);
    }
        
    return TRUE; 
 }

