/*
 * capwnd.c
 *
 *
 * 32-bit Video Tool driver
 * capture window subclass routines
 * for Data Translation Video Tool.
 *
 * Mark Vrabel, October 95
 */

#include <vcdriver.h>
#include "hardware.h"
#include <bwdlgs.h>
#include <bwuser.h>
#include <rcuser.h>
#include "realtime.h"

LPHWINFO glpHwTest = NULL;
FARPROC gfpOrigCapWndProc = NULL; 

extern LPHWINFO glpHwCurrent;


LRESULT CALLBACK 
CapWndSubclassProc (HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    LPHWINFO lpHw = glpHwCurrent;
	FARPROC fpOrigCapWndProc = gfpOrigCapWndProc;
	LRESULT lResult;

	/* 
	 * this should never happen!!! 
	 */
    if (fpOrigCapWndProc == NULL) {
		return 0;
	}

	/* if the info is bad we're hosed */
	if (IsBadHWINFO(lpHw)) {
        return CallWindowProc(fpOrigCapWndProc, hWnd, uMsg, wParam, lParam);
    }

	/* deal with the message */
    switch (uMsg) {

/* 
 * the following functionality is not supported
 * when the capture mode is real time mpeg
 */
		case WM_CAP_SET_CALLBACK_FRAME:
		case WM_CAP_FILE_SAVEDIBA:
		case WM_CAP_FILE_SAVEDIBW:
		case WM_CAP_FILE_SET_INFOCHUNK:
		case WM_CAP_EDIT_COPY:
		case WM_CAP_SET_SCALE:
		case WM_CAP_GRAB_FRAME:
		case WM_CAP_GRAB_FRAME_NOSTOP:
		case WM_CAP_SINGLE_FRAME_OPEN:
		case WM_CAP_SINGLE_FRAME_CLOSE:
		case WM_CAP_SINGLE_FRAME:
		case WM_CAP_SET_SCROLL:
		case WM_CAP_PAL_OPENA:
		case WM_CAP_PAL_SAVEA:
		case WM_CAP_PAL_OPENW:
		case WM_CAP_PAL_SAVEW:
		case WM_CAP_PAL_PASTE:
		case WM_CAP_PAL_AUTOCREATE:
		case WM_CAP_PAL_MANUALCREATE:
		case WM_CAP_DLG_VIDEOCOMPRESSION:
			
/* NOTE: Seems to work ok if we let vfw handle these!!!
 *
 *			if (lpHw->CaptureMode != CAPMODE_IFRAME) {
 *				return (FALSE);
 *			}
 */
			break;           

/* 
 * the following functionality are handled by the vidcap
 * window class regardless of the capture mode
 */

	/* 
	 * open/close, capabilities and user data functions 
	 */
		case WM_CAP_DRIVER_CONNECT:
			/* 
			 * we are already connected - don't connect again 
			 */
			return FALSE;
	
		case WM_CAP_DRIVER_DISCONNECT:
			/* 
			 * The rules of the private interface say you can't
			 * disconnect the driver without first releaseing the 
			 * private interface. We'll let 'em do it, but there 
			 * going may generate an exception!!!
			 */
			RT_ReleaseCaptureWindow(hWnd);
			break;

		case WM_CAP_GET_USER_DATA:
			break;		
		
		case WM_CAP_SET_USER_DATA:
			break;

		case WM_CAP_DRIVER_GET_CAPS:
			break;        

	/*
	 * Driver name function
	 */
		case WM_CAP_DRIVER_GET_NAMEA:
		case WM_CAP_DRIVER_GET_NAMEW:
			break;      

	/*
	 * Dialog Box functions
	 */
		case WM_CAP_DLG_VIDEOFORMAT:
			break;        

		case WM_CAP_DLG_VIDEOSOURCE:
			break;        
		
		case WM_CAP_DLG_VIDEODISPLAY:
			break;       
		
	/* 
	 * file functions
	 */
		case WM_CAP_FILE_SET_CAPTURE_FILEA:
		case WM_CAP_FILE_SET_CAPTURE_FILEW:
			return RT_CapFileSetCaptureFile (lpHw, (LPSTR)lParam);
			break; 
		
		case WM_CAP_FILE_GET_CAPTURE_FILEA:
		case WM_CAP_FILE_GET_CAPTURE_FILEW:
			return RT_CapFileGetCaptureFile (lpHw, (LPSTR)lParam, wParam);
		
		case WM_CAP_FILE_SAVEASA:
		case WM_CAP_FILE_SAVEASW:
			break;           
		
		case WM_CAP_FILE_ALLOCATE:
			break;          

	/*
	 * Preview functions
	 */
		case WM_CAP_SET_PREVIEWRATE:
			break;            
		
		case WM_CAP_SET_PREVIEW:
			return RT_CapPreview (lpHw, (BOOL)wParam);
		
	/* 
	 * overlay functions
	 */
		case WM_CAP_SET_OVERLAY:
			break;            
		
	/*
	 * MCI command functions
	 */
		case WM_CAP_SET_MCI_DEVICEA:
		case WM_CAP_SET_MCI_DEVICEW:
			break;        
		
		case WM_CAP_GET_MCI_DEVICEA:
		case WM_CAP_GET_MCI_DEVICEW:
			break;        

	/*
	 * Set/Get format functions
	 */
		case WM_CAP_SET_VIDEOFORMAT:
			break;
				   
		case WM_CAP_SET_AUDIOFORMAT:
			break;
		
		case WM_CAP_GET_AUDIOFORMAT:
			break;

	/* 
	 * Set/Get Capture Setup 
	 */
		case WM_CAP_SET_SEQUENCE_SETUP:
			break;
		
		case WM_CAP_GET_SEQUENCE_SETUP:
			break;


/* 
 * the following functionality are handled by the vidcap
 * window class if the capture mode is editable I-frame,
 * and handled directly if the capture mode is real time 
 * mpeg.
 */
	/*
	 * Set callback functions
	 */
		case WM_CAP_SET_CALLBACK_ERRORW:
		case WM_CAP_SET_CALLBACK_ERRORA:
			return RT_CapSetCallbackOnError(lpHw, (FARPROC)lParam);

		case WM_CAP_SET_CALLBACK_STATUSW:
		case WM_CAP_SET_CALLBACK_STATUSA:
			return RT_CapSetCallbackOnStatus(lpHw, (FARPROC)lParam);

		case WM_CAP_SET_CALLBACK_YIELD:
			return RT_CapSetCallbackOnYield(lpHw, (FARPROC)lParam);
		
		case WM_CAP_SET_CALLBACK_VIDEOSTREAM:
			return RT_CapSetCallbackOnVideoStream(lpHw, (FARPROC)lParam);
		
		case WM_CAP_SET_CALLBACK_WAVESTREAM:
			return RT_CapSetCallbackOnWaveStream(lpHw, (FARPROC)lParam);
		
		case WM_CAP_SET_CALLBACK_CAPCONTROL:
			return RT_CapSetCallbackOnCapControl(lpHw, (FARPROC)lParam);

	/*
	 * Driver version function
	 */
		case WM_CAP_DRIVER_GET_VERSIONA:
		case WM_CAP_DRIVER_GET_VERSIONW:
			{
				OSVERSIONINFO VerInfo;
			
				/* win nt seems to have problems reporting the correct version */
				VerInfo.dwOSVersionInfoSize = sizeof (OSVERSIONINFO);

				if (GetVersionEx(&VerInfo)) {
					if (VerInfo.dwPlatformId != VER_PLATFORM_WIN32_WINDOWS) {
						return RT_CapDriverGetVersion (wParam, (LPSTR)lParam);
					}
				}
			}
			break;    
						
	/* 
	 * status function
	 */
		case WM_CAP_GET_STATUS:
			return RT_CapGetStatus(lpHw, (CAPSTATUS*)lParam, wParam);
	
		case WM_CAP_GET_VIDEOFORMAT:
			return RT_CapGetVideoFormat(lpHw, (LPBITMAPINFOHEADER)lParam, wParam);

	/* 
	 * Stream capture functions
	 */
		case WM_CAP_SEQUENCE:

			if ((HWND)wParam == hWnd && (LPHWINFO)lParam == lpHw && lpHw->fCaptureInit) {
				return RT_CaptureLoop (lpHw);
			} else {
				return RT_CapCaptureSequence(lpHw);
			}
		
		case WM_CAP_SEQUENCE_NOFILE:
			return RT_CapCaptureSequenceNoFile(lpHw);
		
		case WM_CAP_STOP:
			return RT_CapCaptureStop(lpHw);
		
		case WM_CAP_ABORT:
			return RT_CapCaptureAbort(lpHw);

	}
	
	lResult = CallWindowProc(fpOrigCapWndProc, hWnd, uMsg, wParam, lParam);
	return lResult;
}


LRESULT CALLBACK capVideoFrameCallback(HWND hWnd, LPVIDEOHDR lpVHdr)
{
    LPHWINFO lpHw = glpHwTest;
	glpHwTest = NULL;
	
	/* verify hardware info */
    if (IsBadHWINFO(lpHw) || lpHw != glpHwCurrent) {
		capSetUserData(hWnd, 0);
    } else {
		capSetUserData(hWnd, (long)lpHw);
	}

	return TRUE;
}

				  
BOOL RT_GetCaptureWindow(HWND hWndCap, CAPCALLBACKS *lpCallbacks)
{										
	long lCapWndUserData;
    LPHWINFO lpHw;
	char buf[256];
	CAPSTATUS capstatus;

	/* 
	 * verify that this is a capture window and it 
	 * currently has this driver open for capture
	 */
	capDriverGetName(hWndCap, buf, 80);
	if (lstrcmp ("Broadway MPEG Capture/Compression", buf)) {
		return FALSE;
	}			
	
	/* get the current status so we can restore it when we're done */
	capGetStatus(hWndCap,&capstatus, sizeof(capstatus));

	/* can't initialize while captureing */
	if (capstatus.fCapturingNow) {
		return FALSE;
	} 

	/* 
	 * a tricky little way to get the hardware pointer for the 
	 * board associated with this capture window. HW_Frame() in
	 * hardware.c always sets a global hardware pointer to it's 
	 * hardware pointer. The callback reads this and stores it in 
	 * the capwnd's user data.
	 */
	lCapWndUserData = capGetUserData(hWndCap);

	capPreview (hWndCap, TRUE);
	capOverlay (hWndCap, FALSE);

	capSetUserData(hWndCap, 0);
	capSetCallbackOnFrame(hWndCap, capVideoFrameCallback);
	capGrabFrame (hWndCap);
	capSetCallbackOnFrame(hWndCap, NULL);

    /* the user data should now set to the hardware pointer */
	lpHw = (LPHWINFO)capGetUserData(hWndCap);

	/* restore the user data and overlay/preview state */
	capSetUserData(hWndCap, lCapWndUserData);

	capPreview (hWndCap, capstatus.fLiveWindow);
	capOverlay (hWndCap, capstatus.fOverlayWindow);

    /* verify hardware info */
    if (IsBadHWINFO(lpHw)) {
        return FALSE;
    }

	/* 
	 * store the info
	 */
    gfpOrigCapWndProc = (FARPROC)GetWindowLong(hWndCap, GWL_WNDPROC);

	lpHw->hWndCap = hWndCap;

	/* we need events to signal when we're done setup and capture */
	lpHw->hCaptureComplete = CreateEvent (NULL, FALSE, FALSE, NULL);

	/* subclass the capture window */
    SetWindowLong(hWndCap, GWL_WNDPROC, (LONG)CapWndSubclassProc);

	/* 
	 * set all the callback functions so we can trap the calls in the 
	 * subclassed window proc and record them appropriately
	 */
	if (lpCallbacks != NULL && !IsBadReadPtr (lpCallbacks, sizeof (CAPCALLBACKS))) {

		capSetCallbackOnError(hWndCap, lpCallbacks->lpfnErrorCallback);
		capSetCallbackOnStatus(hWndCap, lpCallbacks->lpfnStatusCallback);
		capSetCallbackOnYield(hWndCap, lpCallbacks->lpfnYieldCallback);
		capSetCallbackOnFrame(hWndCap, lpCallbacks->lpfnFrameCallback);
		capSetCallbackOnVideoStream(hWndCap, lpCallbacks->lpfnVideoStreamCallback);
		capSetCallbackOnWaveStream(hWndCap, lpCallbacks->lpfnWaveStreamCallback);
		capSetCallbackOnCapControl(hWndCap, lpCallbacks->lpfnCapControlCallback);
	}

	return TRUE;
}
							

BOOL RT_ReleaseCaptureWindow(HWND hWndCap)
{
    LPHWINFO lpHw = glpHwCurrent;
	
	/* 
	 * this should never happen!!! 
	 */
    if (gfpOrigCapWndProc == NULL) {
		return FALSE;
	}

	/* if the info is bad we're hosed */
	if (IsBadHWINFO(lpHw)) {
        return FALSE;
    } else if (lpHw->hWndCap != hWndCap) {
		return FALSE;
	}

	/* make sure capture is terminated */
	RT_CapCaptureAbort(lpHw);

	/* get rid of the capture events */
	CloseHandle (lpHw->hCaptureComplete);

	/* restore the capture window to it's original state */
    SetWindowLong(hWndCap, GWL_WNDPROC, (LONG)gfpOrigCapWndProc);

	/* reset the global window proc to NULL */
	gfpOrigCapWndProc = NULL; 
}
