#include <windows.h>
#include "xlate.h"
#include "fconv.h"


int FrameConverter::ClipInited = 0;
unsigned char FrameConverter::ClipTable[1024];	//table to clip to 8 bits
unsigned char *FrameConverter::clip;				//0 point of table;
short FrameConverter::CrToR[256];		//look up Cr to red value (YUV to RGB conversion)
short FrameConverter::CrToG[256];		//look up Cr to green value (YUV to RGB conversion)
short FrameConverter::CbToB[256];		//look up Cb to red blue (YUV to RGB conversion)
short FrameConverter::CbToG[256];		//look up Cb to red green (YUV to RGB conversion)


int FrameConverter::Init(int nhres,int nvres)
{

	xres=nhres;
	yres=nvres;
	hdisplay=nhres;
	vdisplay=nvres;
	return 0;
}

DWORD FrameConverter::GetOutputFrameSize()
{
	return (OutputBytesPerPixel*xres*yres);
}
DWORD FrameConverter::GetInputFrameSize()
{
	return (InputBytesPerPixel*xres*yres);
}

void FrameConverter::ProcessFrame(UCHAR *Output,const UCHAR *Input)
{
	int row;
	int isize=InputBytesPerPixel*hdisplay;
	int osize=OutputBytesPerPixel*hdisplay;
	int docols=hdisplay/PixelsPerProcess;
	if (!invert)
	{
		for (row=0;row<vdisplay;row++)
		{
				ProcessLine(Output,Input);
				Output+=osize;
				Input+=isize;
		}
	}
	else
	{
		Output+=osize*(vdisplay-1);
		for (row=0;row<vdisplay;row++)
		{
			ProcessLine(Output,Input);
			Output-=osize;
			Input+=isize;
		}
	}

}

FrameConverter::~FrameConverter()
{
}
FrameConverter::FrameConverter()
{
	Compression=0;
	OutputBytesPerPixel=2;
	InputBytesPerPixel=2;
	PixelsPerProcess=2;
	invert=TRUE;

	//init static clip table if needed
	if (!ClipInited) {
		clip = ClipTable + 384;
		for (int i=-384; i<640; i++) 
			clip[i] = (i<0) ? 0 : ((i>255) ? 255 : i);
		for (int val=0; val<256; val++) {
			CrToR[val] = (short)( 1.371*(val-128));
			CrToG[val] = (short)(-0.698*(val-128));
			CbToB[val] = (short)( 1.732*(val-128));
			CbToG[val] = (short)(-0.336*(val-128));
		}
		ClipInited = 1;
	}
}

FromRGBToYuv::FromRGBToYuv()
{
	OutputBytesPerPixel=2;
	InputBytesPerPixel=3;
	InputBitsPerPixel=24;
	OutputBitsPerPixel=16;
	PixelsPerProcess=2;
	invert=TRUE;
}

void FrameConverter::ProcessPixel(UCHAR *Output,const UCHAR *Input)
{
	
}

// rgb2playyuv422   Entry - pointer to pic data (3 bytes/pix = 6 bytes)
//                  Exit - 2 words of Aura playback YUV422


FromRGB32ToYuv::FromRGB32ToYuv()
{
	OutputBytesPerPixel=2;
	InputBytesPerPixel=3;
	PixelsPerProcess=2;
	InputBitsPerPixel=24;
	OutputBitsPerPixel=16;
	invert=TRUE;
}

FromRGB16ToYuv::FromRGB16ToYuv()
{
	OutputBytesPerPixel=2;
	InputBytesPerPixel=2;
	PixelsPerProcess=2;
	InputBitsPerPixel=16;
	OutputBitsPerPixel=16;
	invert=TRUE;
}

void FromRGB16ToYuv::ProcessPixel(UCHAR *Output,const UCHAR *Input)
{
    struct {UCHAR r,g,b;} p[2];
	unsigned short *in=(unsigned short *)Input; 
	for (int i=0;i<PixelsPerProcess;i++)
	{
		p[i].r=(*in &0x7c00) >> 7;
		p[i].g=(*in & 0x3e0) >> 2;
		p[i].b=(*in++ & 0x1f) << 3;
	}
	rgb2yuv422((const unsigned char*)&p, Output);
}




/*
void FromRGB32ToYuv::ProcessPixel(UCHAR *Output,const UCHAR *Input)
{
	rgb2yuv422((const unsigned char*)Input, (unsigned short*)Output);
}
*/
#if 0
void __inline FromRGBToYuv::rgb2yuv422(const unsigned char *lppic ,  unsigned short *w )
{
    unsigned int r, g, b;
    short y0, y1;
	double yt;
    double u;
    double v;

    b = *lppic++;
    g = *lppic++;
    r = *lppic++;
/*
    y0 = ( r * 30 + g * 59 + b * 11 + 50 ) / 100;

    u += ( (((int)b - y0) * 127) / 226 );
    v += ( (((int)r - y0) * 127) / 179 );
*/
#ifdef FULL_RANGE_VUV
	yt= (r * (77/256.0) + g* (150 /256.0) + b*(29/256.0)) ;
	u= -.1719*r  -0.3398*g +0.1517*b  ; // + 128? 
	v= 0.5117*r  -0.4297*g -0.0820*b ;
#else
	yt= 0.257*r + 0.504*g + 0.098*b +16;
	u= -0.148*r  -0.291*g+ +0.439*b ;
	v= 0.439*r  -0.368*g -0.071*b ;
	// u=(int)((b-yt)*0.5774+128.5);
	// v=(r-yt)*0.7294+128.5;
#endif
	y0=(short) (yt+0.5);	
	b = *lppic++;
    g = *lppic++;
    r = *lppic++;
#ifdef FULL_RANGE_VUV
	yt= (r * (77/256.0) + g* (150 /256.0) + b*(29/256.0)) ;

	u+= -.1719*r  -0.3398*g +0.1517*b+256.5  ; // + 128? 
	v+= 0.5117*r  -0.4297*g -0.0820*b+256.5 ;
	// u=(int)((b-yt)*0.5774+128.5);
	// v=(r-yt)*0.7294+128.5;
#else
	yt= 0.257*r + 0.504*g + 0.098*b +16;
	u+= -0.148*r  -0.291*g+ +0.439*b+256.5 ;
	// u=(int)((r-yt)*0.5774+128.5);
	v+= 0.439*r  -0.368*g -0.071*b +256.5;
#endif
	y1=(short) (yt+0.5);	

     *w++ = (y0<<8) + (((short)u>>1)&0xff);
     *w   = (y1<<8) + (((short)v>>1)&0xff);
}
#else
void __inline FromRGBToYuv::rgb2yuv422(const unsigned char *lppic ,  unsigned char *w )
/*
this function converts 2 pixels to yuv 422 format: u,y0,v,y1 -->  Cb,y0,Cr,y1
*/
{
    int r, g, b,r1,g1,b1;
    int y0, y1;
	int yt;
    int u;
    int v;

	//read rgb values for both pixels
    b = *lppic++;
    g = *lppic++;
    r = *lppic++;
	b1 = *lppic++;
    g1 = *lppic++;
    r1 = *lppic++;

	//compute lum values for each output pixel
	y0= (77*r  + 150*g  + 29*b ) >> 8;
	y1= (77*r1 + 150*g1 + 29*b1) >> 8;

	//average both pixels to compute the chrom value
	yt=y0+y1;
	u = clip[((591*(b+b1-yt)) >>11) + 128];
//	v = clip[((747*(r+r1-yt)) >>11) + 128];
	v = clip[((750*(r+r1-yt)) >>11) + 128];  // compensation from actual invert

	//output the 422 pixels
	*w++ = (unsigned char)u;
	*w++ = (unsigned char)y0;
	*w++ = (unsigned char)v;
	*w   = (unsigned char)y1;
}
/*
void __inline FromRGBToYuv::rgb2yuv422(const unsigned char *lppic ,  unsigned short *w )
{
    unsigned int r, g, b,r1,b1;
    int y0, y1;
	int yt;
    int u;
    int v;

    b = *lppic++;
    g = *lppic++;
    r = *lppic++;

	y0= (int)(0.257*r + 0.504*g + 0.098*b +16.5);

	b1 = *lppic++;
    g = *lppic++;
    r1 = *lppic++;

	y1= (int)(0.257*r1 + 0.504*g + 0.098*b1 +16.5);

	yt=y0+y1;

	u=(int)(((b+b1)-yt)*0.5774+256.5);
	v=(int)(((r+r1)-yt)*0.7294+256.5);


	 *w++ = (y0<<8) + (((short)u>>1)&0xff);
     *w   = (y1<<8) + (((short)v>>1)&0xff);
}
*/
#endif


void FromRGBToYuv::ProcessPixel(UCHAR *Output,const UCHAR *Input)
{

	rgb2yuv422((const unsigned char*)Input, Output);
}


FromYuvToRGB::FromYuvToRGB()
{
	OutputBytesPerPixel=3;
	InputBytesPerPixel=2;
	PixelsPerProcess=2;
	InputBitsPerPixel=16;
	OutputBitsPerPixel=24;
	invert=TRUE;
}

inline UCHAR Clip(int value)
{
	return value> 255 ?  255 : (value < 0 ? 0 :value);
}
#if 0
inline void FromYuvToRGB::OutputPixel(UCHAR *Output,const UCHAR *Input)
{
	int Cr,Cb,y1,y2;

	Cb=*Input++;
	y1=*Input++;
	Cr=*Input++;
	y2=*Input++;

	int tr,tg,tb;
	tr=CrToR[Cr];
	tg=CrToG[Cr] + CbToG[Cb];
	tb=CbToB[Cb];
#if 1
	*Output++=clip[y1+tb];// blue 1
	*Output++=clip[y1+tg];// green 1
	*Output++=clip[y1+tr];// red 1
	*Output++=clip[y2+tb];// blue 2
	*Output++=clip[y2+tg];// green 2
	*Output++=clip[y2+tr];// red 2
#else
	*Output++=Clip(y1+tb);// blue 1
	*Output++=Clip(y1+tg);// green 1
	*Output++=Clip(y1+tr);// red 1
	*Output++=Clip(y2+tb);// blue 2
	*Output++=Clip(y2+tg);// green 2
	*Output++=Clip(y2+tr);// red 2
#endif
}
#endif

void FromYuvToRGB::ProcessLine(UCHAR *Output,const UCHAR *Input)
{
	int docols=hdisplay>>1; // /PixelsPerProcess;
	int Cr,Cb,y1,y2;
	int tr,tg,tb;
//	int isize=PixelsPerProcess*InputBytesPerPixel;
//	int osize=PixelsPerProcess*OutputBytesPerPixel;
	for (int col=0;col<docols;col++)
	{

#if 1	 // use pre built tables
		y1=*Input++;
		Cb=*Input++;
		y2=*Input++;
		Cr=*Input++;
		tr=CrToR[Cr];
		tg=CrToG[Cr] + CbToG[Cb];
		tb=CbToB[Cb];
#else		// math is 20% slower on pentium
		y1=*Input++;
		Cb=*Input++ -128;
		y2=*Input++;
		Cr=*Input++ -128;
		tr=1398*Cr>>10;
		tg=(-717*Cr - 342*Cb) >> 10;
		tb=1773*Cb>>10;
#endif
	#if 0 // function seems ~5% faster
		*Output++=clip[y1+tb];// blue 1
		*Output++=clip[y1+tg];// green 1
		*Output++=clip[y1+tr];// red 1
		*Output++=clip[y2+tb];// blue 2
		*Output++=clip[y2+tg];// green 2
		*Output++=clip[y2+tr];// red 2
	#else
		*Output++=Clip(y1+tb);// blue 1
		*Output++=Clip(y1+tg);// green 1
		*Output++=Clip(y1+tr);// red 1
		*Output++=Clip(y2+tb);// blue 2
		*Output++=Clip(y2+tg);// green 2
		*Output++=Clip(y2+tr);// red 2
	#endif
	}
}


#if 0
void FromYuvToRGB::ProcessFrame(UCHAR *Output,const UCHAR *Input)
{
	int row,col;
	int isize=PixelsPerProcess*InputBytesPerPixel;
	int osize=PixelsPerProcess*OutputBytesPerPixel;
	int docols=hdisplay/PixelsPerProcess;
	if (!invert)
	{
		for (row=0;row<vdisplay;row++)
		{
			for (col=0;col<docols;col++)
			{
				OutputPixel(Output,Input);
				Output+=osize;
				Input+=isize;
			}
		}
	}
	else
	{
		int OutputDelta=hdisplay*OutputBytesPerPixel;
		Output+=OutputDelta*(vdisplay-1);
		OutputDelta<<=1;  // must go up two rows
		for (row=0;row<vdisplay;row++)
		{
			for (col=0;col<docols;col++)
			{
				OutputPixel(Output,Input);
				Output+=osize;
				Input+=isize;
			}
			Output-=OutputDelta;
		}
	}

}
#endif

FromYuvToRGB16::FromYuvToRGB16()
{
	OutputBytesPerPixel=2;
	InputBytesPerPixel=2;
	PixelsPerProcess=2;
	InputBitsPerPixel=16;
	OutputBitsPerPixel=16;
	invert=TRUE;
	Table=(WORD*)BuildYUVToRGB555();
	for (int i=0;i<256;i++)
	{
		YDitherTable[i]=min(i+4,235) & 0xfc;
	}
}

FromYuvToRGB16::~FromYuvToRGB16()
{
	delete Table;
}
/*
void FromYuvToRGB16::ProcessFrame(UCHAR *Output,const UCHAR *Input)
{
	CopyYUVToRGB555((PWORD)Table,xres,yres,Output,(UCHAR *)Input,TRUE);
}
*/

void FromYuvToRGB16::ProcessLine(UCHAR *pOutput,const UCHAR *pInput)
{
	int docols=hdisplay>>1; // /PixelsPerProcess;
	unsigned short uv55 ,*pOut=(WORD*)pOutput;
	if (DitherCount++ & 1)
	{
		for (int col=0;col<docols;col++)
		{
			uv55=(pInput[1] & 0xf8) <<2 | (pInput[3] & 0xf8) >> 3;
			*pOut++ = Table[ ((pInput[0]  & 0xfc) << 8) | uv55 ] ;
			*pOut++ = Table[ (YDitherTable[pInput[2]] << 8) | uv55 ] ;
			pInput += sizeof(DWORD);
		}
	}
	else
	{
		for (int col=0;col<docols;col++)
		{
			uv55=(pInput[1] & 0xf8) <<2 | (pInput[3] & 0xf8) >> 3;
			*pOut++ = Table[ (YDitherTable[pInput[0]] << 8) | uv55 ] ;
			*pOut++ = Table[ (((pInput[2] ) & 0xfc) << 8) | uv55 ] ;
			pInput += sizeof(DWORD);
		}
	}
}


void FrameConverter::ProcessLine(UCHAR *pOutput,const UCHAR *pInput)
{
	int docols=hdisplay/PixelsPerProcess;
	int isize=PixelsPerProcess*InputBytesPerPixel;
	int osize=PixelsPerProcess*OutputBytesPerPixel;
	for (int col=0;col<docols;col++)
	{
		ProcessPixel(pOutput,pInput);
		pOutput+=osize;
		pInput+=isize;
	}
}

void FromRGB32ToYuv::ProcessLine(UCHAR *pOutput,const UCHAR *pInput)
{
	int docols=hdisplay/PixelsPerProcess;
	int isize=PixelsPerProcess*InputBytesPerPixel;
	int osize=PixelsPerProcess*OutputBytesPerPixel;
	for (int col=0;col<docols;col++)
	{
		rgb2yuv422((const unsigned char*)pInput, pOutput);
		pOutput+=osize;
		pInput+=isize;
	}
}



/* this function not finnished !!!!!!!!!!!!!!!! 
void FromRGB16ToYuv::ProcessLine(UCHAR *pOutput,const UCHAR *pInput)
{
	int docols=hdisplay/PixelsPerProcess;
	int isize=PixelsPerProcess*InputBytesPerPixel;
	int osize=PixelsPerProcess*OutputBytesPerPixel;
	for (int col=0;col<docols;col++)
	{
   struct {UCHAR r,g,b;} p[2];
	unsigned short *in=(unsigned short *)Input; 
	for (int i=0;i<PixelsPerProcess;i++)
	{
		p[i].r=(*in &0x7c00) >> 7;
		p[i].g=(*in & 0x3e0) >> 2;
		p[i].b=(*in++ & 0x1f) << 3;
	}
		rgb2yuv422((const unsigned char*)pInput, (unsigned short*)pOutput);
		pOutput+=osize;
		pInput+=isize;
	}
}
*/
