// YUV frame conversion routines

class FrameConverter
{
public:
	virtual int Init(int nhres,int nvres);
	virtual void ProcessFrame(UCHAR *Output,const UCHAR *Input);
	virtual ~FrameConverter();
	FrameConverter();

//	virtual Begin()=0;
	virtual void ProcessPixel(UCHAR *Output,const UCHAR *Input);
	DWORD GetOutputFrameSize();
	DWORD GetInputFrameSize();
	int GetInputBitsPerPixel() {return InputBitsPerPixel;}  
	int GetOutputBitsPerPixel() {return OutputBitsPerPixel;}  
	FOURCC GetCompression() {return Compression;};
	int xres;  // image width
	int yres;	// image height
protected:
	virtual void ProcessLine(UCHAR *pOutput,const UCHAR *pInput);
	int hdisplay;  // display width
	int vdisplay; // display width
	int InputBytesPerPixel;    
	int OutputBytesPerPixel;    
	int PixelsPerProcess;
	int InputBitsPerPixel;
	int OutputBitsPerPixel;
	int Compression;
	int Direction; // 1=from dib 2=to dib
	BOOL invert;

	static unsigned char ClipTable[1024];	//table to clip to 8 bits
	static unsigned char *clip;				//0 point of table;
	static short CrToR[256];				//look up Cr to red value (YUV to RGB conversion)
	static short CrToG[256];				//look up Cr to green value (YUV to RGB conversion)
	static short CbToB[256];				//look up Cb to red blue (YUV to RGB conversion)
	static short CbToG[256];				//look up Cb to red green (YUV to RGB conversion)
	static int ClipInited;
};
/*
class GenericConverter: public FrameConverter
{
public:
	GenericConverter();
	virtual void ProcessFrame(UCHAR *Output,const UCHAR *Input);
protected:
	virtual void ProcessPixel(UCHAR *Output,const UCHAR *Input);
	int srcPixelSize;
	int destPixelSize;
};
*/




class FromRGBToYuv: public FrameConverter
{
public:
	FromRGBToYuv();
//	virtual Begin()=0;
//	virtual ProcessFrame(UCHAR *Output,const UCHAR *Input);
protected:
	virtual void ProcessPixel(UCHAR *Output,const UCHAR *Input);
//	virtual ~FromRGBToYuv();
	inline void  rgb2yuv422(const unsigned char *lppic , unsigned char *w );
};

class FromRGB16ToYuv: public FromRGBToYuv 
{
public:
	FromRGB16ToYuv();
//	virtual void ProcessPixel(UCHAR *Output,const UCHAR *Input);
//	virtual Begin()=0;
	//virtual ProcessFrame(UCHAR *Output,const UCHAR *Input);
//	virtual ~FromRGB32ToYuv();
protected:
	virtual void ProcessPixel(UCHAR *Output,const UCHAR *Input);
//	virtual void ProcessLine(UCHAR *pOutput,const UCHAR *pInput);
private:
};

class FromRGB32ToYuv: public FromRGBToYuv 
{
public:
	FromRGB32ToYuv();
//	virtual void ProcessPixel(UCHAR *Output,const UCHAR *Input);
//	virtual Begin()=0;
	//virtual ProcessFrame(UCHAR *Output,const UCHAR *Input);
//	virtual ~FromRGB32ToYuv();
protected:
	virtual void ProcessLine(UCHAR *pOutput,const UCHAR *pInput);
};

class FromYuvToRGB: public FrameConverter
{
public:
	FromYuvToRGB();
//	virtual void ProcessFrame(UCHAR *Output,const UCHAR *Input);
private:
//	inline void OutputPixel(UCHAR *Output,const UCHAR *Input);
protected:
	virtual void ProcessLine(UCHAR *pOutput,const UCHAR *pInput);
};

class FromYuvToRGB16: public FrameConverter
{
public:
	FromYuvToRGB16();
	~FromYuvToRGB16();
//	virtual void ProcessFrame(UCHAR *Output,const UCHAR *Input);
protected:
	WORD *Table;
	UCHAR YDitherTable[256];
	int DitherCount;
	virtual void ProcessLine(UCHAR *pOutput,const UCHAR *pInput);
};
