/*
 * hardware.c
 *
 *
 * 32-bit Video Tool driver
 * hardware table entry routines 
 * for Data Translation Video Tool.
 *
 * Mark Vrabel, October 95
 */

#include <vcdriver.h>		   
#include <bwuser.h>
#include "hardware.h"
#include <mmtimers.h>
#include <vt_vrp.h>

#include <math.h>
#include <io.h>
#include <conio.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>


#define IFRAME_START_PAD	(3)

#define FRAME_SIZE_MASK		(0xfffffff0)
#define ONE_MEGABYTE		(1024*1024)

extern LPHWINFO glpHwTest;

                                                                
/* define local functions */
int HW_GetAudioDelay (BW_CAPTURE_PREFERENCES FAR * lpPreferences);
BOOL HW_ConfigCapture(VCUSER_HANDLE vh, LPCONFIG_INFO lpGeneric, int nFirstPage);

/* capture half a second of audio */
#define BUFFER_SIZE  (Format.nAvgBytesPerSec)


/* 
 * start the capture hardware
 */
BOOL
HW_Start (LPHWINFO lpHw, BOOL bLoad, BOOL bStart)
{
    /* initialize the video buffers */
    HW_InitializeBuffers (lpHw->vh);

	/* clear multiframe */
	lpHw->bMultiFrame = FALSE;

    /* 
     * load first Master1 DMA buffer (passthru/overlay). Even if we're not 
     * doing passthrough, we should make sure Master1 is loaded with a valid
     * address.  It would be a bad thing if it accidentaly threw data into
     * an invalid address.
     */
    VT_LoadDma(lpHw->DevID
               ,MASTER1
               ,lpHw->PassBufs.Bufs[0].dwLength*2
               ,lpHw->PassBufs.Bufs[0].dwPhys
               );

    /* increment the dma buffer index */
    lpHw->PassBufs.dwDmaIndex = (lpHw->PassBufs.dwDmaIndex + 1) % lpHw->PassBufs.dwNumberBufs;
	
	/* load first Master2 DMA buffer (I-frame capture) */
    VT_LoadDma(lpHw->DevID
               ,MASTER2
               ,lpHw->CaptBufs.Bufs[0].dwLength
               ,lpHw->CaptBufs.Bufs[0].dwPhys
               );

    /* increment the dma buffer index */
    lpHw->CaptBufs.dwDmaIndex = (lpHw->CaptBufs.dwDmaIndex + 1) % lpHw->CaptBufs.dwNumberBufs;

    /* make sure all is clear */
    lpHw->lpRegs->Status = (DONE_FLAG_1+DONE_FLAG_2+FIFO_OVERFLOW);

	/* enable/disable passthrough */
	VT_EnablePassthrough(lpHw->DevID, lpHw->bOverlay);

    /* if requested, load microcode */
	if (bLoad) {
		VT_SetMode(lpHw->DevID, VT_IENCODE);
	}

	if (!lpHw->bOverlay) {
		lpHw->dwLast = (lpHw->dwCaptTail + (lpHw->dwMemorySize - 1)) % lpHw->dwMemorySize;
		lpHw->lpLinearAddr[lpHw->dwLast] = 0xffffffff;
	}

    /* start it up!!! */
	if (bStart) {						 
		DWORD val = lpHw->CaptureInfo.Blanklines;
		VT_Start(lpHw->DevID);
		VT_Write(lpHw->DevID, VR_DRAM, 0x39f, &val, 1);
    }

    /* if needed load second Master1 DMA buffer (passthru/overlay) */
	if (lpHw->PassBufs.dwNumberBufs > 1) {             
		VT_LoadDma(lpHw->DevID
                   ,MASTER1
                   ,lpHw->PassBufs.Bufs[1].dwLength
                   ,lpHw->PassBufs.Bufs[1].dwPhys
                   );
    
		/* increment the dma buffer index */
		lpHw->PassBufs.dwDmaIndex = (lpHw->PassBufs.dwDmaIndex + 1) % lpHw->PassBufs.dwNumberBufs;
	}

    /* if needed load second Master2 DMA buffer (I-frame capture) */
    if (lpHw->CaptBufs.dwNumberBufs > 1) {             
		VT_LoadDma(lpHw->DevID
                   ,MASTER2
                   ,lpHw->CaptBufs.Bufs[1].dwLength
                   ,lpHw->CaptBufs.Bufs[1].dwPhys
                   );
    
		/* increment the dma buffer index */
		lpHw->CaptBufs.dwDmaIndex = (lpHw->CaptBufs.dwDmaIndex + 1) % lpHw->CaptBufs.dwNumberBufs;
	}
	
	return TRUE;
}


BOOL
HW_Stop (LPHWINFO lpHw, BOOL bUnload, BOOL bStop)
{
	if (bStop) {

		if (VT_Stop(lpHw->DevID) != VT_NO_ERROR) {
			VT_SetMode(lpHw->DevID, VT_IDLE);
			VT_SetMode(lpHw->DevID, VT_IENCODE);
		}
	}

	if (bUnload) {
		VT_Reset(lpHw->DevID);
		VT_SetMode(lpHw->DevID, VT_IDLE);
	}

	/* disable passthrough */
	VT_EnablePassthrough(lpHw->DevID, FALSE);
	
	return TRUE;
}


/*
 * Make an attempt to determine the startup delay of the
 * system audio card at the currently requested settings.
 * This function returns a delay of 0 if it fails for any
 * reason or if the parent app of the driver is not 
 * Broadway.
 */
int HW_GetAudioDelay (BW_CAPTURE_PREFERENCES FAR * lpPreferences)
{
    HWAVEIN hWave;
    WAVEFORMATEX Format;
    UINT status;
    WAVEHDR waveHdr;
    
    DWORD dwEnd, dwStart, dwCalc;
    DWORD dwBytes = 0;
    MMTIME mmTime;
    int nDelay;
    
    WORD nChannels, wBitsPerSample;
    DWORD nSamplesPerSec;
    BOOL bNegative = FALSE;

    switch (lpPreferences->nAudioMode) {

        case AUDIO_MONO:
            nChannels = 1;
            break;

        case AUDIO_JOINTSTEREO:
        case AUDIO_DUALCHANNEL:
        case AUDIO_STEREO:
            nChannels = 2;
            break;
											  
        default:
            return 0;
    }

    wBitsPerSample = 16;
    nSamplesPerSec = lpPreferences->nAudioRate;


    /* set up the requested audio mode */
    Format.wFormatTag      = WAVE_FORMAT_PCM;        
    Format.nChannels       = nChannels;         
    Format.wBitsPerSample  = wBitsPerSample;    
    Format.nSamplesPerSec  = nSamplesPerSec;   
    Format.nBlockAlign     = (Format.nChannels * Format.wBitsPerSample) / 8;      
    Format.nAvgBytesPerSec = Format.nSamplesPerSec * Format.nBlockAlign;  
    Format.cbSize          = 0;            

    /* allocate the wave buffer */
    waveHdr.lpData          = (LPSTR)GlobalAllocPtr (GPTR, BUFFER_SIZE);
    waveHdr.dwBufferLength  = BUFFER_SIZE;
    waveHdr.dwBytesRecorded = 0;
    waveHdr.dwUser          = 0;
    waveHdr.dwFlags         = 0;
    waveHdr.dwLoops         = 0;
    waveHdr.lpNext          = 0;
    waveHdr.reserved        = 0;

    /* open the wave input device */
    status = waveInOpen(&hWave, 0, &Format, 0, 0, 0); 
    if (status != 0) {
        GlobalFreePtr(waveHdr.lpData);
        return 0;
    }

    /* prepare the header */
    status = waveInPrepareHeader (hWave, &waveHdr, sizeof(WAVEHDR));                      
    if (status != 0) {
        GlobalFreePtr(waveHdr.lpData);
        waveInClose (hWave);
        return 0;
    }

    /* add the buffer */
    status = waveInAddBuffer (hWave, &waveHdr, sizeof(WAVEHDR));
    if (status != 0) {
        waveInUnprepareHeader (hWave, &waveHdr, sizeof(WAVEHDR));                      
        GlobalFreePtr(waveHdr.lpData);
        waveInClose (hWave);
        return 0;
    }

    /* start it up!!! */
    dwStart = timeGetTime();
    status = waveInStart (hWave);
    if (status != 0) {
        waveInReset (hWave);
        waveInUnprepareHeader (hWave, &waveHdr, sizeof(WAVEHDR));                      
        GlobalFreePtr(waveHdr.lpData);
        waveInClose (hWave);
        return 0;
    }

    /* 
     * if we got something already, stop here. we'll end up 
     * with a negative delay (by the time stream start is 
     * called we've already got audio).
     */
    mmTime.wType = TIME_BYTES;
    status = waveInGetPosition (hWave, &mmTime, sizeof(MMTIME));
    if (mmTime.u.cb != 0) {
        dwEnd = timeGetTime();
        bNegative = TRUE;
    } else {
        
        /* wait for some audio - the delay wil be positive. when stream start
         * is called we'll have to wait for audio.
         */
        while (timeGetTime() < (dwStart+2000)) {
        
            mmTime.wType = TIME_BYTES;
            status = waveInGetPosition (hWave, &mmTime, sizeof(MMTIME));
            if (mmTime.u.cb != 0) {
                dwEnd = timeGetTime();
                dwBytes = mmTime.u.cb;
                break;
            }
        }
    }

	/* if we captured nothing - the whole thing is bogus!!! */
    status = waveInGetPosition (hWave, &mmTime, sizeof(MMTIME));
    if (mmTime.u.cb == 0) {
		return 0;	
	}

    /* reset the device */
    waveInReset (hWave);
    waveInUnprepareHeader (hWave, &waveHdr, sizeof(WAVEHDR));                      
    waveInClose (hWave);
    
    /* calculate how long it should have taken */
    dwCalc = MulDiv (dwBytes, 1000L, Format.nAvgBytesPerSec);
    
    /* calculate the delay */
    if ((dwEnd - dwStart) < dwCalc) {
        nDelay = 0;
    } else {
        nDelay = (dwEnd - dwStart) - dwCalc; 
    }

    GlobalFreePtr(waveHdr.lpData);

    return ((bNegative) ?nDelay * -1:nDelay);    
}


BOOL
HW_OptionsChanged(LPHWINFO lpHw, BW_CAPTUREOPTIONS *lpCapOpt, BW_RECODEOPTIONS *lpRecOpt) 
{
	BW_CAPTURE_PREFERENCES *lpCapPref = lpCapOpt->lpPreferences; 
	BW_RECODE_PREFERENCES *lpRecPref = lpRecOpt->lpPreferences; 

	BW_RECODEOPTIONS *lpLocalRecOpt = &lpHw->RecodeOptions;
	BW_CAPTURE_PREFERENCES *lpLocalCapPref = &lpHw->CapturePreferences; 
	BW_RECODE_PREFERENCES *lpLocalRecPref = &lpHw->RecodePreferences; 
						   
	BOOL bNotSame = FALSE;

	/* check if capture options have changed */
    bNotSame |= lpLocalCapPref->nAutoDetect	 != lpCapPref->nAutoDetect;                      
    bNotSame |= lpLocalCapPref->nNTSC        != lpCapPref->nNTSC;                            
    bNotSame |= lpLocalCapPref->nComposite   != lpCapPref->nComposite;                       
	bNotSame |= lpLocalCapPref->nAudioEnable != lpCapPref->nAudioEnable;                     
    bNotSame |= lpLocalCapPref->nAudioMode   != lpCapPref->nAudioMode;                       
    bNotSame |= lpLocalCapPref->nAudioRate   != lpCapPref->nAudioRate;                       
	bNotSame |= lpLocalCapPref->nHue         != lpCapPref->nHue;                             
    bNotSame |= lpLocalCapPref->nSaturation  != lpCapPref->nSaturation;                      
    bNotSame |= lpLocalCapPref->nIntensity   != lpCapPref->nIntensity;                       
    bNotSame |= lpLocalCapPref->nContrast    != lpCapPref->nContrast;                        
	bNotSame |= lpLocalCapPref->nFilterValue != lpCapPref->nFilterValue;                     
    bNotSame |= lpLocalCapPref->nBitRate     != lpCapPref->nBitRate;                         
	bNotSame |= lpLocalCapPref->nCaptureMode != lpCapPref->nCaptureMode;                     
	bNotSame |= lpLocalCapPref->nBlankLines  != lpCapPref->nBlankLines;                      
    
	bNotSame |= lstrcmp (lpLocalRecPref->EncodePlaybackTarget, lpRecPref->EncodePlaybackTarget);
                                                                                             
	/* check if recode options have changed */                                               
	bNotSame |= lpLocalRecPref->nEncodeVideoEnable    != lpRecPref->nEncodeVideoEnable;        
	bNotSame |= lpLocalRecPref->nEncodeVideoBitRate   != lpRecPref->nEncodeVideoBitRate;       
	bNotSame |= lpLocalRecPref->nEncodeSquarePixels   != lpRecPref->nEncodeSquarePixels;       
	bNotSame |= lpLocalRecPref->nEncodeSofteningLevel != lpRecPref->nEncodeSofteningLevel;     
	bNotSame |= lpLocalRecPref->nLargeImage           != lpRecPref->nLargeImage;               
	bNotSame |= lpLocalRecPref->nEncodeAudioEnable    != lpRecPref->nEncodeAudioEnable;        
	bNotSame |= lpLocalRecPref->nEncodeAudioMode      != lpRecPref->nEncodeAudioMode;          
	bNotSame |= lpLocalRecPref->nEncodeAudioBitRate   != lpRecPref->nEncodeAudioBitRate;       
	bNotSame |= lpLocalRecPref->nCompressionEstimate  != lpRecPref->nCompressionEstimate;      
                                                                                             
	bNotSame |= lpLocalRecOpt->nEncodeUseOptimumRate != lpRecOpt->nEncodeUseOptimumRate;        
	bNotSame |= lpLocalRecOpt->nEncodeStreamDataRate != lpRecOpt->nEncodeStreamDataRate;        
	bNotSame |= lpLocalRecOpt->nEncodePackSize       != lpRecOpt->nEncodePackSize;              
	bNotSame |= lpLocalRecOpt->nEncodePacketsPerPack != lpRecOpt->nEncodePacketsPerPack;        
	bNotSame |= lpLocalRecOpt->nStreamMixType        != lpRecOpt->nStreamMixType;               

	/* if not the same, store the new settings */
	if (bNotSame) {

		lpLocalCapPref->nAutoDetect	 = lpCapPref->nAutoDetect;                     
     	lpLocalCapPref->nNTSC        = lpCapPref->nNTSC;                           
     	lpLocalCapPref->nComposite   = lpCapPref->nComposite;                      
     	lpLocalCapPref->nAudioEnable = lpCapPref->nAudioEnable;                    
	 	lpLocalCapPref->nAudioMode   = lpCapPref->nAudioMode;                      
     	lpLocalCapPref->nAudioRate   = lpCapPref->nAudioRate;                      
     	lpLocalCapPref->nHue         = lpCapPref->nHue;                            
	 	lpLocalCapPref->nSaturation  = lpCapPref->nSaturation;                     
     	lpLocalCapPref->nIntensity   = lpCapPref->nIntensity;                      
     	lpLocalCapPref->nContrast    = lpCapPref->nContrast;                       
     	lpLocalCapPref->nFilterValue = lpCapPref->nFilterValue;                    
	 	lpLocalCapPref->nBitRate     = lpCapPref->nBitRate;                        
     	lpLocalCapPref->nCaptureMode = lpCapPref->nCaptureMode;                    
	 	lpLocalCapPref->nBlankLines  = lpCapPref->nBlankLines;                     
	 	                                                                           
		lstrcpy (lpLocalRecPref->EncodePlaybackTarget, lpRecPref->EncodePlaybackTarget);
	                                                                                         
        /* check if recode options have changed */                                           
	    lpLocalRecPref->nEncodeVideoEnable    = lpRecPref->nEncodeVideoEnable;                 
	 	lpLocalRecPref->nEncodeVideoBitRate   = lpRecPref->nEncodeVideoBitRate;      
	 	lpLocalRecPref->nEncodeSquarePixels   = lpRecPref->nEncodeSquarePixels;      
	 	lpLocalRecPref->nEncodeSofteningLevel = lpRecPref->nEncodeSofteningLevel;    
	 	lpLocalRecPref->nLargeImage           = lpRecPref->nLargeImage;              
	 	lpLocalRecPref->nEncodeAudioEnable    = lpRecPref->nEncodeAudioEnable;       
	 	lpLocalRecPref->nEncodeAudioMode      = lpRecPref->nEncodeAudioMode;         
	 	lpLocalRecPref->nEncodeAudioBitRate   = lpRecPref->nEncodeAudioBitRate;      
	 	lpLocalRecPref->nCompressionEstimate  = lpRecPref->nCompressionEstimate;     
	 	                                                                           
      	lpLocalRecOpt->nEncodeUseOptimumRate = lpRecOpt->nEncodeUseOptimumRate;       
	 	lpLocalRecOpt->nEncodeStreamDataRate = lpRecOpt->nEncodeStreamDataRate;       
	 	lpLocalRecOpt->nEncodePackSize       = lpRecOpt->nEncodePackSize;             
	 	lpLocalRecOpt->nEncodePacketsPerPack = lpRecOpt->nEncodePacketsPerPack;       
	 	lpLocalRecOpt->nStreamMixType        = lpRecOpt->nStreamMixType;              
	}	 

	return bNotSame;
}


/*
 * Configure the destination format - the size, bitdepth of the
 * destination dib, and set any necessary translation table to
 * translate from YUV to the specified format.
 */
BOOL
HW_ConfigCapture(VCUSER_HANDLE vh, LPCONFIG_INFO lpGeneric, int nFirstPage)
{
    LPHWINFO lpHw = (LPHWINFO)VC_GetHWInfo(vh);
    BW_CAPTURE_CONFIG *lpConfig = (BW_CAPTURE_CONFIG FAR*) lpGeneric;

	BW_CAPTUREOPTIONS CaptureOptions;			
	BW_RECODEOPTIONS RecodeOptions;				
	BW_CAPTURE_PREFERENCES CapturePreferences;  
	BW_RECODE_PREFERENCES RecodePreferences;	

	BW_CAPTUREOPTIONS *lpCapOpt = &CaptureOptions; 
	BW_RECODEOPTIONS *lpRecOpt = &RecodeOptions; 
	BW_CAPTURE_PREFERENCES *lpCapPref = &CapturePreferences; 
	BW_RECODE_PREFERENCES *lpRecPref = &RecodePreferences; 

	char szFileName[MAX_PATH];

	CAPTUREINFO CaptureInfo;
	BOOL bCapture, bOverlay;
    BW_BOARDINFO BoardInfo;
    UINT nComposite, nNTSC;
	int nHeight, nWidth;
	
    /* verify hardware info */
    if (IsBadHWINFO(lpHw)) {
        return FALSE;
    }

    /* verify structure size */
    if (lpConfig->ulStructSize < sizeof(BW_CAPTURE_CONFIG)) {
        return FALSE;
    }

	/* 
	 * internal hack - this is a vfw fake out to make 
	 * preview work in mpg QSIF.
	 */
	if (lpHw->bNoDialog) {

		lpConfig->nWidth = 352;
		lpConfig->nHeight = lpHw->VideoMode == NTSC ?240:288;
		lpConfig->nSizeImage = lpHw->dwMaxCalcFrameSize*sizeof(DWORD);

		lpHw->bNoDialog = FALSE;
		return TRUE;
	}
 
    /* can't change options while streaming */
    if (lpHw->bVideoIn) {
        return FALSE;
    }

	/* initize the options structures */
	lpCapOpt = &CaptureOptions; 
	lpRecOpt = &RecodeOptions; 

	lpCapPref = &CapturePreferences; 
	lpRecPref = &RecodePreferences; 
	
	lpCapOpt->ulStructSize		= sizeof(BW_CAPTUREOPTIONS);
	lpCapPref->ulStructSize		= sizeof(BW_CAPTURE_PREFERENCES);
	lpCapOpt->lpPreferences		= lpCapPref;

	lpRecOpt->ulStructSize		= sizeof(BW_RECODEOPTIONS);
	lpRecPref->ulStructSize		= sizeof(BW_RECODE_PREFERENCES);
	lpRecOpt->lpPreferences		= lpRecPref;

	lpCapOpt->lpProf            = lpConfig->lpProf;
    lpCapOpt->nMode			    = lpConfig->nMode;
    lpCapOpt->nIsBroadway	    = lpConfig->nIsBroadway | lpHw->hWndCap != NULL;
	lpCapOpt->nPrivateInterface = lpHw->hWndCap != NULL;
    lpCapOpt->hWndParent        = (HWND)lpConfig->nWndParent;

    lpRecOpt->nMode				= lpConfig->nMode;
    lpRecOpt->hWndParent		= (HWND)lpConfig->nWndParent;

    /* save current overlay and capture states */
    bCapture = lpHw->bCapture;
    bOverlay = lpHw->bOverlay;

    /* fill in the board info */
    BoardInfo.vh       = vh;
    BoardInfo.DevID    = lpHw->DevID;
    BoardInfo.bOverlay = bCapture;
    BoardInfo.bOverlay = bOverlay;

    /* we don't care about modify current settings */
    if (lpCapOpt->nMode == MODE_MODIFY_CURRENT) {
		return TRUE;
	}
     
    /* get/set the capture options */        
    if (!BW_GetCaptureOptions(&BoardInfo, lpCapOpt, lpRecOpt, nFirstPage)) {
        return FALSE;
    }

	/* if nothing changed, go no furthur */
	if (!HW_OptionsChanged(lpHw, lpCapOpt, lpRecOpt)) {	
		lpConfig->nWidth = 352;
		lpConfig->nHeight = lpHw->VideoMode == NTSC ?240:288;
		lpConfig->nSizeImage = lpHw->dwMaxCalcFrameSize*sizeof(DWORD);
		return TRUE;
	}

    /* get the audio delay */
    lpHw->nAudioDelay = HW_GetAudioDelay (&lpHw->CapturePreferences);

	/* get last used settings */
    nComposite = VC_ReadProfileUser (lpCapOpt->lpProf, "Settings", "LastComposte", 1);
    nNTSC = VC_ReadProfileUser (lpCapOpt->lpProf, "Settings", "LastNTSC", 1);

    /* 
     * auto detect is a very special case. PAL/NTSC can be detected by 
     * the hardware, but composite/svideo can't, so we have to do it 
     * in software!!!
     */
    if (lpCapPref->nAutoDetect) {
        /* set the video connector to the current Auto comp setting */
        lpHw->VideoConnector = nComposite ?COMPOSITE:SVIDEO;
        VT_SetupVideoMode(lpHw->DevID, AUTO_DETECT, lpHw->VideoConnector);

        /* wait awhile... */
        VC_Delay(500);
        
        /* if it fails, set it to the other connector */
        if (VT_GetVideoMode(lpHw->DevID, &lpHw->VideoMode, &lpHw->VideoConnector) != VT_NO_ERROR) {
           
            lpHw->VideoConnector = nComposite ?SVIDEO:COMPOSITE;
            VT_SetupVideoMode(lpHw->DevID, AUTO_DETECT, lpHw->VideoConnector);

            /* wait awhile... */
            VC_Delay(500);

            /* 
             * if it still fails, use the last used setting and hope 
             * for the best.
             */
            if (VT_GetVideoMode(lpHw->DevID, &lpHw->VideoMode, &lpHw->VideoConnector) != VT_NO_ERROR) {

                /* write it */
                lpHw->VideoMode = nNTSC ?NTSC:PAL;
                lpHw->VideoConnector = nComposite ?COMPOSITE:SVIDEO;
            } 
        }
    } else {
        /* no autodetect - just set it */
        lpHw->VideoMode = lpCapPref->nNTSC ?NTSC:PAL;
        lpHw->VideoConnector = lpCapPref->nComposite ?COMPOSITE:SVIDEO;
    }
    
	/* write the last used settings to the registry */
    nComposite = lpHw->VideoConnector == COMPOSITE ?1:0;
    nNTSC = lpHw->VideoMode == NTSC ?1:0;    

    VC_WriteProfileUser (lpCapOpt->lpProf, "Settings", "LastComposte", nComposite);
    VC_WriteProfileUser (lpCapOpt->lpProf, "Settings", "LastNTSC", nNTSC);

    /* fill in the capture settings */
    lpHw->CaptureInfo.FrameRate   = lpHw->VideoMode == NTSC ?2997:2500;
	lpHw->CaptureInfo.Hue         = (short)BW_ColorToHardware (lpCapPref->nHue);
    lpHw->CaptureInfo.Saturation  = (short)BW_ColorToHardware (lpCapPref->nSaturation);
    lpHw->CaptureInfo.Intensity   = (short)BW_ColorToHardware (lpCapPref->nIntensity);
    lpHw->CaptureInfo.Contrast    = (short)BW_ColorToHardware (lpCapPref->nContrast);
	lpHw->CaptureInfo.ShiftUp     = lpHw->VideoMode == NTSC ?-1:0;
	lpHw->CaptureInfo.FilterValue = (short)lpCapPref->nFilterValue;
	lpHw->CaptureInfo.Blanklines  = (short)lpCapPref->nBlankLines;
		
	/* capture mode */
	lpHw->CaptureMode = lpCapPref->nCaptureMode;

	/* Broadway??? */
	GetModuleFileName(NULL, szFileName, sizeof (szFileName));
	CharLowerBuff(szFileName, lstrlen(szFileName));
    if (strstr (szFileName, "broadway.exe") != NULL) {
		lpHw->IsBroadway = TRUE;
    } else {
        lpHw->IsBroadway = FALSE;
    }

	/* I-Frame or full MPEG??? */
	if (lpHw->CaptureMode == CAPMODE_RTMPEG) {

		lpHw->CaptureInfo.DataRate = lpRecPref->nEncodeVideoBitRate;
		lpHw->CaptureInfo.IBP = TRUE;

		/* we can have square pixels or qsif with realtime mpeg */
		if (lpRecPref->nLargeImage) {
			lpHw->CaptureInfo.FrameType = lpHw->VideoMode == NTSC ?FT_NTSC:FT_PAL;
				
			nWidth  = 352;
			nHeight = lpHw->VideoMode == NTSC ?240:288;

		} else {
			lpHw->CaptureInfo.FrameType = lpHw->VideoMode == NTSC ?FT_NTSC_QSIF:FT_PAL_QSIF;

			nWidth  = 176;
			nHeight = lpHw->VideoMode == NTSC ?112:144;
		}

		lpHw->dwAvgCalcFrameSize = 45000;

	} else {
		
		lpHw->CaptureInfo.DataRate = (lpCapPref->nBitRate * (8 * ONE_MEGABYTE)) / 60;
		lpHw->CaptureInfo.FrameType = lpHw->VideoMode == NTSC ?FT_NTSC:FT_PAL;
		lpHw->CaptureInfo.IBP = FALSE;

		nWidth  = 352;
		nHeight = lpHw->VideoMode == NTSC ?240:288;

	    lpHw->dwAvgCalcFrameSize = ((DWORD)(lpCapPref->nBitRate) * ONE_MEGABYTE) / (lpHw->VideoMode == NTSC ?1800:1500);
	}

    /* save these for acces by the broadway app */
    lpHw->AudioEnable = (UINT)lpCapPref->nAudioEnable;
    lpHw->AudioMode	  = (UINT)lpCapPref->nAudioMode;
    lpHw->AudioRate   = (UINT)lpCapPref->nAudioRate;
    lpHw->Quality     = (UINT)lpCapPref->nBitRate;

    lpHw->CaptureInfo.FrameWidth  = (short)nWidth;
    lpHw->CaptureInfo.FrameHeight = (short)nHeight;

    /* and add 30% for good measure */
    lpHw->dwMaxCalcFrameSize = (DWORD)((double)lpHw->dwAvgCalcFrameSize * 1.3);
    lpHw->dwMinCalcFrameSize = (DWORD)((double)lpHw->dwAvgCalcFrameSize * 0.7);

    /* 
     * convert min and max frame size to dwords and 
     * place on 16 dword boundries 
     */
    lpHw->dwMinCalcFrameSize = (lpHw->dwMinCalcFrameSize/sizeof(DWORD)) & FRAME_SIZE_MASK;
    lpHw->dwMaxCalcFrameSize = (lpHw->dwMaxCalcFrameSize/sizeof(DWORD)) & FRAME_SIZE_MASK;

    /* set the image size */
	lpConfig->nSizeImage = lpHw->dwMaxCalcFrameSize*sizeof(DWORD);
	VC_SetImageSize (vh, lpConfig->nSizeImage);

	/*
	 * vfw fake out to get preview to work in mpeg QSIF
	 */
	lpConfig->nWidth = 352;
	lpConfig->nHeight = lpHw->VideoMode == NTSC ?240:288;

    /* pause the event timer */
    VC_TimerDisable (lpHw->vh);

    /* 
     * disable hardware  
     */
	HW_Stop (lpHw, TRUE, FALSE);

	/* 
	 * mepeg capture reqires funky setup for preview to
	 * avoid freezing while fast fowarding/rewinding VCR
	 */
	CopyMemory (&CaptureInfo, &lpHw->CaptureInfo, sizeof(CaptureInfo));
	if (lpHw->CaptureMode == CAPMODE_RTMPEG) {

		CaptureInfo.IBP = FALSE;
		
		if (!lpHw->bOverlay) {
			CaptureInfo.DataRate = (DWORD)((double)CaptureInfo.DataRate * 3.5);
		}
	}

    /* set the capture settings */
    VT_SetupVideoMode(lpHw->DevID, lpHw->VideoMode, lpHw->VideoConnector);
    VT_SetCaptureInfo(lpHw->DevID, &CaptureInfo);

    /* start hardware */
    HW_Start (lpHw, TRUE, bCapture && !bOverlay);

    /* resume the event timer */
    VC_TimerEnable (lpHw->vh);

    return(TRUE);
}


/*
 * Configure the destination format - the size, bitdepth of the
 * destination dib, and set any necessary translation table to
 * translate from YUV to the specified format.
 */
BOOL
HW_ConfigFormat(VCUSER_HANDLE vh, LPCONFIG_INFO lpGeneric)
{
    return HW_ConfigCapture(vh, lpGeneric, PAGE_FORMAT);
}


/*
 * Configure the adjustable parameters of the overlay or
 * external monitor display.
 */
BOOL
HW_ConfigDisplay(VCUSER_HANDLE vh, LPCONFIG_INFO lpGeneric)
{
    return HW_ConfigCapture(vh, lpGeneric, PAGE_DISPLAY);
}


/*
 * configure the source parameters - this controls the acquisition
 * of the source image, and will thus affect both the overlay output
 * and the captured data.
 */
BOOL
HW_ConfigSource(VCUSER_HANDLE vh, LPCONFIG_INFO lpGeneric)
{
    return HW_ConfigCapture(vh, lpGeneric, PAGE_SOURCE);
}


/*
 * overlay keying:
 *
 * we use a key colour and a simple rectangle to place the overlay.
 * we don't support hardware overlay, but we do support user mode
 * software overlay emulation. 
 */
DWORD
HW_GetOverlayMode(VCUSER_HANDLE vh)
{
    DWORD dwMode = VCO_KEYCOLOUR|VCO_SIMPLE_RECT|VCO_SOFTWARE;
    
    return dwMode;
}


/*
 * set the key colour given RGB value
 *
 * we  don't support this at the hardware 
 * level
 */
BOOL 
HW_SetKeyRGB(VCUSER_HANDLE vh, LPRGBQUAD lpRGB)
{
    return FALSE;
}


/*
 * set the key colour given a palette index 
 *
 * we  don't support this at the hardware 
 * level
 */
BOOL 
HW_SetKeyPalIdx(VCUSER_HANDLE vh, ULONG palidx)
{
    return FALSE;
}


/*
 * return the current key colour.
 *
 * we don't support this at the hardware 
 * level
 */
ULONG   
HW_GetKeyColour(VCUSER_HANDLE vh)
{
    return 0;
}

		  
/*
 * set the overlay window 
 *
 * if "software" overlay is supported we must 
 * know the desination window for the overlay
 */
BOOL 
HW_SetOverlayWindow (VCUSER_HANDLE vh, HWND hWnd)
{
    LPHWINFO lpHw = (LPHWINFO)VC_GetHWInfo(vh);

    /* verify hardware info */
    if (IsBadHWINFO(lpHw)) {
        return FALSE;
    }

    return VO_SetOverlayParentWindow(lpHw->voh, hWnd);
}


/*
 * set the overlay rectangle 
 *
 * we  don't support this at the hardware 
 * level
 */
BOOL 
HW_SetOverlayRects(VCUSER_HANDLE vh, LPOVERLAY_RECTS lpOR)
{
    LPHWINFO lpHw = (LPHWINFO)VC_GetHWInfo(vh);

    /* verify hardware info */
    if (IsBadHWINFO(lpHw)) {
        return FALSE;
    }

    if (lpOR->ulCount > 1) {
        return FALSE;
    } else {
        DWORD Width  = lpOR->rcRects[0].right-lpOR->rcRects[0].left;
        DWORD Height = lpOR->rcRects[0].bottom-lpOR->rcRects[0].top;

        return VO_SetOverlayDestFormat(lpHw->voh, Width, Height);
    }
}    


/*
 * offset the start of the video from the start of the overlay
 * window 
 *
 * we  don't support this at the hardware 
 * level
 */
BOOL 
HW_SetOverlayOffset(VCUSER_HANDLE vh, LPRECT lprc)
{
    return TRUE;
}


/*
 * enable/disable overlay on the board. 
 */
BOOL 
HW_Overlay(VCUSER_HANDLE vh, BOOL bOverlay)
{
    LPHWINFO lpHw = (LPHWINFO)VC_GetHWInfo(vh);
    BOOL bOK = FALSE;

    /* verify hardware info */
    if (IsBadHWINFO(lpHw)) {
        return FALSE;
    }

    if (bOverlay == lpHw->bOverlay) {
        return TRUE;
    }

    /* do nothing if stream captureing or initialized for mpeg capture!!! */
    if (lpHw->bVideoIn || lpHw->StreamInit) {
        return TRUE;
    }

    /* pause the event timer */
    VC_TimerDisable (vh);

	/* stop before changing */
	HW_Stop (lpHw, FALSE, !lpHw->bOverlay);

    /* do what 'ya gotta do... */
    if (bOverlay) {

        DWORD VidMode, VidSource;
        DWORD Width, Height;

        /* get the current video mode */
        if (VT_GetVideoMode(lpHw->DevID, &VidMode, &VidSource) != VT_NO_ERROR) 
		{
            VidMode = NTSC;
        } 
        
        /* set the current format and start overlay */
        if (lpHw->RevisionID > 0) {
			Width = (VidMode == NTSC) ?356:356;
		} else {
			Width = 360;
		}

		Height = (VidMode == NTSC) ?240:288;

		if ((bOK = VO_SetOverlaySourceFormat(lpHw->voh, Width, Height, lpHw->CaptureInfo.Blanklines))) {
            bOK = VO_StartOverlay (lpHw->voh);
        }
    } else {
        /* stop overlay */
        bOK = VO_StopOverlay (lpHw->voh);
    }

    /* set the current state */
    if (bOK) {
		lpHw->bOverlay = bOverlay;

		if (lpHw->bOverlay) {
			lpHw->bPreview = FALSE;
		}
    }

	/* restart the hardware */
	HW_Start (lpHw, FALSE, lpHw->bCapture && !lpHw->bOverlay);

    /* Restart the event timer */
    VC_TimerEnable (vh);

    return bOK;
}


/*
 * enable/disable acquisition on the board. If true, freeze the board
 * by disabling capture.
 *
 */
BOOL 
HW_Capture(VCUSER_HANDLE vh, BOOL bCapture)
{
    LPHWINFO lpHw = (LPHWINFO)VC_GetHWInfo(vh);

    /* verify hardware info */
    if (IsBadHWINFO(lpHw)) {
        return FALSE;
    }

    /* do nothing if no change */
    if (bCapture == lpHw->bCapture) {
        return TRUE;
    }

    if (bCapture) {
        
		/* clear some flags */
        lpHw->dwLastFrameSize = 0;
        lpHw->bCaptureFrame = FALSE;
        lpHw->bVideoIn = FALSE;

		/* start it up!!! */
		HW_Start(lpHw, FALSE, !lpHw->bOverlay);

        /* mark capture as enabled!!! */
        lpHw->bCapture = TRUE;

        /* start the event timer */
        VC_StartEventTimer (vh, PERIOD_MIN);

    } else {

        /* make SURE capture is marked disabled */
        lpHw->bCapture = FALSE;
        lpHw->dwLastFrameSize = 0;
        lpHw->bCaptureFrame = FALSE;

        /* stop the event timer */
        VC_StopEventTimer (vh);

		/* stop capture!!! */
		HW_Stop(lpHw, lpHw->bNoRestart, lpHw->bOverlay || lpHw->bNoRestart ?FALSE:TRUE);
	}
    
    return TRUE;
}


/*
 * Draw a frame of data directly to the screen
 *
 * we  don't support this at the hardware 
 * level yet...
 */
BOOL 
HW_DrawFrame(VCUSER_HANDLE vh, LPDRAWBUFFER lpDraw)
{
    return FALSE;
}


/*
 * capture a single frame into the supplied buffer
 */
BOOL 
HW_Frame(VCUSER_HANDLE vh, LPVIDEOHDR lpVideoHdr)
{
    LPHWINFO lpHw = (LPHWINFO)VC_GetHWInfo(vh);
    ULONG ulLength, ulSize, ulTime = 0; 
    DWORD nMillisecs = 66, ret = 0;     

    /* verify hardware info */
    if (IsBadHWINFO(lpHw)) {
        return FALSE;
    }

    /* mark bytes used as 0 */
    lpVideoHdr->dwBytesUsed = 0;
    *((DWORD FAR*)lpVideoHdr->lpData) = 0xff000000;
	
	/* 
	 * in case this is a call just to get the hardwarwe
	 * info pointer - see capwnd,c
	 */
	glpHwTest = lpHw;

    /* make sure we're in the right state */
    if (!lpHw->bCapture) {
        return FALSE;
    }

	if ((lpHw->CaptureMode == CAPMODE_RTMPEG || lpHw->IsBroadway) && lpHw->bOverlay) {
		return FALSE;
	}

	if (lpHw->CaptureMode != CAPMODE_RTMPEG) {

	    /* set the frame capture flag */
		VC_TimerDisable (vh);

		lpHw->dwLastFrameSize = 0;
		lpHw->bCaptureFrame = TRUE;

		VC_TimerEnable (vh);

		/* wait for the next frame */
		ret = WaitForSingleObject (lpHw->hFrameEvent, nMillisecs);
		lpHw->bCaptureFrame = FALSE;

		/* if no source video is attached - there won't be a frame captured!!! */
		if (ret != WAIT_OBJECT_0) {
			return FALSE;
		}
	}

    /*
     * find the length of the data buffer
     */
    ulLength = lpVideoHdr->dwBufferLength;
        
    /* capture a frame */
    ulSize = HW_CaptureFrame(vh, lpVideoHdr->lpData, ulLength);

    if (ulSize > 0 && ulSize <= ulLength) {    
        lpVideoHdr->dwBytesUsed = ulSize;
        lpVideoHdr->dwTimeCaptured = timeGetTime();
        lpVideoHdr->dwFlags |= VHDR_DONE;
    } else {
        lpVideoHdr->dwBytesUsed = 0;
        lpVideoHdr->dwTimeCaptured = 0;
        return FALSE;
    }

    return TRUE;
}


/*
 * initialise and cue ready for streaming from the framebuffer into
 * memory. The argument gives the desired frame rate.
 */
BOOL 
HW_StreamInit(VCUSER_HANDLE vh, ULONG microsecperframe)
{
    LPHWINFO lpHw = (LPHWINFO)VC_GetHWInfo(vh);

    /* verify hardware info */
    if (IsBadHWINFO(lpHw)) {
        return FALSE;
    }

    /* remember the frame rate. Keep it in microsec per frame
     * to reduce the rounding errors that would be introduced if we
     * did everything in millisecs
     */
    lpHw->dwMicroPerFrame = microsecperframe;

    /* clear our error counters */   
    lpHw->TimerOverRun		= 0L;       
    lpHw->FifoOverFlow		= 0L;       
    lpHw->NoStartCode		= 0L;        
    lpHw->FrameAllZeros		= 0L;        
    lpHw->FrameTooBig		= 0L;  
    lpHw->ExtractFrame		= 0L;   
	lpHw->NoBuffers			= 0L;
	lpHw->CopyOverRun		= 0L;
    lpHw->Skipped			= 0L;      
    lpHw->Repeated			= 0L;      
	lpHw->MultiFrame		= 0L;
	lpHw->dwCaptureStopped  = 0L;

    /* clear out the frame counters */
    lpHw->dwNextFrameNr		= 0L;
    lpHw->dwNextFrameTime	= 0L;
    lpHw->dwVideoTime		= 0L;
    lpHw->dwInterruptCount	= 0L;
    
    /* notify overlay that we're about to start streaming */
    VO_NotifyStreamStart (lpHw->voh, TRUE);
    
	/* for real time capture we alway stream capture in preview mode */
  	if (lpHw->CaptureMode == CAPMODE_RTMPEG) {
		
		VC_TimerDisable (vh);

		lpHw->bPreviousOverlay = lpHw->bOverlay;
		lpHw->bOverlay = FALSE;
		
		HW_Stop (lpHw, TRUE, FALSE);
		VT_SetCaptureInfo(lpHw->DevID, &lpHw->CaptureInfo);
		HW_Start (lpHw, TRUE, FALSE);

		VO_StopOverlay (lpHw->voh);
		
	} else if (lpHw->bOverlay) {
		
		HW_Start(lpHw, FALSE, TRUE);
		Sleep(100);
	}

	lpHw->StreamInit = TRUE;
    return(TRUE);
}



/* 
 * clean up after streaming. 
 */
BOOL
HW_StreamFini(VCUSER_HANDLE vh)
{	
    LPHWINFO lpHw = (LPHWINFO)VC_GetHWInfo(vh);
    SYSTEMTIME time;
    double TotalTime, fps;

    /* verify hardware info */
    if (IsBadHWINFO(lpHw)) {
        return FALSE;
    }

    /* notify overlay that we're done streaming */
    VO_NotifyStreamStart (lpHw->voh, FALSE);
	lpHw->StreamInit = FALSE;

    /* de we need to log errors */
    if (lpHw->bLogFileEnable) {
        char buf[100];
        HFILE hFile;
        DWORD dwSize;
        UINT ret;
        int flags;
        HANDLE hFileS;

        /* no frame captured - exit */
        if (lpHw->dwInterruptCount == 0) {
            return TRUE;
        }

        /* don't let it get too big!!! */
        hFileS = CreateFile("c:\\bwlog.txt", 0, 0, 0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
        if (hFileS !=  INVALID_HANDLE_VALUE) {

            /* don't let it get too big!!! */
            if ((dwSize = GetFileSize(hFileS, NULL)) != 0xFFFFFFFF && dwSize > 0x10000) {
                flags =  _O_CREAT | _O_TRUNC | O_TEXT | _O_WRONLY,_S_IREAD | _S_IWRITE;
            } else {
                flags =  _O_CREAT | _O_APPEND | O_TEXT | _O_WRONLY,_S_IREAD | _S_IWRITE;
            }

            CloseHandle(hFileS);
        }

        /* open the log file */
        hFile = _open( "c:\\bwlog.txt", _O_CREAT | _O_APPEND | O_TEXT | _O_WRONLY,_S_IREAD | _S_IWRITE );  
        if (hFile == HFILE_ERROR) {
            return (TRUE);
        }

        /* let em know if passthru is enabled... */
	  	if (lpHw->CaptureMode == CAPMODE_RTMPEG) {
            ret = wsprintf (buf, "Real-Time IBP MPG ");
            _write(hFile, buf, ret);
		} else {
            ret = wsprintf (buf, "I-Frame only AVI ");
            _write(hFile, buf, ret);
		}

        /* let em know if passthru is enabled... */
        if (lpHw->bOverlay) {
            ret = wsprintf (buf, "Frame Summary with Passthru...\n");
            _write(hFile, buf, ret);
        } else {
            ret = wsprintf (buf, "Frame Summary without Passthru...\n");
            _write(hFile, buf, ret);
        }
        
        /* and when the file was captured */
        GetLocalTime(&time);
        ret = wsprintf (buf, "%02d-%02d-%02d %02d:%02d:%04d\n\n", time.wMonth, time.wDay, time.wYear, 
                                            time.wHour, time.wMinute, time.wSecond);
        _write(hFile, buf, ret);

        ret = wsprintf (buf, "   Timer Over Run  = %d\n", lpHw->TimerOverRun);
        _write(hFile, buf, ret);

        ret = wsprintf (buf, "   Copy Over Run   = %d\n", lpHw->CopyOverRun);
        _write(hFile, buf, ret);
		
        ret = wsprintf (buf, "   Fifo Over Flow  = %d\n", lpHw->FifoOverFlow);
        _write(hFile, buf, ret);

        ret = wsprintf (buf, "   No Start Code   = %d\n", lpHw->NoStartCode);
        _write(hFile, buf, ret);

        ret = wsprintf (buf, "   Frame All Zeros = %d\n", lpHw->FrameAllZeros);
        _write(hFile, buf, ret);

        ret = wsprintf (buf, "   Frame Too Big   = %d\n", lpHw->FrameTooBig);
        _write(hFile, buf, ret);

        ret = wsprintf (buf, "   No Frames Found = %d\n", lpHw->ExtractFrame);
        _write(hFile, buf, ret);

        ret = wsprintf (buf, "   Out Of Buffers  = %d\n", lpHw->NoBuffers);
        _write(hFile, buf, ret);

        ret = wsprintf (buf, "   Fifo Reset      = %d\n", lpHw->FifoReset);
        _write(hFile, buf, ret);

        ret = wsprintf (buf, "   Bad Vsync       = %d\n", lpHw->BadToggle);
        _write(hFile, buf, ret);

        ret = wsprintf (buf, "   No Source       = %d\n", lpHw->dwCaptureStopped);
        _write(hFile, buf, ret);

        ret = wsprintf (buf, "   Skipped Frames  = %d\n", lpHw->Skipped);
        _write(hFile, buf, ret);

        ret = wsprintf (buf, "   Repeated Frames = %d\n", lpHw->Repeated);
        _write(hFile, buf, ret);

        ret = wsprintf (buf, "   Multiple Frames = %d\n", lpHw->MultiFrame);
        _write(hFile, buf, ret);

		ret = wsprintf (buf, "   Total Frames Captured = %d\n", lpHw->dwInterruptCount);
        _write(hFile, buf, ret);

        ret = wsprintf (buf, "   Total Frames To Disk  = %d\n", lpHw->dwCurrentVideoFrame);
        _write(hFile, buf, ret);

        TotalTime =  (double)(lpHw->dwEndTime-lpHw->dwStartTime) / 1000.0;
        
        ret = sprintf (buf, "   CaptureTime = %.3f\n", TotalTime);
        _write(hFile, buf, ret);

        fps = (double)lpHw->dwInterruptCount/TotalTime;

        ret = sprintf (buf, "   Frames per Second = %.3f\n", fps);
        _write(hFile, buf, ret);

        ret = sprintf (buf,  "  Audio Cal Factor  = %.9f\n", lpHw->dfAudioFudgeFactor);
        _write(hFile, buf, ret);
        
        ret = sprintf (buf, "   Audio Delay = %ld ms\n\n", lpHw->nAudioDelay);
        _write(hFile, buf, ret);
        
        _close(hFile);
    }

    return (TRUE);
}

extern int dead;

/*
 * start streaming data into the users buffers.
 *
 * Reset the counters and flag that capture is in progress.
 * Then enable interrupts (per frame) and wait for the right number
 * of interrupts.
 */
BOOL
HW_StreamStart(VCUSER_HANDLE vh)
{
    LPHWINFO lpHw = (LPHWINFO)VC_GetHWInfo(vh);
    DWORD dwDelay;

    /* verify hardware info */
    if (IsBadHWINFO(lpHw)) {
        return FALSE;
    }

    /* we set the microsecs per vsync interrupt based on the
     * PAl/NTSC field rate, and we use this to update the video
     * clock every interrupt. When this reaches the next frame time,
     * we capture the frame.
     *
     */
    lpHw->dwNextFrameNr    = 0L;
    lpHw->dwNextFrameTime  = 0L;
    lpHw->dwVideoTime      = 0L;
    lpHw->dwInterruptCount = 0L;
	lpHw->dwTimePrevious   = 0L;
	lpHw->dwTimeOffset	   = 0L;
	lpHw->dwLostFrames	   = 0L;
	lpHw->FifoReset		   = 0L;
	lpHw->ResetFifoCount   = 0L;
	lpHw->BadToggle		   = 0L;
	lpHw->dwSkipFrame	   = 0L;
	lpHw->dwLastInterrupt  = 0L;
	lpHw->dwRepeatFrames   = 0L;
	lpHw->dwTimeRestart	   = 0L;
		
	lpHw->bSkipFrame	   = FALSE;
	lpHw->bRepeatFrame     = FALSE;
	lpHw->bGotIFrame	   = FALSE;
	lpHw->bStoppingNow     = FALSE;
	lpHw->bRestart		   = FALSE;

	dead = 0;

    lpHw->dwMicroPerInterrupt = (lpHw->VideoMode == PAL) 
            ? PAL_MICROSPERFRAME : NTSC_MICROSPERFRAME;

	/* realtime we want to stop now so we can start again in HW_Start */
	if (lpHw->CaptureMode == CAPMODE_RTMPEG) {

		/* disable timer during state change */
		VC_TimerDisable (vh);

		/* start capture */
		HW_Start(lpHw, FALSE, TRUE);
  		
	} else {

		/* compensate for the c-cube pipeline delay and audio delay */
		dwDelay = MulDiv(IFRAME_START_PAD, lpHw->dwMicroPerInterrupt, 1000L);
		if ((lpHw->nAudioDelay >= 0) || ((int)dwDelay > abs(lpHw->nAudioDelay))) {
			DWORD dwEndTime = timeGetTime() + dwDelay + lpHw->nAudioDelay;
			while (timeGetTime() < dwEndTime);
		}

		/* disable timer during state change */
		VC_TimerDisable (vh);
	}

	/* 
	 * if not passthru mode, mark the circular buffer for overrun
	 * detection.
	 */
	if (!lpHw->bOverlay) {
		lpHw->dwLast = (lpHw->dwCaptTail + (lpHw->dwMemorySize - 1)) % lpHw->dwMemorySize;
		lpHw->lpLinearAddr[lpHw->dwLast] = 0xffffffff;
	}

    /* notify timer proc we've started streaming */
    lpHw->bVideoIn = TRUE;

	/* initialze the pc timer */
	pcBegin();

	/* save the real start time */
	lpHw->dwStartTime = timeGetTime();

	/* reenable timer */
    VC_TimerEnable (vh);

    return(TRUE);
}
						

/*
 * stop streaming 
 *
 */
BOOL
HW_StreamStop(VCUSER_HANDLE vh)
{
    LPHWINFO lpHw = (LPHWINFO)VC_GetHWInfo(vh);
	
    /* verify hardware info */
    if (IsBadHWINFO(lpHw)) {
        return FALSE;
    }

    /* if we we're never running, just return */
	if (!lpHw->bCapture) {
		return TRUE;
	}
	if (!lpHw->StreamInit && !lpHw->bVideoIn) {
		return TRUE;
    }

    /* get our stop time */
	lpHw->dwEndTime = timeGetTime();

    /* notify timer proc we're done captureing */
    VC_TimerDisable (vh);
    lpHw->bVideoIn = FALSE;

	if (lpHw->CaptureMode == CAPMODE_RTMPEG) {

		/* all done mpeg capture */
		lpHw->StreamInit = FALSE;
	
		/* 
		 * in mpeg capture the bw app allways disconnects the
		 * driver after stream capture, so we make the stop
		 * quicker by not restarting the board.
		 */
		if (lpHw->IsBroadway) {
			lpHw->bNoRestart = TRUE;
			HW_Capture (vh, FALSE);
		} else {
			CAPTUREINFO CaptureInfo;

			/* reload the I-Frame only microcode */
			CopyMemory (&CaptureInfo, &lpHw->CaptureInfo, sizeof(CaptureInfo));
			CaptureInfo.IBP = FALSE;
			CaptureInfo.DataRate = (DWORD)((double)CaptureInfo.DataRate * 3.5);

			HW_Stop (lpHw, TRUE, FALSE);

			VT_SetCaptureInfo(lpHw->DevID, &CaptureInfo);
			lpHw->bOverlay = ~lpHw->bPreviousOverlay;

			HW_Start (lpHw, TRUE, !lpHw->bOverlay);

			if (lpHw->bOverlay) {
				VO_StartOverlay (lpHw->voh);
			}
		}
	} else {
		HW_Stop(lpHw, FALSE, lpHw->bOverlay ?TRUE:FALSE);
		VT_EnablePassthrough(lpHw->DevID, lpHw->bOverlay);
	}

	VC_TimerEnable (vh);

    return(TRUE);
}


/*
 * get the position - for this we return the number of millisecs
 * since capture began, according to the video vsync clock
 */
ULONG
HW_StreamGetPosition(VCUSER_HANDLE vh)
{
    LPHWINFO lpHw = (LPHWINFO)VC_GetHWInfo(vh);
    DWORD dwPosition;

    /* verify hardware info */
    if (IsBadHWINFO(lpHw)) {
        return 0;
    }

    dwPosition = lpHw->dwVideoTime;
    return dwPosition;
}

/*
 * Incremnt the local skip frame count
 */
VOID
HW_StreamReportSkip(VCUSER_HANDLE vh, DWORD dwCount)
{
    LPHWINFO lpHw = (LPHWINFO)VC_GetHWInfo(vh);

    /* verify hardware info */
    if (IsBadHWINFO(lpHw)) {
        return;
    }

	lpHw->NoBuffers += dwCount;	
}
